@@ -20,6 +20,9 @@ import java.applet.Applet;
 import java.io.*;
 import java.lang.reflect.*;
 import java.net.*;
+import javax.net.ssl.*;
+import java.security.cert.*;
+import java.security.KeyStore;
 
 
 /** This class consists of static utility methods. */
@@ -45,7 +48,7 @@ public class IJ {
 	private static ProgressBar progressBar;
 	private static TextPanel textPanel;
 	private static String osname, osarch;
-	private static boolean isMac, isWin, isJava2, isJava14, isJava15, isJava16, isJava17, isJava18, isLinux, is64Bit;
+	private static boolean isMac, isWin, isJava16, isJava17, isJava18, isLinux, is64Bit;
 	private static boolean controlDown, altDown, spaceDown, shiftDown;
 	private static boolean macroRunning;
 	private static Thread previousThread;
@@ -60,8 +63,10 @@ public class IJ {
 	private static Hashtable commandTable;
 	private static Vector eventListeners = new Vector();
 	private static String lastErrorMessage;
-	private static Properties properties;
-
+	private static Properties properties;	private static DecimalFormat[] df;
+	private static DecimalFormat[] sf;
+	private static DecimalFormatSymbols dfs;
+	private static boolean trustManagerCreated;
 			
 	static {
 		osname = System.getProperty("os.name");
@@ -70,13 +75,22 @@ public class IJ {
 		isLinux = osname.startsWith("Linux");
 		String version = System.getProperty("java.version").substring(0,3);
 		if (version.compareTo("2.9")<=0) {  // JVM on Sharp Zaurus PDA claims to be "3.1"!
-			isJava2 = version.compareTo("1.1")>0;
-			isJava14 = version.compareTo("1.3")>0;
-			isJava15 = version.compareTo("1.4")>0;
 			isJava16 = version.compareTo("1.5")>0;
 			isJava17 = version.compareTo("1.6")>0;
 			isJava18 = version.compareTo("1.7")>0;
 		}
+		dfs = new DecimalFormatSymbols(Locale.US);
+		df = new DecimalFormat[10];
+		df[0] = new DecimalFormat("0", dfs);
+		df[1] = new DecimalFormat("0.0", dfs);
+		df[2] = new DecimalFormat("0.00", dfs);
+		df[3] = new DecimalFormat("0.000", dfs);
+		df[4] = new DecimalFormat("0.0000", dfs);
+		df[5] = new DecimalFormat("0.00000", dfs);
+		df[6] = new DecimalFormat("0.000000", dfs);
+		df[7] = new DecimalFormat("0.0000000", dfs);
+		df[8] = new DecimalFormat("0.00000000", dfs);
+		df[9] = new DecimalFormat("0.000000000", dfs);
 	}
 			
 	static void init(ImageJ imagej, Applet theApplet) {
@@ -204,15 +218,18 @@ public class IJ {
 				new PlugInFilterRunner(thePlugIn, commandName, arg);
 		}
 		catch (ClassNotFoundException e) {
-			if (className.indexOf('_')!=-1 && !suppressPluginNotFoundError)
+			if (className.contains("_")  && !suppressPluginNotFoundError)
 				error("Plugin or class not found: \"" + className + "\"\n(" + e+")");
 		}
 		catch (NoClassDefFoundError e) {
 			int dotIndex = className.indexOf('.');
-			if (dotIndex>=0)
+			if (dotIndex>=0 && className.contains("_")) {
+				// rerun plugin after removing folder name
+				if (debugMode) IJ.log("runUserPlugIn: rerunning "+className);
 				return runUserPlugIn(commandName, className.substring(dotIndex+1), arg, createNewLoader);
-			if (className.indexOf('_')!=-1 && !suppressPluginNotFoundError)
-				error("Plugin or class not found: \"" + className + "\"\n(" + e+")");
+			}
+			if (className.contains("_") && !suppressPluginNotFoundError)
+				error("Run User Plugin", "Class not found while attempting to run \"" + className + "\"\n \n   " + e);
 		}
 		catch (InstantiationException e) {error("Unable to load plugin (ins)");}
 		catch (IllegalAccessException e) {error("Unable to load plugin, possibly \nbecause it is not public.");}
@@ -236,10 +253,14 @@ public class IJ {
 		error(s);
 	}
 	
-    /** Starts executing a menu command in a separete thread and returns immediately. */
+    /** Runs a menu command on a separete thread and returns immediately. */
 	public static void doCommand(String command) {
-		if (ij!=null)
-			ij.doCommand(command);
+		new Executer(command, null);
+	}
+	
+    /** Runs a menu command on a separete thread, using the specified image. */
+	public static void doCommand(ImagePlus imp, String command) {
+		new Executer(command, imp);
 	}
 	
     /** Runs an ImageJ command. Does not return until 
@@ -252,7 +273,10 @@ public class IJ {
 	
     /** Runs an ImageJ command, with options that are passed to the
 		GenericDialog and OpenDialog classes. Does not return until
-		the command has finished executing. */
+		the command has finished executing. To generate run() calls,
+		start the recorder (Plugins/Macro/Record) and run commands
+		from the ImageJ menu bar.
+	*/
 	public static void run(String command, String options) {
 		//IJ.log("run1: "+command+" "+Thread.currentThread().hashCode()+" "+options);
 		if (ij==null && Menus.getCommands()==null)
@@ -307,6 +331,7 @@ public class IJ {
 			commandTable.put("XY Coodinates... ", "XY Coordinates... ");
 			commandTable.put("Statistics...", "Statistics");
 			commandTable.put("Channels Tool... ", "Channels Tool...");
+			commandTable.put("Profile Plot Options...", "Plots...");
 		}
 		String command2 = (String)commandTable.get(command);
 		if (command2!=null)
@@ -315,8 +340,12 @@ public class IJ {
 			return command;
 	}
 
-	/** Runs an ImageJ command using the specified image and options. */
+	/** Runs an ImageJ command using the specified image and options.
+		To generate run() calls, start the recorder (Plugins/Macro/Record)
+		and run commands from the ImageJ menu bar.*/
 	public static void run(ImagePlus imp, String command, String options) {
+		if (ij==null && Menus.getCommands()==null)
+			init();
 		if (imp!=null) {
 			ImagePlus temp = WindowManager.getTempCurrentImage();
 			WindowManager.setTempCurrentImage(imp);
@@ -355,7 +384,8 @@ public class IJ {
 	
 	/**Displays a message in the ImageJ status bar.*/
 	public static void showStatus(String s) {
-		if (ij!=null) ij.showStatus(s);
+		if (ij!=null)
+			ij.showStatus(s);
 		ImagePlus imp = WindowManager.getCurrentImage();
 		ImageCanvas ic = imp!=null?imp.getCanvas():null;
 		if (ic!=null)
@@ -579,7 +609,7 @@ public class IJ {
 		Writes to the Java console if ImageJ is not present. */
 	public static void showMessage(String title, String msg) {
 		if (ij!=null) {
-			if (msg!=null && msg.startsWith("<html>")) {
+			if (msg!=null && (msg.startsWith("<html>")||msg.startsWith("<HTML>"))) {
 				HTMLDialog hd = new HTMLDialog(title, msg);
 				if (isMacro() && hd.escapePressed())
 					throw new RuntimeException(Macro.MACRO_CANCELED);
@@ -752,10 +782,6 @@ public class IJ {
 		return d2s(n, 2);
 	}
 	
-	private static DecimalFormat[] df;
-	private static DecimalFormat[] sf;
-	private static DecimalFormatSymbols dfs;
-
 	/** Converts a number to a rounded formatted string.
 		The 'decimalPlaces' argument specifies the number of
 		digits to the right of the decimal point (0-9). Uses
@@ -771,6 +797,8 @@ public class IJ {
 			decimalPlaces = -decimalPlaces;
 			if (decimalPlaces>9) decimalPlaces=9;
 			if (sf==null) {
+				if (dfs==null)
+					dfs = new DecimalFormatSymbols(Locale.US);
 				sf = new DecimalFormat[10];
 				sf[1] = new DecimalFormat("0.0E0",dfs);
 				sf[2] = new DecimalFormat("0.00E0",dfs);
@@ -786,20 +814,6 @@ public class IJ {
 		}
 		if (decimalPlaces<0) decimalPlaces = 0;
 		if (decimalPlaces>9) decimalPlaces = 9;
-		if (df==null) {
-			dfs = new DecimalFormatSymbols(Locale.US);
-			df = new DecimalFormat[10];
-			df[0] = new DecimalFormat("0", dfs);
-			df[1] = new DecimalFormat("0.0", dfs);
-			df[2] = new DecimalFormat("0.00", dfs);
-			df[3] = new DecimalFormat("0.000", dfs);
-			df[4] = new DecimalFormat("0.0000", dfs);
-			df[5] = new DecimalFormat("0.00000", dfs);
-			df[6] = new DecimalFormat("0.000000", dfs);
-			df[7] = new DecimalFormat("0.0000000", dfs);
-			df[8] = new DecimalFormat("0.00000000", dfs);
-			df[9] = new DecimalFormat("0.000000000", dfs);
-		}
 		return df[decimalPlaces].format(n);
 	}
 
@@ -870,9 +884,11 @@ public class IJ {
 				break;
 			case KeyEvent.VK_ALT:
 				altDown=true;
+				updateStatus();
 				break;
 			case KeyEvent.VK_SHIFT:
 				shiftDown=true;
+				updateStatus();
 				if (debugMode) beep();
 				break;
 			case KeyEvent.VK_SPACE: {
@@ -893,8 +909,8 @@ public class IJ {
 		switch (key) {
 			case KeyEvent.VK_CONTROL: controlDown=false; break;
 			case KeyEvent.VK_META: if (isMacintosh()) controlDown=false; break;
-			case KeyEvent.VK_ALT: altDown=false; break;
-			case KeyEvent.VK_SHIFT: shiftDown=false; if (debugMode) beep(); break;
+			case KeyEvent.VK_ALT: altDown=false; updateStatus(); break;
+			case KeyEvent.VK_SHIFT: shiftDown=false; updateStatus();  if (debugMode) beep(); break;
 			case KeyEvent.VK_SPACE:
 				spaceDown=false;
 				ImageWindow win = WindowManager.getCurrentWindow();
@@ -906,6 +922,17 @@ public class IJ {
 		}
 	}
 	
+	private static void updateStatus() {
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp!=null) {
+			ImageCanvas ic = imp.getCanvas();
+			if (ic!=null && imp.getCalibration().scaled()) {
+				Point p = ic.getCursorLoc();
+				imp.mouseMoved(p.x, p.y);
+			}
+		}
+	}
+	
 	public static void setInputEvent(InputEvent e) {
 		altDown = e.isAltDown();
 		shiftDown = e.isShiftDown();
@@ -928,17 +955,17 @@ public class IJ {
 	
 	/** Always returns true. */
 	public static boolean isJava2() {
-		return isJava2;
+		return true;
 	}
 	
-	/** Returns true if ImageJ is running on a Java 1.4 or greater JVM. */
+	/** Always returns true. */
 	public static boolean isJava14() {
-		return isJava14;
+		return true;
 	}
 
-	/** Returns true if ImageJ is running on a Java 1.5 or greater JVM. */
+	/** Always returns true. */
 	public static boolean isJava15() {
-		return isJava15;
+		return true;
 	}
 
 	/** Returns true if ImageJ is running on a Java 1.6 or greater JVM. */
@@ -1176,21 +1203,31 @@ public class IJ {
 	}
 
 	/** Sets the lower and upper threshold levels of the specified image and updates the display using
-		the specified <code>displayMode</code> ("Red", "Black & White", "Over/Under" or "No Update"). */
+		the specified <code>displayMode</code> ("Red", "Black & White", "Over/Under" or "No Update").
+		With calibrated images, 'lowerThreshold' and 'upperThreshold' must be density calibrated values.
+		Use setRawThreshold() to set the threshold using raw (uncalibrated) values. */
 	public static void setThreshold(ImagePlus img, double lowerThreshold, double upperThreshold, String displayMode) {
+		Calibration cal = img.getCalibration();
+		if (displayMode==null || !displayMode.contains("raw")) {
+			lowerThreshold = cal.getRawValue(lowerThreshold); 
+			upperThreshold = cal.getRawValue(upperThreshold);
+		}
+		setRawThreshold(img, lowerThreshold, upperThreshold, displayMode);
+	}
+
+	/** This is a version of setThreshold() that always uses raw (uncalibrated) values
+		 in the range 0-255 for 8-bit images and 0-65535 for 16-bit images. */
+	public static void setRawThreshold(ImagePlus img, double lowerThreshold, double upperThreshold, String displayMode) {
 		int mode = ImageProcessor.RED_LUT;
 		if (displayMode!=null) {
 			displayMode = displayMode.toLowerCase(Locale.US);
-			if (displayMode.indexOf("black")!=-1)
+			if (displayMode.contains("black"))
 				mode = ImageProcessor.BLACK_AND_WHITE_LUT;
-			else if (displayMode.indexOf("over")!=-1)
+			else if (displayMode.contains("over"))
 				mode = ImageProcessor.OVER_UNDER_LUT;
-			else if (displayMode.indexOf("no")!=-1)
+			else if (displayMode.contains("no"))
 				mode = ImageProcessor.NO_LUT_UPDATE;
 		}
-		Calibration cal = img.getCalibration();
-		lowerThreshold = cal.getRawValue(lowerThreshold); 
-		upperThreshold = cal.getRawValue(upperThreshold); 
 		img.getProcessor().setThreshold(lowerThreshold, upperThreshold, mode);
 		if (mode!=ImageProcessor.NO_LUT_UPDATE && img.getWindow()!=null) {
 			img.getProcessor().setLutAnimation(true);
@@ -1285,8 +1322,10 @@ public class IJ {
             WindowManager.setWindow(null);
 		} else {
 			ImageWindow win = imp.getWindow();
-			win.toFront();
-			WindowManager.setWindow(win);
+			if (win!=null) {
+				win.toFront();
+				WindowManager.setWindow(win);
+			}
 			long start = System.currentTimeMillis();
 			// timeout after 2 seconds unless current thread is event dispatch thread
 			String thread = Thread.currentThread().getName();
@@ -1296,7 +1335,7 @@ public class IJ {
 				imp = WindowManager.getCurrentImage();
 				if (imp!=null && imp.getID()==id)
 					return; // specified image is now active
-				if ((System.currentTimeMillis()-start)>timeout) {
+				if ((System.currentTimeMillis()-start)>timeout && win!=null) {
 					WindowManager.setCurrentWindow(win);
 					return;
 				}
@@ -1559,6 +1598,12 @@ public class IJ {
 		}
 	}
 	
+	/** Alias for getDirectory(). */
+	public static String getDir(String title) {
+		return getDirectory(title);
+	}
+
+	
 	/** Displays an open file dialog and returns the path to the
 		choosen file, or returns null if the dialog is canceled. */
 	public static String getFilePath(String dialogTitle) {
@@ -1580,7 +1625,8 @@ public class IJ {
 		With 1.41k or later, opens images specified by a URL.
 		*/
 	public static void open(String path) {
-		if (ij==null && Menus.getCommands()==null) init();
+		if (ij==null && Menus.getCommands()==null)
+			init();
 		Opener o = new Opener();
 		macroRunning = true;
 		if (path==null || path.equals(""))		
@@ -1592,17 +1638,20 @@ public class IJ {
 		
 	/** Opens and displays the nth image in the specified tiff stack. */
 	public static void open(String path, int n) {
-		if (ij==null && Menus.getCommands()==null) init();
+		if (ij==null && Menus.getCommands()==null)
+			init();
 		ImagePlus imp = openImage(path, n);
 		if (imp!=null) imp.show();
 	}
 
-	/** Opens the specified file as a tiff, bmp, dicom, fits, pgm, gif 
-		or jpeg image and returns an ImagePlus object if successful.
+	/** Opens the specified file as a tiff, bmp, dicom, fits, pgm, gif, jpeg 
+		or text image and returns an ImagePlus object if successful.
 		Calls HandleExtraFileTypes plugin if the file type is not recognised.
 		Displays a file open dialog if 'path' is null or an empty string.
 		Note that 'path' can also be a URL. Some reader plugins, including
-		the Bio-Formats plugin, display the image and return null. */
+		the Bio-Formats plugin, display the image and return null.
+		Use IJ.open() to display a file open dialog box.
+	*/
 	public static ImagePlus openImage(String path) {
 		return (new Opener()).openImage(path);
 	}
@@ -1612,6 +1661,11 @@ public class IJ {
 		return (new Opener()).openImage(path, n);
 	}
 
+	/** Opens the specified tiff file as a virtual stack. */
+	public static ImagePlus openVirtual(String path) {
+		return FileInfoVirtualStack.openVirtual(path);
+	}
+
 	/** Opens an image using a file open dialog and returns it as an ImagePlus object. */
 	public static ImagePlus openImage() {
 		return openImage(null);
@@ -1621,14 +1675,18 @@ public class IJ {
 		Returns "<Error: message>" if there an error, including
 		host or file not found. */
 	public static String openUrlAsString(String url) {
+		//if (!trustManagerCreated && url.contains("nih.gov")) trustAllCerts();
+		url = Opener.updateUrl(url);
+		if (debugMode) log("OpenUrlAsString: "+url);
 		StringBuffer sb = null;
 		url = url.replaceAll(" ", "%20");
 		try {
+			//if (url.contains("nih.gov")) addRootCA();
 			URL u = new URL(url);
 			URLConnection uc = u.openConnection();
 			long len = uc.getContentLength();
-			if (len>1048576L)
-				return "<Error: file is larger than 1MB>";
+			if (len>5242880L)
+				return "<Error: file is larger than 5MB>";
 			InputStream in = u.openStream();
 			BufferedReader br = new BufferedReader(new InputStreamReader(in));
 			sb = new StringBuffer() ;
@@ -1644,6 +1702,49 @@ public class IJ {
 		else
 			return "";
 	}
+	
+	/* 
+	public static void addRootCA() throws Exception {
+		String path = "/Users/wayne/Downloads/Certificates/lets-encrypt-x1-cross-signed.pem";
+		InputStream fis = new BufferedInputStream(new FileInputStream(path));
+		Certificate ca = CertificateFactory.getInstance("X.509").generateCertificate(fis);
+		KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
+		ks.load(null, null);
+		ks.setCertificateEntry(Integer.toString(1), ca);
+		TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
+		tmf.init(ks);
+		SSLContext ctx = SSLContext.getInstance("TLS");
+		ctx.init(null, tmf.getTrustManagers(), null); 
+		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());
+	}
+	*/
+	
+	/*
+	// Create a new trust manager that trust all certificates
+	// http://stackoverflow.com/questions/10135074/download-file-from-https-server-using-java
+	private static void trustAllCerts() {
+		trustManagerCreated = true;
+		TrustManager[] trustAllCerts = new TrustManager[] {
+			new X509TrustManager() {
+				public java.security.cert.X509Certificate[] getAcceptedIssuers() {
+					return null;
+				}
+				public void checkClientTrusted (java.security.cert.X509Certificate[] certs, String authType) {
+				}
+				public void checkServerTrusted (java.security.cert.X509Certificate[] certs, String authType) {
+				}
+			}
+		};
+		// Activate the new trust manager
+		try {
+			SSLContext sc = SSLContext.getInstance("SSL");
+			sc.init(null, trustAllCerts, new java.security.SecureRandom());
+			HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
+		} catch (Exception e) {
+			IJ.log(""+e);
+		}
+	}
+	*/
 
 	/** Saves the current image, lookup table, selection or text window to the specified file path. 
 		The path must end in ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", ".fits", ".pgm", ".png", ".lut", ".roi" or ".txt".  */
@@ -1652,16 +1753,23 @@ public class IJ {
 	}
 
 	/** Saves the specified image, lookup table or selection to the specified file path. 
-		The file path must end with ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", 
-		".fits", ".pgm", ".png", ".lut", ".roi" or ".txt". */
+		The file path should end with ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", 
+		".fits", ".pgm", ".png", ".lut", ".roi" or ".txt". The specified image is saved in 
+		TIFF format if there is no extension. */
 	public static void save(ImagePlus imp, String path) {
+		ImagePlus imp2 = imp;
+		if (imp2==null)
+			imp2 = WindowManager.getCurrentImage();
 		int dotLoc = path.lastIndexOf('.');
+		if (dotLoc==-1 && imp2!=null) {
+			path = path + ".tif"; // save as TIFF if file name does not have an extension
+			dotLoc = path.lastIndexOf('.');
+		}
 		if (dotLoc!=-1) {
-			ImagePlus imp2 = imp;
-			if (imp2==null) imp2 = WindowManager.getCurrentImage();
 			String title = imp2!=null?imp2.getTitle():null;
 			saveAs(imp, path.substring(dotLoc+1), path);
-			if (title!=null) imp2.setTitle(title);
+			if (title!=null)
+				imp2.setTitle(title);
 		} else
 			error("The file path passed to IJ.save() method or save()\nmacro function is missing the required extension.\n \n\""+path+"\"");
 	}
@@ -1679,8 +1787,10 @@ public class IJ {
 		"jpeg", "gif", "zip", "raw", "avi", "bmp", "fits", "pgm", "png", 
 		"text image", "lut", "selection" or "xy Coordinates". */
  	public static void saveAs(ImagePlus imp, String format, String path) {
-		if (format==null) return;
-		if (path!=null && path.length()==0) path = null;
+		if (format==null)
+			return;
+		if (path!=null && path.length()==0)
+			path = null;
 		format = format.toLowerCase(Locale.US);
 		if (format.indexOf("tif")!=-1) {
 			saveAsTiff(imp, path);
@@ -1695,7 +1805,7 @@ public class IJ {
 			path = updateExtension(path, ".txt");
 			format = "Text Image...";
 		} else if (format.indexOf("text")!=-1 || format.indexOf("txt")!=-1) {
-			if (path!=null && !path.endsWith(".xls"))
+			if (path!=null && !path.endsWith(".xls") && !path.endsWith(".csv"))
 				path = updateExtension(path, ".txt");
 			format = "Text...";
 		} else if (format.indexOf("zip")!=-1) {
