@@ -17,6 +17,8 @@ import java.io.*;
 import java.net.*;
 import java.awt.image.*;
 import javax.swing.ImageIcon;
+import javax.swing.JLabel;
+
 
 /**
 This frame is the main ImageJ class.
@@ -77,8 +79,8 @@ public class ImageJ extends Frame implements ActionListener,
 	MouseListener, KeyListener, WindowListener, ItemListener, Runnable {
 
 	/** Plugins should call IJ.getVersion() or IJ.getFullVersion() to get the version string. */
-	public static final String VERSION = "1.49i";
-	public static final String BUILD = ""; 
+	public static final String VERSION = "1.50i";
+	public static final String BUILD = "";
 	public static Color backgroundColor = new Color(237,237,237);
 	/** SansSerif, 12-point, plain font. */
 	public static final Font SansSerif12 = new Font("SansSerif", Font.PLAIN, 12);
@@ -104,12 +106,13 @@ public class ImageJ extends Frame implements ActionListener,
 	private Toolbar toolbar;
 	private Panel statusBar;
 	private ProgressBar progressBar;
-	private Label statusLine;
+	private JLabel statusLine;
 	private boolean firstTime = true;
 	private java.applet.Applet applet; // null if not running as an applet
 	private Vector classes = new Vector();
 	private boolean exitWhenQuitting;
 	private boolean quitting;
+	private boolean quitMacro;
 	private long keyPressedTime, actionPerformedTime;
 	private String lastKeyCommand;
 	private boolean embedded;
@@ -161,7 +164,7 @@ public class ImageJ extends Frame implements ActionListener,
 		statusBar.setLayout(new BorderLayout());
 		statusBar.setForeground(Color.black);
 		statusBar.setBackground(backgroundColor);
-		statusLine = new Label();
+		statusLine = new JLabel();
 		statusLine.setFont(new Font("SansSerif", Font.PLAIN, 13));
 		statusLine.addKeyListener(this);
 		statusLine.addMouseListener(this);
@@ -185,7 +188,7 @@ public class ImageJ extends Frame implements ActionListener,
 		if (mode!=NO_SHOW) {
 			if (IJ.isWindows()) try {setIcon();} catch(Exception e) {}
 			setLocation(loc.x, loc.y);
-			setResizable(!IJ.isMacOSX());
+			setResizable(false);
 			pack();
 			setVisible(true);
 		}
@@ -316,6 +319,11 @@ public class ImageJ extends Frame implements ActionListener,
 		if ((e.getSource() instanceof MenuItem)) {
 			MenuItem item = (MenuItem)e.getSource();
 			String cmd = e.getActionCommand();
+			Frame frame = WindowManager.getFrontWindow();
+			if (frame!=null && (frame instanceof Fitter)) {
+				((Fitter)frame).actionPerformed(e);
+				return;
+			}
 			commandName = cmd;
 			ImagePlus imp = null;
 			if (item.getParent()==Menus.getOpenRecentMenu()) {
@@ -347,7 +355,9 @@ public class ImageJ extends Frame implements ActionListener,
 		MenuItem item = (MenuItem)e.getSource();
 		MenuComponent parent = (MenuComponent)item.getParent();
 		String cmd = e.getItem().toString();
-		if ((Menu)parent==Menus.window)
+		if ("Autorun".equals(cmd)) // Examples>Autorun
+			Prefs.autoRunExamples = e.getStateChange()==1;
+		else if ((Menu)parent==Menus.window)
 			WindowManager.activateWindow(cmd, item);
 		else
 			doCommand(cmd);
@@ -467,6 +477,12 @@ public class ImageJ extends Frame implements ActionListener,
 					Roi roi = imp.getRoi();
 					if (IJ.shiftKeyDown()&&imp==Orthogonal_Views.getImage())
 						return;
+					if (IJ.isMacOSX() && IJ.isJava18()) {
+						RoiManager rm = RoiManager.getInstance();
+						boolean rmActive = rm!=null && rm==WindowManager.getActiveWindow();
+						if (rmActive && (keyCode==KeyEvent.VK_DOWN||keyCode==KeyEvent.VK_UP))
+						  rm.repaint();
+					}
 					boolean stackKey = imp.getStackSize()>1 && (roi==null||IJ.shiftKeyDown());
 					boolean zoomKey = roi==null || IJ.shiftKeyDown() || IJ.controlKeyDown();
 					if (stackKey && keyCode==KeyEvent.VK_RIGHT)
@@ -535,7 +551,7 @@ public class ImageJ extends Frame implements ActionListener,
 	
 	private boolean ignoreArrowKeys(ImagePlus imp) {
 		Frame frame = WindowManager.getFrontWindow();
-		String title = frame.getTitle();
+		String title = frame!=null?frame.getTitle():null;
 		if (title!=null && title.equals("ROI Manager"))
 			return true;
 		// Control Panel?
@@ -590,7 +606,11 @@ public class ImageJ extends Frame implements ActionListener,
 	public void windowActivated(WindowEvent e) {
 		if (IJ.isMacintosh() && !quitting) {
 			IJ.wait(10); // may be needed for Java 1.4 on OS X
-			setMenuBar(Menus.getMenuBar());
+			MenuBar mb = Menus.getMenuBar();
+			if (mb!=null && mb!=getMenuBar()) {
+				setMenuBar(mb);
+				Menus.setMenuBarCount++;
+			}
 		}
 	}
 	
@@ -609,6 +629,7 @@ public class ImageJ extends Frame implements ActionListener,
 
 	/** Called by ImageJ when the user selects Quit. */
 	public void quit() {
+		quitMacro = IJ.macroRunning();
 		Thread thread = new Thread(this, "Quit");
 		thread.setPriority(Thread.NORM_PRIORITY);
 		thread.start();
@@ -620,6 +641,11 @@ public class ImageJ extends Frame implements ActionListener,
 		return quitting;
 	}
 	
+	/** Returns true if ImageJ is quitting as a result of a run("Quit") macro call. */
+	public boolean quittingViaMacro() {
+		return quitting && quitMacro;
+	}
+
 	/** Called once when ImageJ quits. */
 	public void savePreferences(Properties prefs) {
 		Point loc = getLocation();
