@@ -10,14 +10,8 @@ import ij.measure.*;
 import ij.plugin.filter.Analyzer;
 import ij.util.*;
 import ij.macro.Interpreter;
-import ij.plugin.frame.ContrastAdjuster;
-import ij.plugin.frame.RoiManager;
-import ij.plugin.frame.Recorder;
-import ij.plugin.Converter;
-import ij.plugin.Duplicator;
-import ij.plugin.RectToolOptions;
-import ij.plugin.Colors;
-import ij.plugin.ContrastEnhancer;
+import ij.plugin.*;
+import ij.plugin.frame.*;
 
 
 /**
@@ -95,6 +89,9 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	private boolean hideOverlay;
 	private static int default16bitDisplayRange;
 	private boolean antialiasRendering = true;
+	private boolean ignoreGlobalCalibration;
+	public boolean setIJMenuBar = true;
+	public boolean typeSet;
 	
 
     /** Constructs an uninitialized ImagePlus. */
@@ -161,10 +158,8 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 		if (locked) {
 			IJ.beep();
 			IJ.showStatus("\"" + title + "\" is locked");
-			//if (IJ.macroRunning()) {
-			//	IJ.error("Image is locked");
-			//	Macro.abort();
-			//}
+			if (IJ.macroRunning())
+				IJ.wait(500);
 			return false;
         } else {
         	locked = true;
@@ -202,7 +197,6 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			double progress;
 			waitStart = System.currentTimeMillis();
 			while (!imageLoaded && !errorLoadingImage) {
-				//IJ.showStatus(imageUpdateY+" "+imageUpdateW);
 				IJ.wait(30);
 				if (imageUpdateW>1) {
 					progress = (double)imageUpdateY/imageUpdateW;
@@ -268,6 +262,42 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 		draw();
 	}
 	
+	/** Sets the display mode of composite color images, where 'mode' 
+		 should be IJ.COMPOSITE, IJ.COLOR or IJ.GRAYSCALE. */
+	public void setDisplayMode(int mode) {
+		if (this instanceof CompositeImage) {
+			((CompositeImage)this).setMode(mode);
+			updateAndDraw();
+		}
+	}
+
+	/** Returns the display mode (IJ.COMPOSITE, IJ.COLOR
+		or IJ.GRAYSCALE) if this is a composite color
+		image, or 0 if it not. */
+	public int getDisplayMode() {
+		if (this instanceof CompositeImage)
+			return ((CompositeImage)this).getMode();
+		else
+			return 0;
+	}
+	
+	/** Controls which channels in a composite color image are displayed, 
+		where 'channels' is a list of ones and zeros that specify the channels to
+		display. For example, "101" causes channels 1 and 3 to be displayed. */ 
+	public void setActiveChannels(String channels) {
+		if (!(this instanceof CompositeImage))
+			return;
+		boolean[] active = ((CompositeImage)this).getActiveChannels();
+		for (int i=0; i<active.length; i++) {
+			boolean b = false;
+			if (channels.length()>i && channels.charAt(i)=='1')
+				b = true;
+			active[i] = b;
+		}
+		updateAndDraw();
+		Channels.updateChannels();
+	}
+
 	/** Updates this image from the pixel data in its 
 		associated ImageProcessor, then displays it.
 		The CompositeImage class overrides this method 
@@ -277,14 +307,12 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	}
 
 	/** Returns a reference to the current ImageProcessor. The
-		CompositeImage class overrides this method so it returns
+		CompositeImage class overrides this method to return
 		the processor associated with the current channel. */
 	public ImageProcessor getChannelProcessor() {
 		return getProcessor();
 	}
 		
-	/* Returns an array containing the lookup tables used by this image,
-		one per channel, or an empty array if this is an RGB image. */
 	/**  Returns an array containing the lookup tables used by this image,
 	 * one per channel, or an empty array if this is an RGB image.
 	 * @see #getNChannels
@@ -560,10 +588,13 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	void setProcessor2(String title, ImageProcessor ip, ImageStack newStack) {
 		//IJ.log("setProcessor2: "+ip+" "+this.ip+" "+newStack);
 		if (title!=null) setTitle(title);
-		if (ip!=null & this.ip!=null && getWindow()!=null)
+		if (ip==null)
+			return;
+		if (this.ip!=null && getWindow()!=null)
 			notifyListeners(UPDATED);
 		this.ip = ip;
-		if (ij!=null) ip.setProgressBar(ij.getProgressBar());
+		if (ij!=null)
+			ip.setProgressBar(ij.getProgressBar());
         int stackSize = 1;
 		if (stack!=null) {
 			stackSize = stack.getSize();
@@ -635,7 +666,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			if (resetCurrentSlice) setSlice(currentSlice);
 			return;
 		}
-		boolean invalidDimensions = (isDisplayedHyperStack()||isComposite()) && !((StackWindow)win).validDimensions();
+		boolean invalidDimensions = (isDisplayedHyperStack()||isComposite()) && (win instanceof StackWindow) && !((StackWindow)win).validDimensions();
 		if (newStackSize>1 && !(win instanceof StackWindow)) {
 			if (isDisplayedHyperStack())
 				setOpenAsHyperStack(true);
@@ -890,7 +921,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			return;
     	if (win!=null) {
     		if (ij!=null)
-				Menus.updateWindowMenuItem(this.title, title);
+				Menus.updateWindowMenuItem(this, this.title, title);
 			String virtual = stack!=null && stack.isVirtual()?" (V)":"";
 			String global = getGlobalCalibration()!=null?" (G)":"";
 			String scale = "";
@@ -1053,8 +1084,11 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
     	return imageType;
     }
 
-    /** Returns the bit depth, 8, 16, 24 (RGB) or 32. RGB images actually use 32 bits per pixel. */
+    /** Returns the bit depth, 8, 16, 24 (RGB) or 32, or 0 if the bit depth 
+    	is unknown. RGB images actually use 32 bits per pixel. */
     public int getBitDepth() {
+    	if (imageType==GRAY8 && ip==null && img==null && !typeSet)
+    		return 0;
     	int bitDepth = 0;
     	switch (imageType) {
 	    	case GRAY8: case COLOR_256: bitDepth=8; break;
@@ -1074,17 +1108,18 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
     	}
 	}
 
-    protected void setType(int type) {
-    	if ((type<0) || (type>COLOR_RGB))
-    		return;
-    	int previousType = imageType;
-    	imageType = type;
+	protected void setType(int type) {
+		if ((type<0) || (type>COLOR_RGB))
+			return;
+		int previousType = imageType;
+		imageType = type;
+		typeSet = true;
 		if (imageType!=previousType) {
 			if (win!=null)
 				Menus.updateMenus();
 			getLocalCalibration().setImage(this);
 		}
-    }
+	}
 		
  	/** Returns the string value from the "Info" property string  
 	 * associated with 'key', or null if the key is not found. 
@@ -1346,8 +1381,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 		}
 	}
 
-	/** Returns the current stack index (one-based) or 1 if
-		this is a single image. */
+	/** Returns the current stack index (one-based) or 1 if this is a single image. */
 	public int getCurrentSlice() {
 		if (currentSlice<1) setCurrentSlice(1);
 		if (currentSlice>getStackSize())
@@ -1505,7 +1539,16 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 				stack.setPixels(ip.getPixels(),currentSlice);
 			ip = getProcessor();
 			setCurrentSlice(n);
-			Object pixels = stack.getPixels(currentSlice);
+			Object pixels = null;
+			Overlay overlay2 = null;
+			if (stack.isVirtual() && !(stack instanceof FileInfoVirtualStack)) {
+				ImageProcessor ip2 = stack.getProcessor(currentSlice);
+				overlay2 = ip2.getOverlay();
+				if (overlay2!=null || getOverlay()!=null)
+					setOverlay(overlay2);
+				pixels = ip2.getPixels();
+			} else
+				pixels = stack.getPixels(currentSlice);
 			if (ip!=null && pixels!=null) {
 				try {
 					ip.setPixels(pixels);
@@ -1513,6 +1556,11 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 				} catch(Exception e) {}
 			} else
 				ip = stack.getProcessor(n);
+			if (compositeImage && getCompositeMode()==IJ.COMPOSITE && ip!=null) {
+				int channel = getC();
+				if (channel>0 && channel<=getNChannels())
+					ip.setLut(((CompositeImage)this).getChannelLut(channel));
+			}
 			if (win!=null && win instanceof StackWindow)
 				((StackWindow)win).updateSliceSelector();
 			if ((Prefs.autoContrast||IJ.shiftKeyDown()) && nChannels==1 && imageType!=COLOR_RGB) {
@@ -1520,13 +1568,10 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 				ContrastAdjuster.update();
 				//IJ.showStatus(n+": min="+ip.getMin()+", max="+ip.getMax());
 			}
-			if (stack.isVirtual()) {
-				Overlay overlay2 = stack.getProcessor(n).getOverlay();
-				if (overlay2!=null)
-					setOverlay(overlay2);
-			}
 			if (imageType==COLOR_RGB)
 				ContrastAdjuster.update();
+			else if (imageType==GRAY16 || imageType==GRAY32)
+				ThresholdAdjuster.update();
 			if (!noUpdateMode)
 				updateAndRepaintWindow();
 			else
@@ -1552,7 +1597,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	public void setRoi(Roi newRoi) {
 		setRoi(newRoi, true);
 	}
-
+	
 	/** Assigns 'newRoi'  to this image and displays it if 'updateDisplay' is true. */
 	public void setRoi(Roi newRoi, boolean updateDisplay) {
 		if (newRoi==null)
@@ -1583,7 +1628,9 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 				ip.resetRoi();
 		}
 		roi.setImage(this);
-		if (updateDisplay) draw();
+		if (updateDisplay)
+			draw();
+		//roi.notifyListeners(RoiListener.CREATED);
 	}
 	
 	/** Creates a rectangular selection. */
@@ -1605,10 +1652,6 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			case Toolbar.RECTANGLE:
 				int cornerDiameter = Toolbar.getRoundRectArcSize();
 				roi = new Roi(sx, sy, this, cornerDiameter);
-				//if (cornerDiameter>0) {
-				//	roi.setStrokeColor(Toolbar.getForegroundColor());
-				//	roi.setStrokeWidth(RectToolOptions.getDefaultStrokeWidth());
-				//}
 				break;
 			case Toolbar.OVAL:
 				if (Toolbar.getOvalToolType()==Toolbar.ELLIPSE_ROI)
@@ -1642,7 +1685,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 						IJ.run(this, "Add Selection...", "");
 					Overlay overlay2 = getOverlay();
 					if (overlay2!=null)
-						overlay2.drawLabels(true);
+						overlay2.drawLabels(!Prefs.noPointLabels);
 					Prefs.pointAddToManager = false;
 				}
 				if (Prefs.pointAutoMeasure || (Prefs.pointAutoNextSlice&&!Prefs.pointAddToManager))
@@ -1652,8 +1695,12 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 					ImageCanvas ic = getCanvas();
 					if (ic!=null) {
 						RoiManager rm = RoiManager.getInstance();
-						if (rm!=null)
-							rm.runCommand("show all with labels");
+						if (rm!=null) {
+							if (Prefs.noPointLabels)
+								rm.runCommand("show all without labels");
+							else
+								rm.runCommand("show all with labels");
+						}
 					}
 				}
 				if (Prefs.pointAutoNextSlice && getStackSize()>1) {
@@ -1669,6 +1716,14 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	public void deleteRoi() {
 		if (roi!=null) {
 			saveRoi();
+			if (!(IJ.altKeyDown()||IJ.shiftKeyDown())) {
+				RoiManager rm = RoiManager.getInstance();
+				if (rm!=null)
+					rm.deselect(roi);
+			}
+			roi.notifyListeners(RoiListener.DELETED);
+			if (roi instanceof PointRoi)
+				((PointRoi)roi).resetCounters();
 			roi = null;
 			if (ip!=null)
 				ip.resetRoi();
@@ -1681,7 +1736,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 		deleteRoi();
 	}
 
-	public void saveRoi() {
+	public synchronized void saveRoi() {
 		if (roi!=null) {
 			roi.endPaste();
 			Rectangle r = roi.getBounds();
@@ -1704,6 +1759,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 				else if (r.width==width && r.height==height) // is it the same size as the image
 					roi.setLocation(0, 0);
 				draw();
+				roi.notifyListeners(RoiListener.CREATED);
 			}
 		}
 	}
@@ -2020,7 +2076,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	/** Returns this image's calibration. */
 	public Calibration getCalibration() {
 		//IJ.log("getCalibration: "+globalCalibration+" "+calibration);
-		if (globalCalibration!=null) {
+		if (globalCalibration!=null && !ignoreGlobalCalibration) {
 			Calibration gc = globalCalibration.copy();
 			gc.setImage(this);
 			return gc;
@@ -2033,7 +2089,6 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 
    /** Sets this image's calibration. */
     public void setCalibration(Calibration cal) {
-		//IJ.write("setCalibration: "+cal);
 		if (cal==null)
 			calibration = null;
 		else {
@@ -2044,7 +2099,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 
     /** Sets the system-wide calibration. */
     public void setGlobalCalibration(Calibration global) {
-		//IJ.log("setGlobalCalibration ("+getTitle()+"): "+global);
+		//IJ.log("setGlobalCalibration: "+calibration);
 		if (global==null)
 			globalCalibration = null;
 		else
@@ -2056,6 +2111,11 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			return globalCalibration;
     }
 
+    /** This is a version of getGlobalCalibration() that can be called from a static context. */
+    public static Calibration getStaticGlobalCalibration() {
+			return globalCalibration;
+    }
+
 	/** Returns this image's local calibration, ignoring 
 		the "Global" calibration flag. */
 	public Calibration getLocalCalibration() {
@@ -2063,6 +2123,10 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			calibration = new Calibration(this);
 		return calibration;
 	}
+	
+	public void setIgnoreGlobalCalibration(boolean ignoreGlobalCalibration) {
+		this.ignoreGlobalCalibration = ignoreGlobalCalibration;
+	}
 
     /** Displays the cursor coordinates and pixel value in the status bar.
     	Called by ImageCanvas when the mouse moves. Can be overridden by
@@ -2087,12 +2151,13 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	String getFFTLocation(int x, int y, Calibration cal) {
 		double center = width/2.0;
 		double r = Math.sqrt((x-center)*(x-center) + (y-center)*(y-center));
-		if (r<1.0) r = 1.0;
 		double theta = Math.atan2(y-center, x-center);
 		theta = theta*180.0/Math.PI;
-		if (theta<0) theta = 360.0+theta;
+		if (theta<0) theta=360.0+theta;
 		String s = "r=";
-		if (cal.scaled())
+		if (r<1.0)
+			return s+"Infinity/c (0)"; //origin ('DC offset'), no angle
+		else if (cal.scaled()) 
 			s += IJ.d2s((width/r)*cal.pixelWidth,2) + " " + cal.getUnit() + "/c (" + IJ.d2s(r,0) + ")";
 		else
 			s += IJ.d2s(width/r,2) + " p/c (" + IJ.d2s(r,0) + ")";
@@ -2104,9 +2169,8 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
     public String getLocationAsString(int x, int y) {
 		Calibration cal = getCalibration();
 		if (getProperty("FHT")!=null)
-			return getFFTLocation(x, height-y-1, cal);
-		//y = Analyzer.updateY(y, height);
-		if (!IJ.altKeyDown()) {
+			return getFFTLocation(x, height-y, cal);
+		if (!(IJ.altKeyDown()||IJ.shiftKeyDown())) {
 			String s = " x="+d2s(cal.getX(x)) + ", y=" + d2s(cal.getY(y,height));
 			if (getStackSize()>1) {
 				int z = isDisplayedHyperStack()?getSlice()-1:getCurrentSlice()-1;
@@ -2151,8 +2215,8 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
     			String s = (int)value==value?IJ.d2s(value,0)+".0":IJ.d2s(value,4,7);
     			return(", value=" + s);
 			case COLOR_RGB:
-				//String hex = Colors.colorToString(new Color(v[0],v[1],v[2]));
-    			return(", value=" + IJ.pad(v[0],3) + "," + IJ.pad(v[1],3) + "," + IJ.pad(v[2],3));
+				String hex = Colors.colorToString(new Color(v[0],v[1],v[2]));
+				return(", value=" + IJ.pad(v[0],3) + "," + IJ.pad(v[1],3) + "," + IJ.pad(v[2],3) + " ("+hex + ")");
     		default: return("");
 		}
     }
@@ -2305,7 +2369,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	public static void removeImageListener(ImageListener listener) {
 		listeners.removeElement(listener);
 	}
-	
+		
 	/** Returns 'true' if the image is locked. */
 	public boolean isLocked() {
 		return locked;
@@ -2366,15 +2430,20 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			ip.resetMinAndMax();
 	}
 	
+	/** Returns 'true' if this image is thresholded. */
+	public boolean isThreshold() {
+		return ip!=null && ip.getMinThreshold()!=ImageProcessor.NO_THRESHOLD;
+	}
+
     /** Set the default 16-bit display range, where 'bitDepth' must be 0 (auto-scaling), 
-    	8 (0-255), 10 (0-1023), 12 (0-4095, 15 (0-32767) or 16 (0-65535). */
+    	8 (0-255), 10 (0-1023), 12 (0-4095, 14 (0-16383), 15 (0-32767) or 16 (0-65535). */
     public static void setDefault16bitRange(int bitDepth) {
-    	if (!(bitDepth==8 || bitDepth==10 || bitDepth==12 || bitDepth==15 || bitDepth==16))
+    	if (!(bitDepth==8 || bitDepth==10 || bitDepth==12 || bitDepth==14 || bitDepth==15 || bitDepth==16))
     		bitDepth = 0;
     	default16bitDisplayRange = bitDepth;
     }
     
-    /** Returns the default 16-bit display range, 0 (auto-scaling), 8, 10, 12, 15 or 16. */
+    /** Returns the default 16-bit display range, 0 (auto-scaling), 8, 10, 12, 14, 15 or 16. */
     public static int getDefault16bitRange() {
     	return default16bitDisplayRange;
     }
@@ -2403,6 +2472,8 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 			}
 		}
 		Overlay overlay2 = getOverlay();
+		if (overlay2!=null && imp2.getRoi()!=null)
+			imp2.deleteRoi();
 		ic2.setOverlay(overlay2);
 		ImageCanvas ic = getCanvas();
 		if (ic!=null)
@@ -2510,6 +2581,17 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 		stack.setPixels(imp2.getProcessor().getPixels(), slice);
 	}
 
+	/** Assigns a LUT (lookup table) to this image.
+	 * @see ij.io.Opener#openLut
+	*/
+	public void setLut(LUT lut) {
+		ImageProcessor ip2 = getProcessor();
+		if (ip2!=null && lut!=null) {
+			ip2.setLut(lut);
+			setProcessor(ip2);
+		}
+	}
+
 	/** Installs a list of ROIs that will be drawn on this image as a non-destructive overlay.
 	 * @see ij.gui.Roi#setStrokeColor
 	 * @see ij.gui.Roi#setStrokeWidth
@@ -2571,7 +2653,7 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	public boolean getHideOverlay() {
 		return hideOverlay;
 	}
-	
+		
 	/** Enable/disable use of antialiasing by the flatten() method. */
 	public void setAntialiasRendering(boolean antialiasRendering) {
 		this.antialiasRendering = antialiasRendering;
@@ -2589,7 +2671,15 @@ public class ImagePlus implements ImageObserver, Measurements, Cloneable {
 	}
 
     public String toString() {
-    	return "imp["+getTitle()+" ("+width+"x"+height+"x"+getNChannels()+"x"+getNSlices()+"x"+getNFrames()+")]";
+    	return "img["+getTitle()+" ("+width+"x"+height+"x"+getNChannels()+"x"+getNSlices()+"x"+getNFrames()+")]";
+    }
+    
+    public void setIJMenuBar(boolean b) {
+    	setIJMenuBar = b;
+    }
+    
+    public boolean setIJMenuBar() {
+    	return setIJMenuBar;
     }
     
 }
