@@ -54,7 +54,7 @@ public class Prefs {
 
 	private static final int USE_SYSTEM_PROXIES=1<<0, USE_FILE_CHOOSER=1<<1,
 		SUBPIXEL_RESOLUTION=1<<2, ENHANCED_LINE_TOOL=1<<3, SKIP_RAW_DIALOG=1<<4,
-		REVERSE_NEXT_PREVIOUS_ORDER=1<<5;
+		REVERSE_NEXT_PREVIOUS_ORDER=1<<5, AUTO_RUN_EXAMPLES=1<<6, SHOW_ALL_POINTS=1<<7;
 	public static final String OPTIONS2 = "prefs.options2";
     
 	/** file.separator system property */
@@ -153,6 +153,11 @@ public class Prefs {
 	public static boolean skipRawDialog;
 	/** Reverse channel-slice-frame priority used by Next Slice and Previous Slice commands. */
 	public static boolean reverseNextPreviousOrder;
+	/** Automatically run examples in Help/Examples menu. */
+	public static boolean autoRunExamples = true;
+	/** Ignore stack positions when displaying points. */
+	public static boolean showAllPoints;
+	
 
 	static Properties ijPrefs = new Properties();
 	static Properties props = new Properties(ijPrefs);
@@ -395,7 +400,10 @@ public class Prefs {
 				if (!f.exists()) f.mkdir(); // create .imagej directory
 			}
 			if (resetPreferences) {
-				new File(path).delete();
+				File f = new File(path);
+				if (!f.exists())
+					IJ.error("Edit>Options>Reset", "Unable to reset preferences. File not found at\n"+path);
+				boolean rtn = f.delete();
 				resetPreferences = false;
 			} else
 				savePrefs(prefs, path);
@@ -416,7 +424,7 @@ public class Prefs {
 	}
 
 	static void loadOptions() {
-		int defaultOptions = ANTIALIASING+AVOID_RESLICE_INTERPOLATION+ANTIALIASED_TOOLS
+		int defaultOptions = ANTIALIASING+AVOID_RESLICE_INTERPOLATION+ANTIALIASED_TOOLS+MULTI_POINT_MODE
 			+(!IJ.isMacOSX()?RUN_SOCKET_LISTENER:0);
 		int options = getInt(OPTIONS, defaultOptions);
 		usePointerCursor = (options&USE_POINTER)!=0;
@@ -457,6 +465,8 @@ public class Prefs {
 		enhancedLineTool = (options2&ENHANCED_LINE_TOOL)!=0;
 		skipRawDialog = (options2&SKIP_RAW_DIALOG)!=0;
 		reverseNextPreviousOrder = (options2&REVERSE_NEXT_PREVIOUS_ORDER)!=0;
+		autoRunExamples = (options2&AUTO_RUN_EXAMPLES)!=0;
+		showAllPoints = (options2&SHOW_ALL_POINTS)!=0;
 	}
 
 	static void saveOptions(Properties prefs) {
@@ -481,7 +491,8 @@ public class Prefs {
 		int options2 = (useSystemProxies?USE_SYSTEM_PROXIES:0)
 			+ (useFileChooser?USE_FILE_CHOOSER:0) + (subPixelResolution?SUBPIXEL_RESOLUTION:0)
 			+ (enhancedLineTool?ENHANCED_LINE_TOOL:0) + (skipRawDialog?SKIP_RAW_DIALOG:0)
-			+ (reverseNextPreviousOrder?REVERSE_NEXT_PREVIOUS_ORDER:0);
+			+ (reverseNextPreviousOrder?REVERSE_NEXT_PREVIOUS_ORDER:0)
+			+ (autoRunExamples?AUTO_RUN_EXAMPLES:0) + (showAllPoints?SHOW_ALL_POINTS:0);
 		prefs.put(OPTIONS2, Integer.toString(options2));
 	}
 
@@ -634,5 +645,9 @@ public class Prefs {
 		return ijPrefs;
 	}
 	
+	public static String defaultResultsExtension() {
+		return get("options.ext", ".xls");
+	}
+		
 }
 
