@@ -154,7 +154,10 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		}
 		if (IJ.isWindows()) columns -= 2;
 		if (columns<1) columns = 1;
-		TextField tf = new TextField(IJ.d2s(defaultValue, digits), columns);
+		String defaultString = IJ.d2s(defaultValue, digits);
+		if (Double.isNaN(defaultValue))
+			defaultString = "";
+		TextField tf = new TextField(defaultString, columns);
 		if (IJ.isLinux()) tf.setBackground(Color.white);
 		tf.addActionListener(this);
 		tf.addTextListener(this);
@@ -663,7 +666,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 
     /** Adds a Panel to the dialog. */
     public void addPanel(Panel panel) {
-    	addPanel(panel , GridBagConstraints.WEST, new Insets(5, 0, 0, 0));
+    	addPanel(panel , GridBagConstraints.WEST, getInsets(5,0,0,0));
     }
 
     /** Adds a Panel to the dialog with custom contraint and insets. The
@@ -679,6 +682,12 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		y++;
     }
     
+	/** Adds an image to the dialog. */
+    public void addImage(ImagePlus image) {
+    	addPanel(new ImagePanel(image));
+    }
+
+    
     /** Set the insets (margins), in pixels, that will be 
     	used for the next component added to the dialog.
     <pre>
@@ -1316,7 +1325,7 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 		IJ.setKeyDown(keyCode); 
 		if (keyCode==KeyEvent.VK_ENTER && textArea1==null) {
 			wasOKed = true;
-			if (IJ.isMacOSX()&&IJ.isJava15())
+			if (IJ.isMacOSX())
 				accessTextFields();
 			dispose();
 		} else if (keyCode==KeyEvent.VK_ESCAPE) { 
@@ -1380,7 +1389,8 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
      *  of the PlugInFilter for preview is avoided in that case.
      */
     private void notifyListeners(AWTEvent e) {
-        if (dialogListeners == null) return;
+        if (dialogListeners==null)
+        	return;
         boolean everythingOk = true;
         for (int i=0; everythingOk && i<dialogListeners.size(); i++)
             try {
@@ -1392,11 +1402,13 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
                 IJ.log("ERROR: "+err+"\nin DialogListener of "+dialogListeners.elementAt(i)+
                 "\nat "+(err.getStackTrace()[0])+"\nfrom "+(err.getStackTrace()[1]));  //requires Java 1.4
             }
-        boolean workaroundOSXbug = IJ.isMacOSX() && !okay.isEnabled() && everythingOk;
+        boolean workaroundOSXbug = IJ.isMacOSX() && okay!=null && !okay.isEnabled() && everythingOk;
         if (previewCheckbox!=null)
             previewCheckbox.setEnabled(everythingOk);
-        okay.setEnabled(everythingOk);
-        if (workaroundOSXbug) repaint(); // OSX 10.4 bug delays update of enabled until the next input
+        if (okay!=null)
+            okay.setEnabled(everythingOk);
+        if (workaroundOSXbug)
+        	repaint(); // OSX 10.4 bug delays update of enabled until the next input
     }
 
 	public void paint(Graphics g) {
@@ -1436,6 +1448,10 @@ FocusListener, ItemListener, KeyListener, AdjustmentListener, WindowListener {
 			new MacroRunner(macro);
 		}
 	}
+	
+	protected boolean isMacro() {
+		return macro;
+	}
     
     public void windowActivated(WindowEvent e) {}
     public void windowOpened(WindowEvent e) {}
