@@ -27,7 +27,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	protected static Cursor crosshairCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
 
 	public static boolean usePointer = Prefs.usePointerCursor;
-	
+
 	protected ImagePlus imp;
 	protected boolean imageUpdated;
 	protected Rectangle srcRect;
@@ -53,7 +53,16 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
     private int mousePressedX, mousePressedY;
     private long mousePressedTime;
     private boolean overOverlayLabel;
-		
+
+    /** If the mouse moves less than this in screen pixels, successive zoom operations are on the same image pixel */
+	protected final static int MAX_MOUSEMOVE_ZOOM = 10;
+	/** Screen coordinates where the last zoom operation was done (initialized to impossible value) */
+	protected int lastZoomSX = -9999999;
+	protected int lastZoomSY = -9999999;
+	/** Image (=offscreen) coordinates where the cursor was moved to for zooming */
+	protected int zoomTargetOX = -1;
+	protected int zoomTargetOY;
+
 	protected ImageJ ij;
 	protected double magnification;
 	protected int dstWidth, dstHeight;
@@ -74,7 +83,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	private boolean scaleToFit;
 	private boolean painted;
 	private boolean hideZoomIndicator;
-
 		
 	public ImageCanvas(ImagePlus imp) {
 		this.imp = imp;
@@ -85,7 +93,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		imageWidth = width;
 		imageHeight = height;
 		srcRect = new Rectangle(0, 0, imageWidth, imageHeight);
-		setDrawingSize(imageWidth, (int)(imageHeight));
+		setSize(imageWidth, imageHeight);
 		magnification = 1.0;
  		addMouseListener(this);
  		addMouseMotionListener(this);
@@ -100,7 +108,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		imageWidth = width;
 		imageHeight = height;
 		srcRect = new Rectangle(0, 0, imageWidth, imageHeight);
-		setDrawingSize(imageWidth, (int)imageHeight);
+		setSize(imageWidth, imageHeight);
 		magnification = 1.0;
 	}
 
@@ -112,27 +120,65 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			return;
 		srcRect = new Rectangle(ic.srcRect.x, ic.srcRect.y, ic.srcRect.width, ic.srcRect.height);
 		setMagnification(ic.magnification);
-		setDrawingSize(ic.dstWidth, ic.dstHeight);
+		setSize(ic.dstWidth, ic.dstHeight);
 	}
 
 	public void setSourceRect(Rectangle r) {
-		srcRect = r;
+		if (r==null)
+			return;
+		if (r.x<0) r.x = 0;
+		if (r.y<0) r.y = 0;
+		if (r.width<1)
+			r.width = 1;
+		if (r.height<1)
+			r.height = 1;
+		if (r.width>imageWidth)
+			r.width = imageWidth;
+		if (r.height>imageHeight)
+			r.height = imageHeight;
+		if (r.x+r.width>imageWidth)
+			r.x = imageWidth-r.width;
+		if (r.y+r.height>imageHeight)
+			r.y = imageHeight-r.height;
+		if (srcRect==null)
+			srcRect = new Rectangle(r.x, r.y, r.width, r.height);
+		else {
+			srcRect.x = r.x;
+			srcRect.y = r.y;
+			srcRect.width = r.width;
+			srcRect.height = r.height;
+		}
+		if (dstWidth==0) {
+			Dimension size = getSize();
+			dstWidth = size.width;
+			dstHeight = size.height;
+		}
+		magnification = (double)dstWidth/srcRect.width;
+		imp.setTitle(imp.getTitle());
+		if (IJ.debugMode) IJ.log("setSourceRect: "+magnification+" "+(int)(srcRect.height*magnification+0.5)+" "+dstHeight+" "+srcRect);
 	}
 
 	void setSrcRect(Rectangle srcRect) {
-		this.srcRect = srcRect;
+		setSourceRect(srcRect);
 	}
 		
 	public Rectangle getSrcRect() {
 		return srcRect;
 	}
 	
+	/** Obsolete; replaced by setSize() */
 	public void setDrawingSize(int width, int height) {
-	    dstWidth = width;
-	    dstHeight = height;
+		dstWidth = width;
+		dstHeight = height;
 		setSize(dstWidth, dstHeight);
 	}
 		
+	public void setSize(int width, int height) {
+		super.setSize(width, height);
+		dstWidth = width;
+		dstHeight = height;
+	}
+
 	/** ImagePlus.updateAndDraw calls this method to force the paint()
 		method to update the image from the ImageProcessor. */
 	public void setImageUpdated() {
@@ -152,11 +198,12 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	}
 
     public void paint(Graphics g) {
-		if (IJ.debugMode) IJ.log("paint");
+		if (IJ.debugMode) IJ.log("ImageCanvas.paint: "+imp);
 		painted = true;
 		Roi roi = imp.getRoi();
 		if (roi!=null || overlay!=null || showAllOverlay!=null || Prefs.paintDoubleBuffered) {
-			if (roi!=null) roi.updatePaste();
+			if (roi!=null)
+				roi.updatePaste();
 			if (!IJ.isMacOSX() && imageWidth!=0) {
 				paintDoubleBuffered(g);
 				setPaintPending(false);
@@ -171,7 +218,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			setInterpolation(g, Prefs.interpolateScaledImages);
 			Image img = imp.getImage();
 			if (img!=null)
- 				g.drawImage(img, 0, 0, (int)(srcRect.width*magnification), (int)(srcRect.height*magnification),
+ 				g.drawImage(img, 0, 0, (int)(srcRect.width*magnification+0.5), (int)(srcRect.height*magnification+0.5),
 				srcRect.x, srcRect.y, srcRect.x+srcRect.width, srcRect.y+srcRect.height, null);
 			if (overlay!=null)
 				drawOverlay(overlay, g);
@@ -251,7 +298,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			slice = imp.getSlice();
 			frame = imp.getFrame();
 		}
-		drawNames = overlay.getDrawNames();
+		drawNames = overlay.getDrawNames() && overlay.getDrawLabels();
 		boolean drawLabels = drawNames || overlay.getDrawLabels();
 		if (drawLabels)
 			labelRects = new Rectangle[n];
@@ -351,6 +398,8 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
     }
     
 	void drawRoiLabel(Graphics g, int index, Roi roi) {
+		if (roi.isCursor())
+			return;
 		boolean pointRoi = roi instanceof PointRoi;
 		Rectangle r = roi.getBounds();
 		int x = screenX(r.x);
@@ -445,8 +494,8 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	// Use double buffer to reduce flicker when drawing complex ROIs.
 	// Author: Erik Meijering
 	void paintDoubleBuffered(Graphics g) {
-		final int srcRectWidthMag = (int)(srcRect.width*magnification);
-		final int srcRectHeightMag = (int)(srcRect.height*magnification);
+		final int srcRectWidthMag = (int)(srcRect.width*magnification+0.5);
+		final int srcRectHeightMag = (int)(srcRect.height*magnification+0.5);
 		if (offScreenImage==null || offScreenWidth!=srcRectWidthMag || offScreenHeight!=srcRectHeightMag) {
 			offScreenImage = createImage(srcRectWidthMag, srcRectHeightMag);
 			offScreenWidth = srcRectWidthMag;
@@ -502,18 +551,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
         return new Dimension(dstWidth, dstHeight);
     }
 
-    int count;
-    
-    /*
-    public Graphics getGraphics() {
-     	Graphics g = super.getGraphics();
-		IJ.write("getGraphics: "+count++);
-		if (IJ.altKeyDown())
-			throw new IllegalArgumentException("");
-    	return g;
-    }
-    */
-
 	/** Returns the current cursor location in image coordinates. */
 	public Point getCursorLoc() {
 		return new Point(xMouse, yMouse);
@@ -644,8 +681,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	}
 		
 	void setMagnification2(double magnification) {
-		if (magnification>32.0) magnification = 32.0;
-		if (magnification<0.03125) magnification = 0.03125;
+		if (magnification>32.0)
+			magnification = 32.0;
+		if (magnification<zoomLevels[0])
+			magnification = zoomLevels[0];
 		this.magnification = magnification;
 		imp.setTitle(imp.getTitle());
 	}
@@ -667,7 +706,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			height = (int)(imageHeight*magnification);
 		Dimension size = getSize();
 		if (srcRect.width<imageWidth || srcRect.height<imageHeight || (painted&&(width!=size.width||height!=size.height))) {
-			setDrawingSize(width, height);
+			setSize(width, height);
 			srcRect.width = (int)(dstWidth/magnification);
 			srcRect.height = (int)(dstHeight/magnification);
 			if ((srcRect.x+srcRect.width)>imageWidth)
@@ -692,7 +731,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		int height=(int)(imageHeight*magnification);
 		if (width==dstWidth&&height==dstHeight) return;
 		srcRect=new Rectangle(0,0,imageWidth, imageHeight);
-		setDrawingSize(width, height);
+		setSize(width, height);
 		getParent().doLayout();
 	}
     
@@ -723,10 +762,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	public static double getLowerZoomLevel(double currentMag) {
 		double newMag = zoomLevels[0];
 		for (int i=0; i<zoomLevels.length; i++) {
-		if (zoomLevels[i] < currentMag)
-			newMag = zoomLevels[i];
-		else
-			break;
+			if (zoomLevels[i] < currentMag)
+				newMag = zoomLevels[i];
+			else
+				break;
 		}
 		return newMag;
 	}
@@ -742,38 +781,45 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		return newMag;
 	}
 
-	/** Zooms in by making the window bigger. If it can't
-		be made bigger, then make the source rectangle 
-		(srcRect) smaller and center it at (sx,sy). Note that
-		sx and sy are screen coordinates. */
+	/** Zooms in by making the window bigger. If it can't be made bigger, then makes 
+		the source rectangle (srcRect) smaller and centers it on the position in the
+		image where the cursor was when zooming has started.
+		Note that sx and sy are screen coordinates. */
 	public void zoomIn(int sx, int sy) {
 		if (magnification>=32) return;
+	    boolean mouseMoved = sqr(sx-lastZoomSX) + sqr(sy-lastZoomSY) > MAX_MOUSEMOVE_ZOOM*MAX_MOUSEMOVE_ZOOM;
+		lastZoomSX = sx;
+		lastZoomSY = sy;
+		if (mouseMoved || zoomTargetOX<0) {
+		    boolean cursorInside = sx >= 0 && sy >= 0 && sx < dstWidth && sy < dstHeight;
+		    zoomTargetOX = offScreenX(cursorInside ? sx : dstWidth/2); //where to zoom, offscreen (image) coordinates
+		    zoomTargetOY = offScreenY(cursorInside ? sy : dstHeight/2);
+		}
 		double newMag = getHigherZoomLevel(magnification);
 		int newWidth = (int)(imageWidth*newMag);
 		int newHeight = (int)(imageHeight*newMag);
 		Dimension newSize = canEnlarge(newWidth, newHeight);
 		if (newSize!=null) {
-			setDrawingSize(newSize.width, newSize.height);
+			setSize(newSize.width, newSize.height);
 			if (newSize.width!=newWidth || newSize.height!=newHeight)
-				adjustSourceRect(newMag, sx, sy);
+				adjustSourceRect(newMag, zoomTargetOX, zoomTargetOY);
 			else
 				setMagnification(newMag);
 			imp.getWindow().pack();
-		} else
-			adjustSourceRect(newMag, sx, sy);
+		} else // can't enlarge window
+			adjustSourceRect(newMag, zoomTargetOX, zoomTargetOY);
 		repaint();
 		if (srcRect.width<imageWidth || srcRect.height<imageHeight)
 			resetMaxBounds();
 	}
-	
+
+	/** Centers the viewable area on offscreen (image) coordinates x, y */
 	void adjustSourceRect(double newMag, int x, int y) {
 		//IJ.log("adjustSourceRect1: "+newMag+" "+dstWidth+"  "+dstHeight);
 		int w = (int)Math.round(dstWidth/newMag);
 		if (w*newMag<dstWidth) w++;
 		int h = (int)Math.round(dstHeight/newMag);
 		if (h*newMag<dstHeight) h++;
-		x = offScreenX(x);
-		y = offScreenY(y);
 		Rectangle r = new Rectangle(x-w/2, y-h/2, w, h);
 		if (r.x<0) r.x = 0;
 		if (r.y<0) r.y = 0;
@@ -783,7 +829,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		setMagnification(newMag);
 		//IJ.log("adjustSourceRect2: "+srcRect+" "+dstWidth+"  "+dstHeight);
 	}
-	
+
+    /** Returns the size to which the window can be enlarged, or null if it can't be enlarged.
+     *  <code>newWidth, newHeight</code> is the size needed for showing the full image
+     *  at the magnification needed */
 	protected Dimension canEnlarge(int newWidth, int newHeight) {
 		if (IJ.altKeyDown())
 			return null;
@@ -811,13 +860,22 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		else
 			return null;
 	}
-		
+
 	/**Zooms out by making the source rectangle (srcRect)  
 		larger and centering it on (x,y). If we can't make it larger,  
-		then make the window smaller.*/
-	public void zoomOut(int x, int y) {
-		if (magnification<=0.03125)
+		then make the window smaller. Note that
+		sx and sy are screen coordinates. */
+	public void zoomOut(int sx, int sy) {
+		if (magnification<=zoomLevels[0])
 			return;
+	    boolean mouseMoved = sqr(sx-lastZoomSX) + sqr(sy-lastZoomSY) > MAX_MOUSEMOVE_ZOOM*MAX_MOUSEMOVE_ZOOM;
+		lastZoomSX = sx;
+		lastZoomSY = sy;
+		if (mouseMoved || zoomTargetOX<0) {
+		    boolean cursorInside = sx >= 0 && sy >= 0 && sx < dstWidth && sy < dstHeight;
+		    zoomTargetOX = offScreenX(cursorInside ? sx : dstWidth/2); //where to zoom, offscreen (image) coordinates
+		    zoomTargetOY = offScreenY(cursorInside ? sy : dstHeight/2);
+		}
 		double oldMag = magnification;
 		double newMag = getLowerZoomLevel(magnification);
 		double srcRatio = (double)srcRect.width/srcRect.height;
@@ -827,21 +885,23 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			double scale = oldMag/newMag;
 			int newSrcWidth = (int)Math.round(srcRect.width*scale);
 			int newSrcHeight = (int)Math.round(srcRect.height*scale);
-			if (newSrcWidth>imageWidth) newSrcWidth=imageWidth;
+			if (newSrcWidth>imageWidth) newSrcWidth=imageWidth; 
 			if (newSrcHeight>imageHeight) newSrcHeight=imageHeight;
 			int newSrcX = srcRect.x - (newSrcWidth - srcRect.width)/2;
 			int newSrcY = srcRect.y - (newSrcHeight - srcRect.height)/2;
+			if (newSrcX + newSrcWidth > imageWidth) newSrcX = imageWidth - newSrcWidth;
+			if (newSrcY + newSrcHeight > imageHeight) newSrcY = imageHeight - newSrcHeight;
 			if (newSrcX<0) newSrcX = 0;
 			if (newSrcY<0) newSrcY = 0;
 			srcRect = new Rectangle(newSrcX, newSrcY, newSrcWidth, newSrcHeight);
-			//IJ.log(newMag+" "+srcRect+" "+dstWidth+" "+dstHeight);
+            //IJ.log(newMag+" "+srcRect+" "+dstWidth+" "+dstHeight);
 			int newDstWidth = (int)(srcRect.width*newMag);
 			int newDstHeight = (int)(srcRect.height*newMag);
 			setMagnification(newMag);
 			setMaxBounds();
-			//IJ.log(newDstWidth+" "+dstWidth+" "+newDstHeight+" "+dstHeight);
+            //IJ.log(newDstWidth+" "+dstWidth+" "+newDstHeight+" "+dstHeight);
 			if (newDstWidth<dstWidth || newDstHeight<dstHeight) {
-				setDrawingSize(newDstWidth, newDstHeight);
+				setSize(newDstWidth, newDstHeight);
 				imp.getWindow().pack();
 			} else
 				repaint();
@@ -852,9 +912,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			if (w*newMag<dstWidth) w++;
 			int h = (int)Math.round(dstHeight/newMag);
 			if (h*newMag<dstHeight) h++;
-			x = offScreenX(x);
-			y = offScreenY(y);
-			Rectangle r = new Rectangle(x-w/2, y-h/2, w, h);
+			Rectangle r = new Rectangle(zoomTargetOX-w/2, zoomTargetOY-h/2, w, h);
 			if (r.x<0) r.x = 0;
 			if (r.y<0) r.y = 0;
 			if (r.x+w>imageWidth) r.x = imageWidth-w;
@@ -863,7 +921,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			setMagnification(newMag);
 		} else {
 			srcRect = new Rectangle(0, 0, imageWidth, imageHeight);
-			setDrawingSize((int)(imageWidth*newMag), (int)(imageHeight*newMag));
+			setSize((int)(imageWidth*newMag), (int)(imageHeight*newMag));
 			setMagnification(newMag);
 			imp.getWindow().pack();
 		}
@@ -873,6 +931,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		repaint();
 	}
 
+    int sqr(int x) {
+        return x*x;
+    }
+
 	/** Implements the Image/Zoom/Original Scale command. */
 	public void unzoom() {
 		double imag = imp.getWindow().getInitialMagnification();
@@ -880,7 +942,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			return;
 		srcRect = new Rectangle(0, 0, imageWidth, imageHeight);
 		ImageWindow win = imp.getWindow();
-		setDrawingSize((int)(imageWidth*imag), (int)(imageHeight*imag));
+		setSize((int)(imageWidth*imag), (int)(imageHeight*imag));
 		setMagnification(imag);
         setMaxBounds();
 		win.pack();
@@ -998,7 +1060,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	}
 
 	public void mousePressed(MouseEvent e) {
-		//if (ij==null) return;
 		showCursorStatus = true;
 		int toolID = Toolbar.getToolId();
 		ImageWindow win = imp.getWindow();
@@ -1013,8 +1074,7 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		int x = e.getX();
 		int y = e.getY();
 		flags = e.getModifiers();
-		//IJ.log("Mouse pressed: " + e.isPopupTrigger() + "  " + ij.modifiers(flags));		
-		//if (toolID!=Toolbar.MAGNIFIER && e.isPopupTrigger()) {
+		
 		if (toolID!=Toolbar.MAGNIFIER && (e.isPopupTrigger()||(!IJ.isMacintosh()&&(flags&Event.META_MASK)!=0))) {
 			handlePopupMenu(e);
 			return;
@@ -1059,12 +1119,10 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 				if (IJ.shiftKeyDown())
 					zoomToSelection(ox, oy);
 				else if ((flags & (Event.ALT_MASK|Event.META_MASK|Event.CTRL_MASK))!=0) {
-					//IJ.run("Out");
 					zoomOut(x, y);
 					if (getMagnification()<1.0)
 						imp.repaintWindow();
 				} else {
-					//IJ.run("In");
 	 				zoomIn(x, y);
 					if (getMagnification()<=1.0)
 						imp.repaintWindow();
@@ -1078,7 +1136,8 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 				break;
 			case Toolbar.WAND:
 				Roi roi = imp.getRoi();
-				if (roi!=null && roi.contains(ox, oy)) {
+				double tolerance = WandToolOptions.getTolerance();
+				if (roi!=null && (tolerance==0.0||imp.isThreshold()) && roi.contains(ox, oy)) {
 					Rectangle r = roi.getBounds();
 					if (r.width==imageWidth && r.height==imageHeight)
 						imp.deleteRoi();
@@ -1096,7 +1155,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 				}
 				setRoiModState(e, roi, -1);
 				String mode = WandToolOptions.getMode();
-				double tolerance = WandToolOptions.getTolerance();
 				int npoints = IJ.doWand(ox, oy, tolerance, mode);
 				if (Recorder.record && npoints>0) {
 					if (Recorder.scriptMode())
@@ -1173,7 +1231,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 			tool.mouseExited(imp, e);
 			if (e.isConsumed()) return;
 		}
-		//autoScroll(e);
 		ImageWindow win = imp.getWindow();
 		if (win!=null)
 			setCursor(defaultCursor);
@@ -1181,32 +1238,6 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		mouseExited = true;
 	}
 
-	/*
-	public void autoScroll(MouseEvent e) {
-		Roi roi = imp.getRoi();
-		if (roi==null || roi.getState()!=roi.CONSTRUCTING || srcRect.width>=imageWidth || srcRect.height>=imageHeight
-		|| !(roi.getType()==Roi.POLYGON || roi.getType()==Roi.POLYLINE || roi.getType()==Roi.ANGLE))
-			return;
-		int sx = e.getX();
-		int sy = e.getY();
-		xMouseStart = srcRect.x+srcRect.width/2;
-		yMouseStart = srcRect.y+srcRect.height/2;
-		Rectangle r = roi.getBounds();
-		Dimension size = getSize();
-		int deltax=0, deltay=0;
-		if (sx<0)
-			deltax = srcRect.width/4;
-		else if (sx>size.width)
-			deltax = -srcRect.width/4;
-		if (sy<0)
-			deltay = srcRect.height/4;
-		else if (sy>size.height)
-			deltay = -srcRect.height/4;
-		//IJ.log("autoscroll: "+sx+" "+sy+" "+deltax+" "+deltay+" "+r);
-		scroll(screenX(xMouseStart+deltax), screenY(yMouseStart+deltay));
-	}
-	*/
-
 	public void mouseDragged(MouseEvent e) {
 		int x = e.getX();
 		int y = e.getY();
@@ -1242,7 +1273,22 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 		boolean multiPointMode = roi!=null && (roi instanceof PointRoi) && handle==-1
 			&& Toolbar.getToolId()==Toolbar.POINT && Toolbar.getMultiPointMode();
 		if (multiPointMode) {
-			imp.setRoi(((PointRoi)roi).addPoint(offScreenXD(sx), offScreenYD(sy)));
+			double oxd = offScreenXD(sx);
+			double oyd = offScreenYD(sy);
+			if (e.isShiftDown()) {
+				FloatPolygon points = roi.getFloatPolygon();
+				if (points.npoints>0) {
+					double x0 = points.xpoints[0];
+					double y0 = points.ypoints[0];
+					double slope = Math.abs((oxd-x0)/(oyd-y0));
+					if (slope>=1.0)
+						oyd = points.ypoints[0];
+					else
+						oxd = points.xpoints[0];
+				}
+			}
+			((PointRoi)roi).addPoint(imp, oxd, oyd);
+			imp.setRoi(roi);
 			return;
 		}
 		setRoiModState(e, roi, handle);
@@ -1592,12 +1638,14 @@ public class ImageCanvas extends Canvas implements MouseListener, MouseMotionLis
 	
 	public boolean hideZoomIndicator(boolean hide) {
 		boolean hidden = this.hideZoomIndicator;
+		if (!(srcRect.width<imageWidth||srcRect.height<imageHeight))
+			return hidden;
 		this.hideZoomIndicator = hide;
 		setPaintPending(true);
 		repaint();
 		long t0 = System.currentTimeMillis();
-		while(getPaintPending() && (System.currentTimeMillis()-t0)<250L)
-			IJ.wait(1);
+		while(getPaintPending() && (System.currentTimeMillis()-t0)<500L)
+			IJ.wait(10);
 		return hidden;
 	}
 
