@@ -8,6 +8,7 @@ import ij.process.*;
 import ij.io.*;
 import ij.measure.*;
 import ij.plugin.frame.*;
+import ij.plugin.PointToolOptions;
 import ij.macro.Interpreter;
 import ij.util.*;
 
@@ -77,8 +78,10 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		boolean openAsHyperStack = imp.getOpenAsHyperStack();
 		ij = IJ.getInstance();
 		this.imp = imp;
-		if (ic==null)
-			{ic=new ImageCanvas(imp); newCanvas=true;}
+		if (ic==null) {
+			ic = (this instanceof PlotWindow) ? new PlotCanvas(imp) : new ImageCanvas(imp);
+			newCanvas=true;
+		}
 		this.ic = ic;
 		ImageWindow previousWindow = imp.getWindow();
 		setLayout(new ImageLayout(ic));
@@ -102,7 +105,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 				ic.update(previousWindow.getCanvas());
 			Point loc = previousWindow.getLocation();
 			setLocation(loc.x, loc.y);
-			if (!(this instanceof StackWindow)) {
+			if (!(this instanceof StackWindow || this instanceof PlotWindow)) { //layout now unless components will be added later
 				pack();
 				show();
 			}
@@ -187,7 +190,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		
 		if (mag<1.0) {
 			initialMagnification = mag;
-			ic.setDrawingSize((int)(width*mag), (int)(height*mag));
+			ic.setSize((int)(width*mag), (int)(height*mag));
 		}
 		ic.setMagnification(mag);
 		if (y+height*mag>screenHeight)
@@ -289,14 +292,14 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		} else {
 			String label = (String)imp.getProperty("Label");
 			if (label!=null) {
-			int newline = label.indexOf('\n');
-			if (newline>0)
-				label = label.substring(0, newline);
-			int len = label.length();
-			if (len>4 && label.charAt(len-4)=='.' && !Character.isDigit(label.charAt(len-1)))
-				label = label.substring(0,len-4);
-			if (label.length()>60)
-				label = label.substring(0, 60);
+				int newline = label.indexOf('\n');
+				if (newline>0)
+					label = label.substring(0, newline);
+				int len = label.length();
+				if (len>4 && label.charAt(len-4)=='.' && !Character.isDigit(label.charAt(len-1)))
+					label = label.substring(0,len-4);
+				if (label.length()>60)
+					label = label.substring(0, 60);
 				s = label + "; ";
 			}
 		}
@@ -318,7 +321,6 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 			}
     	} else
     		s += imp.getWidth() + "x" + imp.getHeight() + " pixels; ";
-		double size = ((double)imp.getWidth()*imp.getHeight()*imp.getStackSize())/1024.0;
     	switch (type) {
 	    	case ImagePlus.GRAY8:
 	    	case ImagePlus.COLOR_256:
@@ -326,19 +328,29 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	    		break;
 	    	case ImagePlus.GRAY16:
 	    		s += "16-bit";
-				size *= 2.0;
 	    		break;
 	    	case ImagePlus.GRAY32:
 	    		s += "32-bit";
-				size *= 4.0;
 	    		break;
 	    	case ImagePlus.COLOR_RGB:
 	    		s += "RGB";
-				size *= 4.0;
 	    		break;
     	}
     	if (imp.isInvertedLut())
     		s += " (inverting LUT)";
+     	return s+"; "+getImageSize(imp);
+    }
+    
+    public static String getImageSize(ImagePlus imp) {
+    	if (imp==null)
+    		return null;
+		double size = ((double)imp.getWidth()*imp.getHeight()*imp.getStackSize())/1024.0;
+		int type = imp.getType();
+    	switch (type) {
+	    	case ImagePlus.GRAY16: size *= 2.0; break;
+	    	case ImagePlus.GRAY32: size *= 4.0; break;
+	    	case ImagePlus.COLOR_RGB: size *= 4.0; break;
+    	}
    		String s2=null, s3=null;
     	if (size<1024.0)
     		{s2=IJ.d2s(size,0); s3="K";}
@@ -349,7 +361,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	   	else
     		{s2=IJ.d2s(size/1048576.0,1); s3="GB";}
     	if (s2.endsWith(".0")) s2 = s2.substring(0, s2.length()-2);
-     	return s+"; "+s2+s3;
+    	return s2+s3;
     }
     
     private String d2s(double n) {
@@ -375,7 +387,7 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		running = running2 = false;
 		boolean virtual = imp.getStackSize()>1 && imp.getStack().isVirtual();
 		if (isRunning) IJ.wait(500);
-		if (ij==null || IJ.getApplet()!=null || Interpreter.isBatchMode() || IJ.macroRunning() || virtual)
+		if (ij==null || ij.quittingViaMacro() || IJ.getApplet()!=null || Interpreter.isBatchMode() || IJ.macroRunning() || virtual)
 			imp.changes = false;
 		if (imp.changes) {
 			String msg;
@@ -509,27 +521,13 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	}
 	
 	public void maximize() {
-		if (maxBounds==null)
-			return;
-		int width = imp.getWidth();
-		int height = imp.getHeight();
-		double aspectRatio = (double)width/height;
-		Dimension extraSize = getExtraSize();
-		int extraHeight = extraSize.height;
-		double mag = (double)(maxBounds.height-extraHeight)/height;
-		if (IJ.debugMode) IJ.log("maximize: "+mag+" "+ic.getMagnification()+" "+maxBounds);
-		setSize(getMaximizedBounds().width, getMaximizedBounds().height);
-		if (mag>ic.getMagnification() || aspectRatio<0.5 || aspectRatio>2.0) {
-			ic.setMagnification2(mag);
-			ic.setSrcRect(new Rectangle(0, 0, width, height));
-			ic.setDrawingSize((int)(width*mag), (int)(height*mag));
-			validate();
-			unzoomWhenMinimizing = true;
-		} else
-			unzoomWhenMinimizing = false;
+		Rectangle rect = getMaximumBounds();
+		if (IJ.debugMode) IJ.log("maximize: "+rect);
+		setLocationAndSize(rect.x, rect.y, rect.width, rect.height);
 	}
 	
 	public void minimize() {
+		if (IJ.debugMode) IJ.log("minimize: "+unzoomWhenMinimizing);
 		if (unzoomWhenMinimizing)
 			ic.unzoom();
 		unzoomWhenMinimizing = true;
@@ -549,17 +547,20 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 
 	public void windowActivated(WindowEvent e) {
 		if (IJ.debugMode) IJ.log("windowActivated: "+imp.getTitle());
+		if (IJ.isMacOSX())
+			setImageJMenuBar(this);
+		if (imp==null)
+			return;
 		ImageJ ij = IJ.getInstance();
-		boolean quitting = ij!=null && ij.quitting();
-		if (IJ.isMacintosh() && ij!=null && !quitting) {
-			IJ.wait(10); // may be needed for Java 1.4 on OS X
-			setMenuBar(Menus.getMenuBar());
-		}
-		if (imp==null) return;
-		if (!closed && !quitting && !Interpreter.isBatchMode())
+		if (ij!=null && !closed && !ij.quitting() && !Interpreter.isBatchMode())
 			WindowManager.setCurrentWindow(this);
 		if (imp.isComposite())
 			Channels.updateChannels();
+		Roi roi = imp.getRoi();
+		if (roi!=null && (roi instanceof PointRoi))
+			PointToolOptions.update();
+		if (imp.isComposite())
+			Channels.updateChannels();
 		imp.setActivated(); // notify ImagePlus that image has been activated
 	}
 	
@@ -580,11 +581,9 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	public void windowStateChanged(WindowEvent e) {
 		int oldState = e.getOldState();
 		int newState = e.getNewState();
-		//IJ.log("WSC: "+getBounds()+" "+oldState+" "+newState);
-		if ((oldState & Frame.MAXIMIZED_BOTH) == 0 && (newState & Frame.MAXIMIZED_BOTH) != 0)
+		if (IJ.debugMode) IJ.log("windowStateChanged: "+oldState+" "+newState);
+		if ((oldState&Frame.MAXIMIZED_BOTH)==0 && (newState&Frame.MAXIMIZED_BOTH)!=0)
 			maximize();
-		else if ((oldState & Frame.MAXIMIZED_BOTH) != 0 && (newState & Frame.MAXIMIZED_BOTH) == 0)
-			minimize();
 	}
 
 	public void windowClosed(WindowEvent e) {}
@@ -614,14 +613,13 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 		int xstart = srcRect.x;
 		int ystart = srcRect.y;
 		if ((ctrl||IJ.shiftKeyDown()) && ic!=null) {
-			int ox = ic.offScreenX(e.getX());
-			int oy = ic.offScreenY(e.getX());
-			if (IJ.debugMode)
-				IJ.log("  x,y: "+ox+","+oy);
+			Point loc = ic.getCursorLoc();
+			int x = ic.screenX(loc.x);
+			int y = ic.screenY(loc.y);
 			if (rotation<0)
-				ic.zoomIn(ox,oy);
+				ic.zoomIn(x, y);
 			else
-				ic.zoomOut(ox,oy);
+				ic.zoomOut(x, y);
 			return;
 		} else if (IJ.spaceBarDown() || srcRect.height==height) {
 			srcRect.x += rotation*amount*Math.max(width/200, 1);
@@ -689,7 +687,23 @@ public class ImageWindow extends Frame implements FocusListener, WindowListener,
 	public int getSliderHeight() {
 		return sliderHeight;
 	}
-
+	
+	public static void setImageJMenuBar(ImageWindow win) {
+		ImageJ ij = IJ.getInstance();
+		boolean setMenuBar = true;
+		ImagePlus imp = win.getImagePlus();
+		if (imp!=null)
+			setMenuBar = imp.setIJMenuBar();
+		MenuBar mb = Menus.getMenuBar();
+		if (mb!=null && mb==win.getMenuBar())
+			setMenuBar = false;
+		if (ij!=null && !ij.quitting() && !Interpreter.nonBatchMacroRunning() && setMenuBar) {
+			IJ.wait(10); // may be needed for Java 1.4 on OS X
+			win.setMenuBar(mb);
+			Menus.setMenuBarCount++;
+		}
+		if (imp!=null) imp.setIJMenuBar(true);
+	}
 			
 } //class ImageWindow
 
