@@ -40,6 +40,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	protected static int pasteMode = Blitter.COPY;
 	protected static int lineWidth = 1;
 	protected static Color defaultFillColor;
+	private static Vector listeners = new Vector();
 	
 	protected int type;
 	protected int xMax, yMax;
@@ -72,6 +73,9 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	private boolean subPixel;
 	private boolean activeOverlayRoi;
 	private Properties props;
+	private boolean isCursor;
+	private double xcenter = Double.NaN;
+	private double ycenter;
 
 
 	/** Creates a rectangular ROI. */
@@ -672,14 +676,12 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		int oy = ic.offScreenY(sy);
 		if (ox<0) ox=0; if (oy<0) oy=0;
 		if (ox>xMax) ox=xMax; if (oy>yMax) oy=yMax;
-		//IJ.log("moveHandle: "+activeHandle+" "+ox+" "+oy);
 		int x1=x, y1=y, x2=x1+width, y2=y+height, xc=x+width/2, yc=y+height/2;
 		if (width > 7 && height > 7) {
 			asp = (double)width/(double)height;
 			asp_bk = asp;
-		} else {
+		} else
 			asp = asp_bk;
-		}
 		
 		switch (activeHandle) {
 			case 0:
@@ -717,7 +719,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		   {height=1; y=y2;}
 		
 		if (center) {
-			switch(activeHandle){
+			switch (activeHandle){
 				case 0:
 					width=(xc-x)*2;
 					height=(yc-y)*2;
@@ -778,7 +780,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 				height=1;
 				y=y2=yc;
 			}
-			switch(activeHandle){
+			switch (activeHandle) {
 				case 0:
 					x=x2-width;
 					y=y2-height;
@@ -804,7 +806,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 					x=x2-width;
 					break;
 			}
-			if (center){
+			if (center) {
 				x=xc-width/2;
 				y=yc-height/2;
 			}
@@ -814,7 +816,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			if (activeHandle==1 || activeHandle==5) width=(int)Math.rint((double)height*asp);
 			else height=(int)Math.rint((double)width/asp);
 			
-			switch(activeHandle){
+			switch (activeHandle){
 				case 0:
 					x=x2-width;
 					y=y2-height;
@@ -840,7 +842,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 					x=x2-width;
 					break;
 			}
-			if (center){
+			if (center) {
 				x=xc-width/2;
 				y=yc-height/2;
 			}
@@ -937,6 +939,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		oldX = x; oldY = y;
 		bounds = null;
 		showStatus();
+		notifyListeners(RoiListener.MOVED);
 	}
 	
 	/** Nudge lower right corner of rectangular and oval ROIs by
@@ -969,8 +972,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		showStatus();
 	}
 	
-	protected void updateClipRect() {
 	// Finds the union of current and previous roi
+	protected void updateClipRect() {
 		clipX = (x<=oldX)?x:oldX;
 		clipY = (y<=oldY)?y:oldY;
 		clipWidth = ((x+width>=oldX+oldWidth)?x+width:oldX+oldWidth) - clipX + 1;
@@ -985,7 +988,6 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		m = (int)(m+getStrokeWidth()*2);
 		clipX-=m; clipY-=m;
 		clipWidth+=m*2; clipHeight+=m*2;
-		//if (IJ.debugMode) IJ.log("updateClipRect: "+m+"  "+clipX+" "+clipY+" "+clipWidth+" "+clipHeight);
 	 }
 	 
 	protected int clipRectMargin() {
@@ -1010,6 +1012,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			default:
 				break;
 		}
+		notifyListeners(state==MOVING?RoiListener.MOVED:RoiListener.MODIFIED);
 	}
 
 	int getHandleSize() {
@@ -1229,7 +1232,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 					Recorder.record("makeRectangle", x, y, width, height);
 				else {
 					if (Recorder.scriptMode())
-						Recorder.recordCall("imp.setRoi(new Roi("+x+", "+y+", "+width+", "+height+", "+cornerDiameter+"));");
+						Recorder.recordCall("imp.setRoi(new Roi("+x+","+y+","+width+","+height+","+cornerDiameter+"));");
 					else
 						Recorder.record("makeRectangle", x, y, width, height, cornerDiameter);
 				}
@@ -1292,9 +1295,9 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		}
 		previousRoi.modState = NO_MODS;
 		PointRoi p1 = (PointRoi)previousRoi;
-		Rectangle r = getBounds();
 		FloatPolygon poly = getFloatPolygon();
-		imp.setRoi(p1.addPoint(poly.xpoints[0], poly.ypoints[0]));
+		p1.addPoint(imp, poly.xpoints[0], poly.ypoints[0]);
+		imp.setRoi(p1);
 	}
 	
 	void subtractPoints() {
@@ -1323,7 +1326,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	 }
 
 	protected void showStatus() {
-		if (imp==null) return;
+		if (imp==null)
+			return;
 		String value;
 		if (state!=CONSTRUCTING && (type==RECTANGLE||type==POINT) && width<=25 && height<=25) {
 			ImageProcessor ip = imp.getProcessor();
@@ -1334,8 +1338,8 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 			value = "";
 		Calibration cal = imp.getCalibration();
 		String size;
-		if (cal.scaled() && !IJ.altKeyDown())
-			size = ", w="+IJ.d2s(width*cal.pixelWidth)+", h="+IJ.d2s(height*cal.pixelHeight);
+		if (cal.scaled() && !(IJ.altKeyDown()||(state==NORMAL&&IJ.shiftKeyDown())))
+			size = ", w="+IJ.d2s(width*cal.pixelWidth)+" ("+width+"), h="+IJ.d2s(height*cal.pixelHeight)+" ("+height+")";
 		else
 			size = ", w="+width+", h="+height;
 		IJ.showStatus(imp.getLocationAsString(x,y)+size+value);
@@ -1351,6 +1355,7 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	
 	public void startPaste(ImagePlus clipboard) {
 		IJ.showStatus("Pasting...");
+		IJ.wait(10);
 		this.clipboard = clipboard;
 		imp.getProcessor().snapshot();
 		updateClipRect();
@@ -1491,7 +1496,6 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 	}
 		
 	public void updateWideLine(float width) {
-		//IJ.log("updateWideLine "+isLine()+"  "+isDrawingTool()+"	"+getType());
 		if (isLine()) {
 			wideLine = true;
 			setStrokeWidth(width);
@@ -1939,16 +1943,101 @@ public class Roi extends Object implements Cloneable, java.io.Serializable {
 		bounds = new Rectangle2D.Double(getXBase(), getYBase(), getFloatWidth(), getFloatHeight());
 		subPixel = true;
 	}
+	
+	public void setIsCursor(boolean isCursor) {
+		this.isCursor = isCursor;
+	}
+
+	public boolean isCursor() {
+		return isCursor;
+	}
 
 	public String getDebugInfo() {
 		return "";
 	}
+	
+	public ImageStatistics getStatistics() {
+		ImageProcessor ip = getMask();
+		Rectangle r = getBounds();
+		if (ip==null)
+			ip = new ByteProcessor(r.width, r.height);
+		Roi roi = (Roi)this.clone();
+		roi.setLocation(0.0, 0.0);
+		ip.setRoi(roi);
+		int params = Measurements.AREA+Measurements.CENTROID+Measurements.ELLIPSE
+			+Measurements.ELLIPSE+Measurements.CIRCULARITY+Measurements.SHAPE_DESCRIPTORS
+			+Measurements.PERIMETER+Measurements.RECT;
+		ImageStatistics stats = ImageStatistics.getStatistics(ip, params, null);
+		stats.mean = stats.min = stats.max = Double.NaN;
+		stats.xCentroid += r.x;
+		stats.yCentroid += r.y;
+		return stats;
+	}
+
+	public FloatPolygon getRotationCenter() {
+		FloatPolygon p = new FloatPolygon();
+		Rectangle2D r = getFloatBounds();
+		if (Double.isNaN(xcenter)) {
+			xcenter = r.getX()+r.getWidth()/2.0;
+			ycenter = r.getY()+r.getHeight()/2.0;
+		}
+		p.addPoint(xcenter,ycenter);
+		return p;
+	}
+
+	public void setRotationCenter(double x, double y) {
+		xcenter = x;
+		ycenter = y;
+	}
+	
+	/* 
+	 * Returns the center of the of this selection's countour, or the
+	 * center of the bounding box of composite selections.<br> 
+	 * Author: Peter Haub (phaub at dipsystems.de)
+	 */
+	public double[] getContourCentroid() {
+		double xC=0, yC=0, lSum=0, x, y, dx, dy, l;
+		FloatPolygon poly = getFloatPolygon();
+		int nPoints = poly.npoints;
+		int n2 = nPoints-1;
+		for (int n1=0; n1<nPoints; n1++){
+			dx = poly.xpoints[n1] - poly.xpoints[n2];
+			dy = poly.ypoints[n1] - poly.ypoints[n2];
+			x = poly.xpoints[n2] + dx/2.0;
+			y = poly.ypoints[n2] + dy/2.0;
+			l = Math.sqrt(dx*dx + dy*dy);
+			xC += x*l;
+			yC += y*l;
+			lSum += l;
+			n2 = n1;
+		}
+		xC /= lSum;
+		yC /= lSum;
+		return new double[]{xC, yC};
+	}
 
 	/** Returns a hashcode for this Roi that typically changes 
-		if it is moved,	 even though it is still the same object. */
+		if it is moved, even though it is still the same object. */
 	public int getHashCode() {
 		return hashCode() ^ (new Double(getXBase()).hashCode()) ^
 			Integer.rotateRight(new Double(getYBase()).hashCode(),16);
 	}
+	
+	public void notifyListeners(int id) {
+		synchronized (listeners) {
+			for (int i=0; i<listeners.size(); i++) {
+				RoiListener listener = (RoiListener)listeners.elementAt(i);
+				listener.roiModified(imp, id);
+			}
+		}
+	}
+
+	public static void addRoiListener(RoiListener listener) {
+		listeners.addElement(listener);
+	}
+	
+	public static void removeRoiListener(RoiListener listener) {
+		listeners.removeElement(listener);
+	}
 
 }
