@@ -11,6 +11,7 @@ import ij.plugin.JpegWriter;
 import ij.plugin.Orthogonal_Views;
 import ij.gui.*;
 import ij.measure.Measurements;
+import ij.util.Tools;
 import javax.imageio.*;
 
 /** Saves images in tiff, gif, jpeg, raw, zip and text format. */
@@ -94,9 +95,10 @@ public class FileSaver {
 	/** Saves the image in TIFF format using the specified path. Equivalent to
 		 IJ.saveAsTiff(imp,path), which is more convenient. */
 	public boolean saveAsTiff(String path) {
+		if (fi.nImages>1)
+			return saveAsTiffStack(path);
 		if (imp.getProperty("FHT")!=null && path.contains("FFT of "))
 			setupFFTSave();
-		fi.nImages = 1;
 		fi.info = imp.getInfoProperty();
 		Object label = imp.getProperty("Label");
 		if (label!=null && (label instanceof String)) {
@@ -106,14 +108,18 @@ public class FileSaver {
 		fi.description = getDescriptionString();
 		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
 		fi.overlay = getOverlay(imp);
+		DataOutputStream out = null;
 		try {
 			TiffEncoder file = new TiffEncoder(fi);
-			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
+			out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
 			file.write(out);
 			out.close();
 		} catch (IOException e) {
-			showErrorMessage(e);
+			showErrorMessage("saveAsTiff", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		updateImp(fi, FileInfo.TIFF);
 		return true;
@@ -160,16 +166,18 @@ public class FileSaver {
 	/** Saves the stack as a multi-image TIFF using the specified path.
 		 Equivalent to IJ.saveAsTiff(imp,path), which is more convenient. */
 	public boolean saveAsTiffStack(String path) {
-		if (fi.nImages==1)
-			{error("This is not a stack"); return false;}
+		if (fi.nImages==1) {
+			error("This is not a stack");
+			return false;
+		}
 		boolean virtualStack = imp.getStack().isVirtual();
 		if (virtualStack)
 			fi.virtualStack = (VirtualStack)imp.getStack();
 		fi.info = imp.getInfoProperty();
 		fi.description = getDescriptionString();
 		if (virtualStack) {
-			FileInfo fi = imp.getOriginalFileInfo();
-			if (path!=null && path.equals(fi.directory+fi.fileName)) {
+			FileInfo ofi = imp.getOriginalFileInfo();
+			if (path!=null && ofi!=null && path.equals(ofi.directory+ofi.fileName)) {
 				error("TIFF virtual stacks cannot be saved in place.");
 				return false;
 			}
@@ -184,21 +192,24 @@ public class FileSaver {
 					labels = new String[vs.getSize()];
 				labels[i-1] = label;
 			}
-			this.fi.sliceLabels = labels;
+			fi.sliceLabels = labels;
 		} else
 			fi.sliceLabels = imp.getStack().getSliceLabels();
 		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
 		fi.overlay = getOverlay(imp);
 		if (imp.isComposite()) saveDisplayRangesAndLuts(imp, fi);
+		DataOutputStream out = null;
 		try {
 			TiffEncoder file = new TiffEncoder(fi);
-			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
+			out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
 			file.write(out);
 			out.close();
-		}
-		catch (IOException e) {
-			showErrorMessage(e);
+		} catch (IOException e) {
+			showErrorMessage("saveAsTiffStack", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		updateImp(fi, FileInfo.TIFF);
 		return true;
@@ -225,6 +236,9 @@ public class FileSaver {
 			out.close();
 		} catch (IOException e) {
 			return null;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		return out.toByteArray();
 	}
@@ -281,17 +295,21 @@ public class FileSaver {
 		if (imp.isComposite()) saveDisplayRangesAndLuts(imp, fi);
 		if (fi.nImages>1 && imp.getStack().isVirtual())
 			fi.virtualStack = (VirtualStack)imp.getStack();
+		DataOutputStream out = null;
 		try {
 			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(path));
-			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(zos));
+			out = new DataOutputStream(new BufferedOutputStream(zos));
         	zos.putNextEntry(new ZipEntry(name));
 			TiffEncoder te = new TiffEncoder(fi);
 			te.write(out);
 			out.close();
 		}
 		catch (IOException e) {
-			showErrorMessage(e);
+			showErrorMessage("saveAsZip", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		updateImp(fi, FileInfo.TIFF);
 		return true;
@@ -457,7 +475,6 @@ public class FileSaver {
 			return saveAsRawStack(path);
 	}
 	
-	/** Save the image as raw data using the specified path. */
 	/** Save the image as raw data using the specified path. */
 	public boolean saveAsRaw(String path) {
 		fi.nImages = 1;
@@ -465,6 +482,7 @@ public class FileSaver {
 		boolean signed16Bit = false;
 		short[] pixels = null;
 		int n = 0;
+		OutputStream out = null;
 		try {
 			signed16Bit = imp.getCalibration().isSigned16Bit();
 			if (signed16Bit) {
@@ -474,13 +492,16 @@ public class FileSaver {
 					pixels[i] = (short)(pixels[i]-32768);
 			}
 			ImageWriter file = new ImageWriter(fi);
-			OutputStream out = new BufferedOutputStream(new FileOutputStream(path));
+			out = new BufferedOutputStream(new FileOutputStream(path));
 			file.write(out);
 			out.close();
 		}
 		catch (IOException e) {
-			showErrorMessage(e);
+			showErrorMessage("saveAsRaw", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		if (signed16Bit) {
 			for (int i=0; i<n; i++)
@@ -503,6 +524,7 @@ public class FileSaver {
 			fi.virtualStack = (VirtualStack)imp.getStack();
 			if (imp.getProperty("AnalyzeFormat")!=null) fi.fileName="FlipTheseImages";
 		}
+		OutputStream out = null;
 		try {
 			signed16Bit = imp.getCalibration().isSigned16Bit();
 			if (signed16Bit && !virtualStack) {
@@ -515,13 +537,15 @@ public class FileSaver {
 				}
 			}
 			ImageWriter file = new ImageWriter(fi);
-			OutputStream out = new BufferedOutputStream(new FileOutputStream(path));
+			out = new BufferedOutputStream(new FileOutputStream(path));
 			file.write(out);
 			out.close();
-		}
-		catch (IOException e) {
-			showErrorMessage(e);
+		} catch (IOException e) {
+			showErrorMessage("saveAsRawStack", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		if (signed16Bit) {
 			for (int slice=0; slice<fi.nImages; slice++) {
@@ -545,6 +569,7 @@ public class FileSaver {
 	
 	/** Save the image as tab-delimited text using the specified path. */
 	public boolean saveAsText(String path) {
+		DataOutputStream out = null;
 		try {
 			Calibration cal = imp.getCalibration();
 			int precision = Analyzer.getPrecision();
@@ -553,13 +578,15 @@ public class FileSaver {
 			if (scientificNotation)
 				precision = -precision;
 			TextEncoder file = new TextEncoder(imp.getProcessor(), cal, precision);
-			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
+			out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
 			file.write(out);
 			out.close();
-		}
-		catch (IOException e) {
-			showErrorMessage(e);
+		} catch (IOException e) {
+			showErrorMessage("saveAsText", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		return true;
 	}
@@ -603,15 +630,19 @@ public class FileSaver {
 		fi.height = 1;
 		fi.pixels = pixels;
 
+		OutputStream out = null;
 		try {
 			ImageWriter file = new ImageWriter(fi);
-			OutputStream out = new FileOutputStream(path);
+			out = new FileOutputStream(path);
 			file.write(out);
 			out.close();
 		}
 		catch (IOException e) {
-			showErrorMessage(e);
+			showErrorMessage("saveAsLut", path, e);
 			return false;
+		} finally {
+			if (out!=null)
+				try {out.close();} catch (IOException e) {}
 		}
 		return true;
 	}
@@ -651,11 +682,12 @@ public class FileSaver {
 		}
 	}
 
-	void showErrorMessage(IOException e) {
+	private void showErrorMessage(String title, String path, IOException e) {
 		String msg = e.getMessage();
 		if (msg.length()>100)
 			msg = msg.substring(0, 100);
-		error("An error occured writing the file.\n \n" + msg);
+		msg = "File saving error (IOException):\n   \"" + msg + "\"";
+		IJ.error("FileSaver."+title, msg+" \n   "+path);
 	}
 	
 	private void error(String msg) {
@@ -684,7 +716,7 @@ public class FileSaver {
 			sb.append("mode="+mode+"\n");
 		}
 		if (fi.unit!=null)
-			sb.append("unit="+(fi.unit.equals("\u00B5m")?"um":fi.unit)+"\n");
+			appendEscapedLine(sb, "unit="+fi.unit);
 		int bitDepth = imp.getBitDepth();
 		if (fi.valueUnit!=null && (fi.calibrationFunction!=Calibration.CUSTOM||bitDepth==32)) {
 			if (bitDepth!=32) {
@@ -694,12 +726,11 @@ public class FileSaver {
 						sb.append("c"+i+"="+fi.coefficients[i]+"\n");
 				}
 			}
-			sb.append("vunit="+fi.valueUnit+"\n");
+			appendEscapedLine(sb, "vunit="+fi.valueUnit);
 			if (cal.zeroClip() && bitDepth!=32)
 				sb.append("zeroclip=true\n");
 		}
-		
-		// get stack z-spacing and fps
+		// get stack z-spacing, more units and fps
 		if (cal.frameInterval!=0.0) {
 			if ((int)cal.frameInterval==cal.frameInterval)
 				sb.append("finterval="+(int)cal.frameInterval+"\n");
@@ -707,7 +738,11 @@ public class FileSaver {
 				sb.append("finterval="+cal.frameInterval+"\n");
 		}
 		if (!cal.getTimeUnit().equals("sec"))
-			sb.append("tunit="+cal.getTimeUnit()+"\n");
+			appendEscapedLine(sb, "tunit="+cal.getTimeUnit());
+		if (!cal.getYUnit().equals(cal.getUnit()))
+			appendEscapedLine(sb, "yunit="+cal.getYUnit());
+		if (!cal.getZUnit().equals(cal.getUnit()))
+			appendEscapedLine(sb, "zunit="+cal.getZUnit());
 		if (fi.nImages>1) {
 			if (fi.pixelDepth!=1.0)
 				sb.append("spacing="+fi.pixelDepth+"\n");
@@ -719,7 +754,7 @@ public class FileSaver {
 			}
 			sb.append("loop="+(cal.loop?"true":"false")+"\n");
 		}
-		
+
 		// get min and max display values
 		ImageProcessor ip = imp.getProcessor();
 		double min = ip.getMin();
@@ -739,13 +774,33 @@ public class FileSaver {
 		if (cal.zOrigin!=0.0)
 			sb.append("zorigin="+cal.zOrigin+"\n");
 		if (cal.info!=null && cal.info.length()<=64 && cal.info.indexOf('=')==-1 && cal.info.indexOf('\n')==-1)
-			sb.append("info="+cal.info+"\n");
+			appendEscapedLine(sb, "info="+cal.info);
+			
+		// get invertY flag
+		if (cal.getInvertY())
+			sb.append("inverty=true\n");
+
 		if (saveName)
-			sb.append("name="+imp.getTitle()+"\n");
+			appendEscapedLine(sb, "name="+imp.getTitle());
 		sb.append((char)0);
 		return new String(sb);
 	}
-	
+
+	// Append a string to a StringBuffer with escaped special characters as needed for java.util.Properties,
+	// and add a linefeed character
+	void appendEscapedLine(StringBuffer sb, String str) {
+		for (int i=0; i<str.length(); i++) {
+			char c = str.charAt(i);
+			if (c>=0x20 && c<0x7f && c!='\\')
+				sb.append(c);
+			else if (c<=0xffff) {   //(supplementary unicode characters >0xffff unsupported)
+				sb.append("\\u");
+				sb.append(Tools.int2hex(c, 4));
+			}
+		}
+		sb.append('\n');
+	}
+
 	/** Specifies the image quality (0-100). 0 is poorest image quality,
 		highest compression, and 100 is best image quality, lowest compression. */
     public static void setJpegQuality(int quality) {
