@@ -3,12 +3,7 @@ import ij.*;
 import ij.gui.*;
 import ij.process.*;
 import ij.plugin.frame.*;
-import ij.plugin.DICOM;
-import ij.plugin.AVI_Reader;
-import ij.plugin.GIF_Reader;
-import ij.plugin.SimpleCommands;
-import ij.plugin.HyperStackConverter;
-import ij.plugin.PluginInstaller;
+import ij.plugin.*;
 import ij.text.TextWindow;
 import ij.util.Java2;
 import ij.measure.ResultsTable;
@@ -57,11 +52,17 @@ public class Opener {
 	public Opener() {
 	}
 
-	/** Displays a file open dialog box and then opens the tiff, dicom, 
-		fits, pgm, jpeg, bmp, gif, lut, roi, or text file selected by 
-		the user. Displays an error message if the selected file is not
-		in one of the supported formats. This is the method that
-		ImageJ's File/Open command uses to open files. */
+	/**
+	 * Displays a file open dialog box and then opens the tiff, dicom, 
+	 * fits, pgm, jpeg, bmp, gif, lut, roi, or text file selected by 
+	 * the user. Displays an error message if the selected file is not
+	 * in a supported format. This is the method that
+	 * ImageJ's File/Open command uses to open files.
+	 * @see ij.IJ#open()
+	 * @see ij.IJ#open(String)
+	 * @see ij.IJ#openImage()
+	 * @see ij.IJ#openImage(String)
+	*/
 	public void open() {
 		OpenDialog od = new OpenDialog("Open", "");
 		String directory = od.getDirectory();
@@ -74,53 +75,13 @@ public class Opener {
 		}
 	}
 
-	/** Displays a JFileChooser and then opens the tiff, dicom, 
-		fits, pgm, jpeg, bmp, gif, lut, roi, or text files selected by 
-		the user. Displays error messages if one or more of the selected 
-		files is not in one of the supported formats. This is the method
-		that ImageJ's File/Open command uses to open files if
-		"Open/Save Using JFileChooser" is checked in EditOptions/Misc. */
-	public void openMultiple() {
-		Java2.setSystemLookAndFeel();
-		// run JFileChooser in a separate thread to avoid possible thread deadlocks
-		try {
-			EventQueue.invokeAndWait(new Runnable() {
-				public void run() {
-					JFileChooser fc = new JFileChooser();
-					fc.setMultiSelectionEnabled(true);
-					File dir = null;
-					String sdir = OpenDialog.getDefaultDirectory();
-					if (sdir!=null)
-						dir = new File(sdir);
-					if (dir!=null)
-						fc.setCurrentDirectory(dir);
-					int returnVal = fc.showOpenDialog(IJ.getInstance());
-					if (returnVal!=JFileChooser.APPROVE_OPTION)
-						return;
-					omFiles = fc.getSelectedFiles();
-					if (omFiles.length==0) { // getSelectedFiles does not work on some JVMs
-						omFiles = new File[1];
-						omFiles[0] = fc.getSelectedFile();
-					}
-					omDirectory = fc.getCurrentDirectory().getPath()+File.separator;
-				}
-			});
-		} catch (Exception e) {}
-		if (omDirectory==null) return;
-		OpenDialog.setDefaultDirectory(omDirectory);
-		for (int i=0; i<omFiles.length; i++) {
-			String path = omDirectory + omFiles[i].getName();
-			open(path);
-			if (i==0 && Recorder.record)
-				Recorder.recordPath("open", path);
-			if (i==0 && !error)
-				Menus.addOpenRecentItem(path);
-		}
-	}
-
-	/** Opens and displays a tiff, dicom, fits, pgm, jpeg, bmp, gif, lut, 
-		roi, or text file. Displays an error message if the specified file
-		is not in one of the supported formats. */
+	/**
+	 * Opens and displays the specified tiff, dicom, fits, pgm, jpeg, 
+	 * bmp, gif, lut, roi, or text file. Displays an error message if 
+	 * the file is not in a supported format.
+	 * @see ij.IJ#open(String)
+	 * @see ij.IJ#openImage(String)
+	*/
 	public void open(String path) {
 		boolean isURL = path.indexOf("://")>0;
 		if (isURL && isText(path)) {
@@ -142,7 +103,11 @@ public class Opener {
 		if (!silentMode)
 			IJ.showStatus("Opening: " + path);
 		long start = System.currentTimeMillis();
-		ImagePlus imp = openImage(path);
+		ImagePlus imp = null;
+		if (path.endsWith(".txt"))
+			fileType = JAVA_OR_TEXT;
+		else
+			imp = openImage(path);
 		if (imp==null && isURL)
 			return;
 		if (imp!=null) {
@@ -214,6 +179,85 @@ public class Opener {
 		}
 	}
 	
+	/** Displays a JFileChooser and then opens the tiff, dicom, 
+		fits, pgm, jpeg, bmp, gif, lut, roi, or text files selected by 
+		the user. Displays error messages if one or more of the selected 
+		files is not in one of the supported formats. This is the method
+		that ImageJ's File/Open command uses to open files if
+		"Open/Save Using JFileChooser" is checked in EditOptions/Misc. */
+	public void openMultiple() {
+		Java2.setSystemLookAndFeel();
+		// run JFileChooser in a separate thread to avoid possible thread deadlocks
+		try {
+			EventQueue.invokeAndWait(new Runnable() {
+				public void run() {
+					JFileChooser fc = new JFileChooser();
+					fc.setMultiSelectionEnabled(true);
+					File dir = null;
+					String sdir = OpenDialog.getDefaultDirectory();
+					if (sdir!=null)
+						dir = new File(sdir);
+					if (dir!=null)
+						fc.setCurrentDirectory(dir);
+					int returnVal = fc.showOpenDialog(IJ.getInstance());
+					if (returnVal!=JFileChooser.APPROVE_OPTION)
+						return;
+					omFiles = fc.getSelectedFiles();
+					if (omFiles.length==0) { // getSelectedFiles does not work on some JVMs
+						omFiles = new File[1];
+						omFiles[0] = fc.getSelectedFile();
+					}
+					omDirectory = fc.getCurrentDirectory().getPath()+File.separator;
+				}
+			});
+		} catch (Exception e) {}
+		if (omDirectory==null) return;
+		OpenDialog.setDefaultDirectory(omDirectory);
+		for (int i=0; i<omFiles.length; i++) {
+			String path = omDirectory + omFiles[i].getName();
+			open(path);
+			if (i==0 && Recorder.record)
+				Recorder.recordPath("open", path);
+			if (i==0 && !error)
+				Menus.addOpenRecentItem(path);
+		}
+	}
+	
+	/**
+	 * Opens, but does not display, the specified image file
+	 * and returns an ImagePlus object object if successful,
+	 * or returns null if the file is not in a supported format
+	 * or is not found. Displays a file open dialog if 'path'
+	 * is null or an empty string.
+	 * @see ij.IJ#openImage(String)
+	 * @see ij.IJ#openImage()
+	*/
+	public ImagePlus openImage(String path) {
+		if (path==null || path.equals(""))
+			path = getPath();
+		if (path==null) return null;
+		ImagePlus img = null;
+		if (path.indexOf("://")>0)
+			img = openURL(path);
+		else
+			img = openImage(getDir(path), getName(path));
+		return img;
+	}
+	
+	/**
+	 * Open the nth image of the specified tiff stack.
+	 * @see ij.IJ#openImage(String,int)
+	*/
+	public ImagePlus openImage(String path, int n) {
+		if (path==null || path.equals(""))
+			path = getPath();
+		if (path==null) return null;
+		int type = getFileType(path);
+		if (type!=TIFF)
+			throw new IllegalArgumentException("TIFF file require");
+		return openTiff(path, n);
+	}
+
 	public static String getLoadRate(double time, ImagePlus imp) {
 		time = (System.currentTimeMillis()-time)/1000.0;
 		double mb = imp.getWidth()*imp.getHeight()*imp.getStackSize();
@@ -251,10 +295,13 @@ public class Opener {
 		return error;
 	}
 
-	/** Attempts to open the specified file as a tiff, bmp, dicom, fits,
-		pgm, gif or jpeg image. Returns an ImagePlus object if successful.
-		Modified by Gregory Jefferis to call HandleExtraFileTypes plugin if 
-		the file type is unrecognised. */
+	/**
+	 * Attempts to open the specified file as a tiff, bmp, dicom, fits,
+	 * pgm, gif or jpeg image. Returns an ImagePlus object if successful.
+	 * Modified by Gregory Jefferis to call HandleExtraFileTypes plugin if 
+	 * the file type is unrecognised.
+	 * @see ij.IJ#openImage(String)
+	*/
 	public ImagePlus openImage(String directory, String name) {
 		ImagePlus imp;
 		FileOpener.setSilentMode(silentMode);
@@ -307,8 +354,15 @@ public class Opener {
 			case ZIP:
 				return openZip(path);
 			case AVI:
-				AVI_Reader reader = (AVI_Reader)IJ.runPlugIn("ij.plugin.AVI_Reader", path);
+				AVI_Reader reader = new AVI_Reader();
+				reader.displayDialog(!IJ.macroRunning());
+				reader.run(path);
 				return reader.getImagePlus();
+			case JAVA_OR_TEXT:
+				if (name.endsWith(".txt"))
+					return openTextImage(directory,name);
+				else
+					return null;
 			case UNKNOWN: case TEXT:
 				// Call HandleExtraFileTypes plugin to see if it can handle unknown format
 				int[] wrap = new int[] {fileType};
@@ -324,32 +378,6 @@ public class Opener {
 		}
 	}
 	
-	/** Attempts to open the specified file as a tiff, bmp, dicom, fits,
-		pgm, gif or jpeg. Displays a file open dialog if 'path' is null or
-		an empty string. Returns an ImagePlus object if successful. */
-	public ImagePlus openImage(String path) {
-		if (path==null || path.equals(""))
-			path = getPath();
-		if (path==null) return null;
-		ImagePlus img = null;
-		if (path.indexOf("://")>0)
-			img = openURL(path);
-		else
-			img = openImage(getDir(path), getName(path));
-		return img;
-	}
-	
-	/** Open the nth image of the specified tiff stack. */
-	public ImagePlus openImage(String path, int n) {
-		if (path==null || path.equals(""))
-			path = getPath();
-		if (path==null) return null;
-		int type = getFileType(path);
-		if (type!=TIFF)
-			throw new IllegalArgumentException("TIFF file require");
-		return openTiff(path, n);
-	}
-
 	String getPath() {
 		OpenDialog od = new OpenDialog("Open", "");
 		String dir = od.getDirectory();
@@ -360,11 +388,24 @@ public class Opener {
 			return dir+name;
 	}
 
-	/** Attempts to open the specified url as a tiff, zip compressed tiff, 
-		dicom, gif or jpeg. Tiff file names must end in ".tif", ZIP file names 
-		must end in ".zip" and dicom file names must end in ".dcm". Returns an 
-		ImagePlus object if successful. */
+	/** Opens the specified text file as a float image. */
+	public ImagePlus openTextImage(String dir, String name) {
+		String path = dir+name;
+		TextReader tr = new TextReader();
+		ImageProcessor ip = tr.open(path);
+		return ip!=null?new ImagePlus(name,ip):null;
+	}
+
+	/**
+	 * Attempts to open the specified url as a tiff, zip compressed tiff, 
+	 * dicom, gif or jpeg. Tiff file names must end in ".tif", ZIP file names 
+	 * must end in ".zip" and dicom file names must end in ".dcm". Returns an 
+	 * ImagePlus object if successful.
+	 * @see ij.IJ#openImage(String)
+	*/
 	public ImagePlus openURL(String url) {
+		url = updateUrl(url);
+		if (IJ.debugMode) IJ.log("OpenURL: "+url);
 		ImagePlus imp = openCachedImage(url);
 		if (imp!=null)
 			return imp;
@@ -415,8 +456,18 @@ public class Opener {
 		} 
 	}
 	
+	/** Can't open imagej.nih.gov URLs due to encryption so redirect to mirror.nih.net. */
+	public static String updateUrl(String url) {
+		if (url==null || !url.contains("nih.gov"))
+			return url;
+		url = url.replace("imagej.nih.gov/ij", "mirror.imagej.net");
+		url = url.replace("rsb.info.nih.gov/ij", "mirror.imagej.net");
+		url = url.replace("rsbweb.nih.gov/ij", "mirror.imagej.net");
+		return url;
+	}
+	
 	private ImagePlus openCachedImage(String url) {
-		if (url==null || !url.contains("ij/images"))
+		if (url==null || !url.contains("/images"))
 			return null;
 		String ijDir = IJ.getDirectory("imagej");
 		if (ijDir==null)
@@ -435,6 +486,10 @@ public class Opener {
 		if (url.endsWith(".pdf")||url.endsWith(".zip"))
 			return;
 		String text = IJ.openUrlAsString(url);
+		if (text!=null && text.startsWith("<Error: ")) {
+			IJ.error("Open Text URL", text);
+			return;
+		}
 		String name = url.substring(7);
 		int index = name.lastIndexOf("/");
 		int len = name.length();
@@ -558,12 +613,17 @@ public class Opener {
 	}
 
 	ImagePlus openPngUsingURL(String title, URL url) {
-		if (url==null) return null;
+		if (url==null)
+			return null;
+		//System.setProperty("jsse.enableSNIExtension","false");
 		Image img = null;
 		try {
-			img = ImageIO.read(url);
+			InputStream in = url.openStream();
+			img = ImageIO.read(in);
+		} catch (FileNotFoundException e) {
+			IJ.error("Open PNG Using URL", ""+e);
 		} catch (IOException e) {
-			IJ.log(""+e);
+			IJ.handleException(e);
 		}
 		if (img!=null) {
 			ImagePlus imp = new ImagePlus(title, img);
@@ -622,10 +682,10 @@ public class Opener {
 		return imp;
 	}
 
-	/** If the specified image is grayscale, convert it to 8-bits. */
+	/** Converts the specified RGB image to 8-bits if the 3 channels are identical. */
 	public static void convertGrayJpegTo8Bits(ImagePlus imp) {
 		ImageProcessor ip = imp.getProcessor();
-		if (ip.isGrayscale()) {
+		if (ip.getBitDepth()==24 && ip.isGrayscale()) {
 			IJ.showStatus("Converting to 8-bit grayscale");
 			new ImageConverter(imp).convertToGray8();
 		}
@@ -790,8 +850,9 @@ public class Opener {
 		TiffDecoder td = new TiffDecoder(getDir(path), getName(path));
 		if (IJ.debugMode) td.enableDebugging();
 		FileInfo[] info=null;
-		try {info = td.getTiffInfo();}
-		catch (IOException e) {
+		try {
+			info = td.getTiffInfo();
+		} catch (IOException e) {
 			String msg = e.getMessage();
 			if (msg==null||msg.equals("")) msg = ""+e;
 			IJ.error("Open TIFF", msg);
@@ -818,6 +879,18 @@ public class Opener {
 		return fo.open(false);
 	}
 
+	/** Returns the FileInfo of the specified TIFF file. */
+	public static FileInfo[] getTiffFileInfo(String path) {
+		Opener o = new Opener();
+		TiffDecoder td = new TiffDecoder(o.getDir(path), o.getName(path));
+		if (IJ.debugMode) td.enableDebugging();
+		try {
+			return td.getTiffInfo();
+		} catch (IOException e) {
+			return null;
+		}
+	}
+
 	/** Attempts to open the specified inputStream as a
 		TIFF, returning an ImagePlus object if successful. */
 	public ImagePlus openTiff(InputStream in, String name) {
@@ -1029,6 +1102,13 @@ public class Opener {
 		return null;
 	}
 
+	/** Opens a lookup table (LUT) and returns it as a LUT object, or returns null if there is an error.
+	 * @see ij.ImagePlus#setLut
+	*/
+	public static LUT openLut(String filePathOrUrl) {
+		return LutLoader.openLut(filePathOrUrl);
+	}
+
 	/** Opens a tab or comma delimited text file in the Results window. */
 	public static void openResultsTable(String path) {
 		try {
