@@ -15,7 +15,6 @@ import java.io.PrintWriter;
 /** This is the recursive descent parser/interpreter for the ImageJ macro language. */
 public class Interpreter implements MacroConstants {
 
-	public static final int NONE=0, STEP=1, TRACE=2, FAST_TRACE=3, RUN=4, RUN_TO_CARET=5;  // debugging modes
 	static final int STACK_SIZE=1000;
 	static final int MAX_ARGS=20;
 
@@ -52,14 +51,18 @@ public class Interpreter implements MacroConstants {
 	double[] rgbWeights;
 	boolean inPrint;
 	static String additionalFunctions;
-	Editor editor;
-	int debugMode = NONE;
+	Debugger debugger;
+	int debugMode = Debugger.NOT_DEBUGGING;
 	boolean showDebugFunctions;
 	static boolean showVariables;
 	boolean wasError;
 	ImagePlus batchMacroImage;
 	boolean inLoop;
+	int loopDepth;
 	static boolean tempShowMode;
+	boolean waitingForUser;
+	int selectCount;
+
 	
 	static TextWindow arrayWindow;
 	int inspectStkIndex = -1;
@@ -217,8 +220,8 @@ public class Interpreter implements MacroConstants {
 
 	final void doStatement() {
 		getToken();
-		if (debugMode!=NONE && editor!=null && !done && token!=';' && token!=FUNCTION)
-			editor.debug(this, debugMode);
+		if (debugMode!=Debugger.NOT_DEBUGGING && debugger!=null && !done && token!=';' && token!=FUNCTION)
+			debugger.debug(this, debugMode);
 		switch (token) {
 			case VAR:
 				doVar();
@@ -482,6 +485,7 @@ public class Interpreter implements MacroConstants {
 	void doFor() {
 		boolean saveLooseSyntax = looseSyntax;
 		looseSyntax = false;
+		loopDepth++;
 		inLoop = true;
 		getToken();
 		if (token!='(')
@@ -544,11 +548,14 @@ public class Interpreter implements MacroConstants {
 			pc = condPC;
 		}
 		looseSyntax = saveLooseSyntax;
-		inLoop = false;
+		loopDepth--;
+		if (loopDepth==0)
+			inLoop = false;
 	}
 
 	void doWhile() {
 		looseSyntax = false;
+		loopDepth++;
 		inLoop = true;
 		int savePC = pc;
 		boolean isTrue;
@@ -569,7 +576,9 @@ public class Interpreter implements MacroConstants {
 			} else
 				skipStatement();
 		} while (isTrue && !done);
-		inLoop = false;
+		loopDepth--;
+		if (loopDepth==0)
+			inLoop = false;
 	}
 
 	void doDo() {
@@ -606,7 +615,7 @@ public class Interpreter implements MacroConstants {
 		getToken();
 		switch (token) {
 			case PREDEFINED_FUNCTION: case USER_FUNCTION: case VAR:
-			case WORD: case '(': case PLUS_PLUS: case RETURN:
+			case WORD: case '(': case PLUS_PLUS: case RETURN: case ARRAY_FUNCTION:
 			case NUMERIC_FUNCTION: case STRING_FUNCTION:
 				skipSimpleStatement();
 				break;
@@ -1732,6 +1741,11 @@ public class Interpreter implements MacroConstants {
 			if (rt!=null && rt.size()>0)
 				rt.show("Results");
 		}
+		if (IJ.isMacOSX() && selectCount>0 && debugger==null) {
+			Frame frame = WindowManager.getFrontWindow();
+			if (frame!=null && (frame instanceof ImageWindow))
+				ImageWindow.setImageJMenuBar((ImageWindow)frame);
+		}
 	}
 	
 	/** Aborts currently running macro. */
@@ -1775,10 +1789,6 @@ public class Interpreter implements MacroConstants {
 		instance = i;
 	}
 
-	//public boolean  inLoop() {
-	//	return !looseSyntax;
-	//}
-
 	static void setBatchMode(boolean b) {
 		batchMode = b;
 		if (b==false)
@@ -1874,16 +1884,19 @@ public class Interpreter implements MacroConstants {
 		return interp!=null && isBatchMode() && interp.func.roiManager!=null;
 	}
 	
-	public void setEditor(Editor ed) {
-		if (ed!=null&&editor==null)
-			ed.fixLineEndings();
-		editor = ed;
-		if (ed!=null)
-			debugMode = STEP;
+	public void setDebugger(Debugger debugger) {
+		this.debugger = debugger;
+		if (debugger!=null)
+			debugMode = Debugger.STEP;
 		else
-			debugMode = NONE;
+			debugMode = Debugger.NOT_DEBUGGING;
 	}
 	
+	// Returns the Debugger (editor), if any, associated with this macro. */
+	public Debugger getDebugger() {
+		return debugger;
+	}
+
 	public void setDebugMode(int mode) {
 		debugMode = mode;
 	}
@@ -1903,7 +1916,7 @@ public class Interpreter implements MacroConstants {
 				ImagePlus imp = WindowManager.getCurrentImage();
 				if (imp!=null) title = imp.getTitle();
 			}
-			if (debugMode==STEP) System.gc();
+			if (debugMode==Debugger.STEP) System.gc();
 			variables[0] = "Memory\t" + IJ.freeMemory();
 			variables[1] = "nImages()\t" + nImages;
 			variables[2] = "getTitle()\t" + (title!=null?"\""+title+"\"":"");
@@ -1924,11 +1937,6 @@ public class Interpreter implements MacroConstants {
 		return done;
 	}
 
-	// Returns the Editor, if any, associated with this macro. */
-	public Editor getEditor() {
-		return editor;
-	}
-
 	// Returns 'true' if this macro generated an error and was aborted. */
 	public boolean wasError() {
 		return wasError;
@@ -2047,8 +2055,8 @@ public class Interpreter implements MacroConstants {
 				}
 				txtPanel.append(newText);
 				txtPanel.scrollToTop();
-				if (editor!=null)
-					editor.toFront();
+				if (debugger!=null && (debugger instanceof Window))
+					((Window)debugger).toFront();
 				//n__  scroll position should not change during single-stepping
 			}
 		}
@@ -2081,6 +2089,19 @@ public class Interpreter implements MacroConstants {
 	static void setTempShowMode(boolean mode) {
 		tempShowMode = mode;
 	}
+	
+	private static Interpreter lastInterp;
+	
+	public static boolean nonBatchMacroRunning() {
+		Interpreter interp = getInstance();
+		if (interp==null)
+			return false;
+		int count =  interp.selectCount;
+		if (interp==lastInterp)
+			interp.selectCount++;
+		lastInterp = interp;
+		return !interp.waitingForUser && interp.debugger==null && count>0 && !isBatchMode();
+	}
 
 } // class Interpreter
 
