@@ -5,7 +5,7 @@ import ij.text.*;
 import ij.process.*;
 import ij.gui.Roi;
 import ij.util.Tools;
-import ij.io.SaveDialog;
+import ij.io.*;
 import java.awt.*;
 import java.text.*;
 import java.util.*;
@@ -26,6 +26,7 @@ public class ResultsTable implements Cloneable {
 	public static final int COLUMN_IN_USE = -2;
 	public static final int TABLE_FULL = -3; // no longer used
 	public static final short AUTO_FORMAT = Short.MIN_VALUE;
+	private static final char commaSubstitute = 0x08B3;
 	
 	public static final int AREA=0, MEAN=1, STD_DEV=2, MODE=3, MIN=4, MAX=5,
 		X_CENTROID=6, Y_CENTROID=7, X_CENTER_OF_MASS=8, Y_CENTER_OF_MASS=9,
@@ -34,11 +35,11 @@ public class ResultsTable implements Cloneable {
 		INTEGRATED_DENSITY=20, MEDIAN=21, SKEWNESS=22, KURTOSIS=23, 
 		AREA_FRACTION=24, RAW_INTEGRATED_DENSITY=25, CHANNEL=26, SLICE=27, FRAME=28, 
 		FERET_X=29, FERET_Y=30, FERET_ANGLE=31, MIN_FERET=32, ASPECT_RATIO=33,
-		ROUNDNESS=34, SOLIDITY=35, LAST_HEADING=35;
+		ROUNDNESS=34, SOLIDITY=35, MIN_THRESHOLD=36, MAX_THRESHOLD=37, LAST_HEADING=37;
 	private static final String[] defaultHeadings = {"Area","Mean","StdDev","Mode","Min","Max",
 		"X","Y","XM","YM","Perim.","BX","BY","Width","Height","Major","Minor","Angle",
 		"Circ.", "Feret", "IntDen", "Median","Skew","Kurt", "%Area", "RawIntDen", "Ch", "Slice", "Frame", 
-		 "FeretX", "FeretY", "FeretAngle", "MinFeret", "AR", "Round", "Solidity"};
+		 "FeretX", "FeretY", "FeretAngle", "MinFeret", "AR", "Round", "Solidity", "MinThr", "MaxThr"};
 
 	private int maxRows = 100; // will be increased as needed
 	private int maxColumns = MAX_COLUMNS; // will be increased as needed
@@ -56,6 +57,8 @@ public class ResultsTable implements Cloneable {
 	private boolean headingSet; 
 	private boolean showRowNumbers = true;
 	private Hashtable stringColumns;
+	private boolean NaNEmptyCells;
+	private boolean quoteCommas;
 
 
 	/** Constructs an empty ResultsTable with the counter=0, no columns
@@ -96,6 +99,8 @@ public class ResultsTable implements Cloneable {
 			for (int i=0; i<=lastColumn; i++) {
 				if (columns[i]!=null) {
 					double[] tmp = new double[maxRows*2];
+					if (NaNEmptyCells)
+						Arrays.fill(tmp, maxRows, tmp.length, Double.NaN);
 					System.arraycopy(columns[i], 0, tmp, 0, maxRows);
 					columns[i] = tmp;
 				}
@@ -146,13 +151,15 @@ public class ResultsTable implements Cloneable {
 			throw new IllegalArgumentException("Counter==0");
 		if (columns[column]==null) {
 			columns[column] = new double[maxRows];
+			if (NaNEmptyCells)
+				Arrays.fill(columns[column], Double.NaN);
 			if (headings[column]==null)
 				headings[column] = "---";
 			if (column>lastColumn) lastColumn = column;
 		}
 		columns[column][counter-1] = value;
 		if (counter<25) {
-			if ((int)value!=value)
+			if ((int)value!=value && !Double.isNaN(value))
 				decimalPlaces[column] = (short)precision;
 		}
 	}
@@ -192,7 +199,10 @@ public class ResultsTable implements Cloneable {
 		addLabel(rowLabelHeading, label);
 	}
 
-	/** Adds a label to the beginning of the current row. Counter must be >0. */
+	/**
+	 * @deprecated
+	 * Replaced by setValue(String,int,String)
+	*/
 	public void addLabel(String columnHeading, String label) {
 		if (counter==0)
 			throw new IllegalArgumentException("Counter==0");
@@ -315,6 +325,8 @@ public class ResultsTable implements Cloneable {
 		for(int i=0; i<headings.length; i++) {
 			if (headings[i]==null) {
 				columns[i] = new double[maxRows];
+				if (NaNEmptyCells)
+					Arrays.fill(columns[i], Double.NaN);
 				headings[i] = heading;
 				if (i>lastColumn) lastColumn = i;
 				return i;
@@ -325,6 +337,8 @@ public class ResultsTable implements Cloneable {
 		addColumns();
 		lastColumn++;
 		columns[lastColumn] = new double[maxRows];
+		if (NaNEmptyCells)
+			Arrays.fill(columns[lastColumn], Double.NaN);
 		headings[lastColumn] = heading;
 		return lastColumn;
 	}
@@ -371,8 +385,15 @@ public class ResultsTable implements Cloneable {
 		if (row<0 || row>=size())
 			throw new IllegalArgumentException("Row out of range");
 		int col = getColumnIndex(column);
-		if (col==COLUMN_NOT_FOUND)
-			throw new IllegalArgumentException("\""+column+"\" column not found");
+		if (col==COLUMN_NOT_FOUND) {
+			String label = null;
+			if ("Label".equals(column))
+				label = getLabel(row);
+			if (label!=null)
+				return label;
+			else
+				throw new IllegalArgumentException("\""+column+"\" column not found");
+		}
 		return getStringValue(col, row);
 	}
 
@@ -428,9 +449,15 @@ public class ResultsTable implements Cloneable {
 		}
 		if (columns[column]==null) {
 			columns[column] = new double[maxRows];
+			if (NaNEmptyCells)
+				Arrays.fill(columns[column], Double.NaN);
 			if (column>lastColumn) lastColumn = column;
 		}
 		columns[column][row] = value;
+		if (counter<25) {
+			if ((int)value!=value && !Double.isNaN(value))
+				decimalPlaces[column] = (short)precision;
+		}
 	}
 
 	/** Sets the string value of the given column and row, where
@@ -548,7 +575,12 @@ public class ResultsTable implements Cloneable {
 		}
 		for (int i=0; i<=lastColumn; i++) {
 			if (columns[i]!=null) {
-				sb.append(getValueAsString(i,row));
+				String value = getValueAsString(i,row);
+				if (quoteCommas) {
+					if (value.contains(","))
+						value = "\""+value+"\"";
+				}
+				sb.append(value);
 				if (i!=lastColumn)
 					sb.append(delimiter);
 			}
@@ -592,14 +624,17 @@ public class ResultsTable implements Cloneable {
 		
 	/**
 	* @deprecated
-	* replaced by addValue(String,double) and setValue(String,int,double)
+	* Replaced by addValue(String,double) and setValue(String,int,double)
 	*/
 	public void setHeading(int column, String heading) {
 		if ((column<0) || (column>=headings.length))
 			throw new IllegalArgumentException("Column out of range: "+column);
 		headings[column] = heading;
-		if (columns[column]==null)
+		if (columns[column]==null) {
 			columns[column] = new double[maxRows];
+			if (NaNEmptyCells)
+				Arrays.fill(columns[column], Double.NaN);
+		}
 		if (column>lastColumn) lastColumn = column;
 		headingSet = true;
 	}
@@ -613,12 +648,11 @@ public class ResultsTable implements Cloneable {
 	/** Sets the decimal places (digits to the right of decimal point)
 		that are used when this table is displayed. */
 	public synchronized void setPrecision(int precision) {
+		if (precision>9) precision=9;
 		this.precision = (short)precision;
-		if (this==Analyzer.getResultsTable()) {
-			for (int i=0; i<decimalPlaces.length; i++) {
-				if (!(decimalPlaces[i]==AUTO_FORMAT||decimalPlaces[i]==0))
-					decimalPlaces[i] = (short)precision;
-			}
+		for (int i=0; i<decimalPlaces.length; i++) {
+			if (decimalPlaces[i]!=AUTO_FORMAT)
+				decimalPlaces[i] = (short)precision;
 		}
 	}
 	
@@ -628,6 +662,11 @@ public class ResultsTable implements Cloneable {
 		decimalPlaces[column] = (short)digits;
 	}
 
+	/** Set 'true' to initially fill data arrays with NaNs instead of zeros. */
+	public void setNaNEmptyCells(boolean NaNEmptyCells) {
+		this.NaNEmptyCells = NaNEmptyCells;
+	}
+
 	public void showRowNumbers(boolean showNumbers) {
 		showRowNumbers = showNumbers;
 	}
@@ -652,6 +691,8 @@ public class ResultsTable implements Cloneable {
 			} else
 				decimalPlaces = 3;
 			if (sf==null) {
+				if (dfs==null)
+					dfs = new DecimalFormatSymbols(Locale.US);
 				sf = new DecimalFormat[10];
 				sf[1] = new DecimalFormat("0.0E0",dfs);
 				sf[2] = new DecimalFormat("0.00E0",dfs);
@@ -779,9 +820,12 @@ public class ResultsTable implements Cloneable {
 			TextWindow win;
 			if (frame!=null && frame instanceof TextWindow) {
 				win = (TextWindow)frame;
-				win.toFront();
+				if (windowTitle==null || !windowTitle.startsWith("Counts_"))
+					win.toFront();
 			} else {
-				int width = getLastColumn()<=1?250:400;
+				int width = getLastColumn()<=0?250:400;
+				if (showRowNumbers)
+					width += 50;
 				win = new TextWindow(windowTitle, "", width, 300);
 				cloneNeeded = true;
 			}
@@ -823,6 +867,8 @@ public class ResultsTable implements Cloneable {
 			//IJ.log(i+"  "+rt2.getColumn(i)+"  "+columns[i]+"  "+rt2.getColumnHeading(i)+"  "+getColumnHeading(i));
 			if (rt2.getColumn(i)!=null && columns[i]==null) {
 				columns[i] = new double[maxRows];
+				if (NaNEmptyCells)
+					Arrays.fill(columns[i], Double.NaN);
 				headings[i] = rt2.getColumnHeading(i);
 				if (i>lastColumn) lastColumn = i;
 			} else if (rt2.getColumn(i)==null && columns[i]!=null && !keep[i])
@@ -866,16 +912,31 @@ public class ResultsTable implements Cloneable {
 	*/
 	public static ResultsTable open(String path) throws IOException {
 		final String lineSeparator =  "\n";
-		final String cellSeparator =  ",\t";
-		String text =IJ.openAsString(path);
+		if (path==null || path.equals("")) {
+			OpenDialog od = new OpenDialog("Open Table", "");
+			String dir = od.getDirectory();
+			String name = od.getFileName();
+			if (name==null)
+				return null;
+			path = dir+name;
+		}
+		String text = IJ.openAsString(path);
 		if (text==null)
 			return null;
 		if (text.startsWith("Error:"))
 			throw new IOException("Error opening "+path);
-		String[] lines = Tools.split(text, lineSeparator);
+		boolean csv = path.endsWith(".csv") || path.endsWith(".CSV");
+		String cellSeparator =  csv?",":"\t";
+		boolean commasReplaced = false;
+		if (csv && text.contains("\"")) {
+			text = replaceQuotedCommas(text);
+			commasReplaced = true;
+		}
+		String commaSubstitute2 = ""+commaSubstitute;
+		String[] lines = text.split(lineSeparator);
 		if (lines.length==0)
 			throw new IOException("Table is empty or invalid");
-		String[] headings = Tools.split(lines[0], cellSeparator);
+		String[] headings = lines[0].split(cellSeparator);
 		if (headings.length==1)
 			throw new IOException("This is not a tab or comma delimited text file.");
 		int numbersInHeadings = 0;
@@ -889,13 +950,18 @@ public class ResultsTable implements Cloneable {
 				headings[i] = "C"+(i+1);
 		}
 		int firstColumn = headings[0].equals(" ")?1:0;
-		for (int i=0; i<headings.length; i++)
+		for (int i=0; i<headings.length; i++) {
 			headings[i] = headings[i].trim();
+			if (commasReplaced) {
+				if (headings[i].startsWith("\"") && headings[i].endsWith("\""))
+					headings[i] = headings[i].substring(1, headings[i].length()-1);
+			}
+		}
 		int firstRow = allNumericHeadings?0:1;
 		boolean labels = firstColumn==1 && headings[1].equals("Label");
 		int type=getTableType(path, lines, firstRow, cellSeparator);
-		if (!labels && (type==1||type==2))
-			labels = true;
+		//if (!labels && (type==1||type==2))
+		//	labels = true;
 		int labelsIndex = (type==2)?0:1;
 		if (lines[0].startsWith("\t")) {
 			String[] headings2 = new String[headings.length+1];
@@ -908,15 +974,21 @@ public class ResultsTable implements Cloneable {
 		ResultsTable rt = new ResultsTable();
 		for (int i=firstRow; i<lines.length; i++) {
 			rt.incrementCounter();
-			String[] items=Tools.split(lines[i], cellSeparator);
+			String[] items=lines[i].split(cellSeparator);
 			for (int j=firstColumn; j<items.length; j++) {
 				if (j==labelsIndex&&labels)
 					rt.addLabel(headings[labelsIndex], items[labelsIndex]);
 				else if (j<headings.length) {
 					double value = Tools.parseDouble(items[j]);
-					if (Double.isNaN(value))
-						rt.addValue(headings[j], items[j]);
-					else
+					if (Double.isNaN(value)) {
+						String item = items[j];
+						if (commasReplaced) {
+							item = item.replaceAll(commaSubstitute2, ",");
+							if (item.startsWith("\"") && item.endsWith("\""))
+								item = item.substring(1, item.length()-1);
+						}
+						rt.addValue(headings[j], item);
+					} else
 						rt.addValue(headings[j], value);
 				}
 			}
@@ -926,7 +998,7 @@ public class ResultsTable implements Cloneable {
 	
 	private static int getTableType(String path, String[] lines, int firstRow, String cellSeparator) {
 		if (lines.length<2) return 0;
-		String[] items=Tools.split(lines[1], cellSeparator);
+		String[] items=lines[1].split(cellSeparator);
 		int nonNumericCount = 0;
 		int nonNumericIndex = 0;
 		for (int i=0; i<items.length; i++) {
@@ -945,19 +1017,44 @@ public class ResultsTable implements Cloneable {
 		return 3;
 	}
 	
+	private static String replaceQuotedCommas(String text) {
+		char[] c = text.toCharArray();
+		boolean inQuotes = false;
+		for (int i=0; i<c.length; i++) {
+			if (c[i]=='"')
+				inQuotes = !inQuotes;
+			if (inQuotes && c[i]==',')
+				c[i] = commaSubstitute;
+		}
+		return new String(c);
+	}
+	
 	/** Saves this ResultsTable as a tab or comma delimited text file. The table
 	     is saved as a CSV (comma-separated values) file if 'path' ends with ".csv".
 	     Displays a file save dialog if 'path' is empty or null. Does nothing if the
-	     table is empty. */
+	     table is empty. Displays an error message and returns 'false' if there is
+	     an error. */
+	public boolean save(String path) {
+		try {
+			saveAs(path);
+			return true;
+		} catch (IOException e) {
+			delimiter = '\t';
+			IJ.error("Save As>Results", ""+"Error saving results:\n   "+e.getMessage());
+			return false;
+		}
+	}
+
 	public void saveAs(String path) throws IOException {
 		if (size()==0 && lastColumn<0) return;
 		if (path==null || path.equals("")) {
-			SaveDialog sd = new SaveDialog("Save Results", "Results", Prefs.get("options.ext", ".xls"));
+			SaveDialog sd = new SaveDialog("Save Results", "Results", Prefs.defaultResultsExtension());
 			String file = sd.getFileName();
 			if (file==null) return;
 			path = sd.getDirectory() + file;
 		}
-		delimiter = path.endsWith(".csv")?',':'\t';
+		boolean csv = path.endsWith(".csv") || path.endsWith(".CSV");
+		delimiter = csv?',':'\t';
 		PrintWriter pw = null;
 		FileOutputStream fos = new FileOutputStream(path);
 		BufferedOutputStream bos = new BufferedOutputStream(fos);
@@ -969,8 +1066,10 @@ public class ResultsTable implements Cloneable {
 			String headings = getColumnHeadings();
 			pw.println(headings);
 		}
+		quoteCommas = csv?true:false;
 		for (int i=0; i<size(); i++)
 			pw.println(getRowAsString(i));
+		quoteCommas = false;
 		showRowNumbers = saveShowRowNumbers;
 		pw.close();
 		delimiter = '\t';
