@@ -90,6 +90,8 @@ import javax.imageio.ImageIO;
  *		- can read AVI-2 files with blank frames into a virtual stack
  *	 2013-10-29
  *		- can read MJPG files where the frames don't have the same pixel number as the overall video
+ *	 2015-09-28
+ *		- reads most ImageJ AVI1 files with size>4 GB (incorrectly written by ImageJ versions before 1.50b)
  *
  * The AVI format looks like this:
  * RIFF AVI					RIFF HEADER, AVI CHUNK					
@@ -178,8 +180,9 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	private final static int   PNG_COMPRESSION2	 = 0x20474e50; //'PNG ' PNG compression of individual frames
 	private final static int   PNG_COMPRESSION3	 = 0x05;	   //BI_PNG PNG compression of individual frames
 
-	private final static int   BITMASK24 = 0x10000;		//for 24-bit (in contrast to 8, 16,... not a bitmask)
-	private final static long  SIZE_MASK = 0xffffffffL; //for conversion of sizes from unsigned int to long
+	private final static int   BITMASK24 = 0x10000;			   //for 24-bit (in contrast to 8, 16,... not a bitmask)
+	private final static long  SIZE_MASK = 0xffffffffL;		   //for conversion of sizes from unsigned int to long
+	private final static long  FOUR_GB	 = 0x100000000L;	   //2^32; above this size of data AVI 1 has a problem for sure
 
 	// flags from AVI chunk header 
 	private final static int   AVIF_HASINDEX	 = 0x00000010;	// Index at end of file?
@@ -193,7 +196,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	//static versions of dialog parameters that will be remembered
 	private static boolean	   staticConvertToGray;
 	private static boolean	   staticFlipVertical;
-	private static boolean	   staticIsVirtual;
+	private static boolean	   staticIsVirtual = true;
 	//dialog parameters
 	private int				   firstFrame = 1;		//the first frame to read
 	private int				   lastFrame = 0;		//the last frame to read; 0 means 'read all'
@@ -218,6 +221,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	private	 int			   lastFrameToRead = Integer.MAX_VALUE;
 	private	 int			   totalFramesFromIndex;//number of frames from 'AVI 2.0' indices
 	private	 boolean		   indexForCountingOnly;//don't read the index, only count int totalFramesFromIndex how many entries
+	private	 boolean		   isOversizedAvi1;		//AVI-1 file > 4GB
 	//derived from BitMapInfo
 	private	 int			   dataCompression;		//data compression type used
 	private	 boolean		   isPlanarFormat;		//I420 & YV12 formats: y frame, then u,v frames
@@ -233,6 +237,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	private	 boolean		   verbose = IJ.debugMode;
 	private	 long			   startTime;
 	private	 boolean		   aborting;
+	private	 boolean		   displayDialog = true;
 
 	//From AVI Header Chunk
 	private	 int			   dwMicroSecPerFrame;
@@ -295,7 +300,8 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			error(exceptionMessage(e));
 			return;
 		}
-		if (!showDialog(fileName)) return;							//ask for parameters
+		if (displayDialog && !showDialog(fileName))
+			return;	 //ask for parameters
 		try {
 			ImageStack stack = makeStack(path, firstFrame, lastFrame, isVirtual, convertToGray, flipVertical);	//read data
 		} catch (Exception e) {
@@ -490,6 +496,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 
 	/** Read into a (virtual) stack */
 	private void readAVI(String path) throws Exception, IOException {
+
 		if (!headerOK)							// we have not read the header yet?
 			openAndReadHeader(path);
 		startTime += System.currentTimeMillis();// taking previously elapsed time into account
@@ -508,11 +515,23 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 				raFile.seek(indexPosition);
 				nextPosition = findFourccAndRead(FOURCC_indx, false, indexPositionEnd, false);
 			}
-			if (hasIndex && frameInfos.size()==0) {	 // got nothing from indx, attempt to read AVI 1 index 'idx1'
+			if (hasIndex && (frameInfos==null ||frameInfos.size()==0)) { // got nothing from indx, attempt to read AVI 1 index 'idx1'
 				raFile.seek(headerPositionEnd);
 				moviPosition = findFourccAndSkip(FOURCC_movi, true, fileSize);	// go behind the 'movi' list
-				if (moviPosition>0)
+				if (moviPosition<0)
+					throw new Exception("AVI File has no movie data");
+				long positionBehindMovie = raFile.getFilePointer();
+				while (positionBehindMovie < fileSize-8) {
+					if (verbose)
+						IJ.log("searching for 'idx1' at 0x"+Long.toHexString(positionBehindMovie));
+					raFile.seek(positionBehindMovie);
+					if (positionBehindMovie > FOUR_GB)
+						isOversizedAvi1 = true;
 					nextPosition = findFourccAndRead(FOURCC_idx1, false, fileSize, false);
+					if (nextPosition >= 0)		//AVI-1 index 'idx1' found
+						break;
+					positionBehindMovie += FOUR_GB;	 //maybe position was wrong because it was a 32-bit number, but > 4GB?
+				}
 			}
 			if (verbose)
 				IJ.log("'frameInfos' has "+frameInfos.size()+" entries");
@@ -562,12 +581,13 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			IJ.log("File header: RIFF type='"+fourccString(riffType)+"' (should be 'AVI ')");
 		if (riffType != FOURCC_AVI)
 			throw new Exception("Not an AVI file.");
+
 		findFourccAndRead(FOURCC_hdrl, true, fileSize, true);
 		startTime -= System.currentTimeMillis(); //becomes minus elapsed Time
 		headerOK = true;
 	}
 
-	/** Read AVIX for large files (sequential reading frame-by-frame beyond the first Chunk) **/
+	/** Read AVIX chunks following the first RIFF AVI for large files (sequential reading frame-by-frame beyond the first chunk) **/
 	private void readAVIX(long endPosition) throws Exception, IOException {
 		if (verbose)
 			IJ.log("Trying to read AVIX"+timeString());
@@ -596,15 +616,19 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			}
 			long size = readInt() & SIZE_MASK;
 			nextPos = raFile.getFilePointer() + size;
+
+			if (nextPos>fileSize || nextPos>endPosition) {
+				IJ.log("AVI File Error: '"+fourccString(type)+"' @ 0x"+Long.toHexString(raFile.getFilePointer()-8)+" has invalid length");
+				return -1;
+			}
 			if (isList && type == FOURCC_LIST)
 				type = readInt();
 			if (verbose)
-				IJ.log("Searching for '"+fourccString(fourcc)+"', found "+fourccString(type)+
-						"' "+posSizeString(nextPos-size, size));
+				IJ.log("Search for '"+fourccString(fourcc)+"', found "+fourccString(type)+"' data "+posSizeString(nextPos-size, size));
 			if (type==fourcc) {
 				contentOk = readContents(fourcc, nextPos);
 			} else if (verbose)
-				IJ.log("Discarded '"+fourccString(type)+"': Contents does not fit");
+				IJ.log("'"+fourccString(type)+"', ignored");
 			raFile.seek(nextPos);
 			if (contentOk)
 				return nextPos;			//found and read, breaks the loop
@@ -628,7 +652,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 				type = readInt();
 			if (verbose)
 				IJ.log("Searching for (to skip) '"+fourccString(fourcc)+"', found "+fourccString(type)+
-						"' "+posSizeString(chunkPos, size));
+						"' data "+posSizeString(chunkPos, size));
 			raFile.seek(nextPos);
 			if (type == fourcc)
 				return chunkPos;		//found and skipped, breaks the loop
@@ -759,12 +783,11 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		int nEntriesInUse = readInt();
 		int dwChunkId = readInt();
 		long qwBaseOffset = readLong();
-		readInt();	// dwReserved3
+		readInt();	// 3rd dwReserved (first two dwreserved are qwBaseOffset!)
 		if (verbose) {
 			String bIndexString = bIndexType == AVI_INDEX_OF_CHUNKS ? ": AVI_INDEX_OF_CHUNKS" :
 					bIndexType == AVI_INDEX_OF_INDEXES ? ": AVI_INDEX_OF_INDEXES" : ": UNSUPPORTED";
 			IJ.log("AVI 2 INDEX:");
-			if (indexForCountingOnly) IJ.log("<just counting frames, not interpreting index now>");
 			IJ.log("   wLongsPerEntry=" + wLongsPerEntry);
 			IJ.log("   bIndexSubType=" + bIndexSubType);
 			IJ.log("   bIndexType=" + bIndexType + bIndexString);
@@ -775,21 +798,23 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		}
 		if (bIndexType == AVI_INDEX_OF_INDEXES) {		// 'indx' points to other indices
 			if (wLongsPerEntry != 4) return;			//badly formed index, ignore it
-			for (int i=0;i<nEntriesInUse;i++) {
+			for (int i=0;i<nEntriesInUse;i++) {			//read all entries (each pointing to an ix00 index)
 				long qwOffset = readLong();
 				int dwSize = readInt();
-				int dwDuration = readInt(); //ignored; not trustworthy anyhow
-				if (verbose) {
-				 //		IJ.log("qwOffset:"+qwOffset);
-					IJ.log("   index data '" +fourccString(dwChunkId)+"' "+posSizeString(qwOffset,dwSize)+timeString());
-				}
-				long temp= raFile.getFilePointer();
-				raFile.seek(qwOffset);
+				int dwDuration = readInt();				//number of frames in ix00; ignored: not always trustworthy
+				if (verbose)
+					IJ.log("   indx entry: '" +fourccString(dwChunkId)+"' incl header "+posSizeString(qwOffset,dwSize)+timeString());
+				long nextIndxEntryPointer = raFile.getFilePointer();
+				raFile.seek(qwOffset);					//qwOffset & dwSize here include chunk header of ix00
 				findFourccAndRead(FOURCC_ix00, false, qwOffset+dwSize, true);
-				raFile.seek(temp);
+				raFile.seek(nextIndxEntryPointer);
 				if (frameNumber>lastFrameToRead) break;
 			}
 		} else if (bIndexType == AVI_INDEX_OF_CHUNKS) {
+			if (verbose) {
+				IJ.log("readAvi2Index frameNumber="+frameNumber+" firstFrame="+firstFrame);
+				if (indexForCountingOnly) IJ.log("<just counting frames, not interpreting index now>");
+			}
 			if (wLongsPerEntry != 2) return;				//badly formed index, ignore it
 			if (dwChunkId != type0xdb && dwChunkId != type0xdc) { //not the stream we search for? (should not happen)
 				if (verbose)
@@ -808,8 +833,8 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 				if (isVirtual) IJ.showProgress((double)frameNumber/lastFrameToRead);
 				if (frameNumber >= firstFrame && dwSize>0) { //only valid frames (no blank frames)
 					frameInfos.add(new long[] {pos, dwSize, (long) frameNumber*dwMicroSecPerFrame});
-					//if (verbose)
-					//IJ.log("movie data "+frameNumber+" '"+fourccString(dwChunkId)+"' "+posSizeString(pos,dwSize)+timeString());
+					if (verbose)
+						IJ.log("movie data "+frameNumber+" '"+fourccString(dwChunkId)+"' "+posSizeString(pos,dwSize)+timeString());
 				}
 				frameNumber++;
 				if (frameNumber>lastFrameToRead) break;
@@ -819,10 +844,12 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		}
 	}
 
-	/** Read AVI 1 index */
+	/** Read AVI 1 index 'idx1' */
 	private void readOldFrameIndex(long endPosition) throws Exception, IOException {
+		//IJ.log("READ AVI 1 INDEX, isOversizedAvi1="+isOversizedAvi1);
 		int offset = -1;		//difference between absolute frame address and address given in idx1
 		int[] offsetsToTry = new int[] {0, (int)moviPosition}; // dwOffset may be w.r.t. file start or w.r.t. 'movi' list.
+		long lastFramePos = 0;
 		while (true) {
 			if ((raFile.getFilePointer()+16) >endPosition) break;
 
@@ -851,11 +878,14 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 					raFile.seek(temp);
 					if (offset < 0) return;					// neither offset works
 				}
+				long framePos = (dwOffset & SIZE_MASK) + offset;
+				if (isOversizedAvi1)
+					while (framePos < lastFramePos) framePos += FOUR_GB; //index entries are modulo 2^32, assume frames are ascending
+				lastFramePos = framePos;
 				if (frameNumber >= firstFrame) {
-					long framepos = (dwOffset & SIZE_MASK) + offset;
-					frameInfos.add(new long[]{framepos+8, dwSize, (long)frameNumber*dwMicroSecPerFrame});
-					//if (verbose)
-						//IJ.log("idx1 movie data '"+fourccString(dwChunkId)+"' "+posSizeString(framepos,dwSize)+timeString());
+					frameInfos.add(new long[]{framePos+8, dwSize, (long)frameNumber*dwMicroSecPerFrame});
+					if (verbose)
+						IJ.log("idx1 movie data '"+fourccString(dwChunkId)+"' "+posSizeString(framePos,dwSize)+timeString());
 				}
 				frameNumber++;
 				if (frameNumber>lastFrameToRead) break;
@@ -984,8 +1014,9 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		scanLineSize = isPlanarFormat ? 
 				(biWidth * biBitCount) / 8 : ((biWidth * biBitCount + 31) / 32) * 4;
 
-		// a value of biClrUsed=0 means we determine this based on the bits per pixel
-		if (readPalette && biClrUsed==0)
+		// a value of biClrUsed=0 means we determine this based on the bits per pixel, if there is a palette
+		long spaceForPalette  = endPosition-raFile.getFilePointer();
+		if (readPalette && biClrUsed==0 && spaceForPalette!=0)
 			biClrUsed = 1 << biBitCount;
 
 		if (verbose) {
@@ -996,8 +1027,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		}
 
 		//read color palette
-		if (readPalette) {
-			long spaceForPalette  = endPosition-raFile.getFilePointer();
+		if (readPalette && biClrUsed > 0) {
 			if (verbose)
 				IJ.log("   Reading "+biClrUsed+" Palette colors: " + posSizeString(spaceForPalette));
 			if (spaceForPalette < biClrUsed*4)
@@ -1018,13 +1048,12 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	/**Read from the 'movi' chunk. Skips audio ('..wb', etc.), 'LIST' 'rec' etc, only reads '..db' or '..dc'*/
 	void readMovieData(long endPosition) throws Exception, IOException {
 		if (verbose)
-			IJ.log("MOVIE DATA "+posSizeString(endPosition-raFile.getFilePointer())+timeString());
-		if (verbose)
-			IJ.log("Searching for stream "+streamNumber+": '"+
+			IJ.log("MOVIE DATA "+posSizeString(endPosition-raFile.getFilePointer())+timeString()+
+					"\nSearching for stream "+streamNumber+": '"+
 					fourccString(type0xdb)+"' or '"+fourccString(type0xdc)+"' chunks");
 		if (isVirtual) {
 			if (frameInfos==null)						// we might have it already from reading the first chunk
-				frameInfos = new Vector<long[]>(100);	// holds frame positions in file (for non-constant frame sizes, should hold long[] with pos and size)
+				frameInfos = new Vector<long[]>(lastFrameToRead);	// holds frame positions in file (for non-constant frame sizes, should hold long[] with pos and size)
 		} else if (stack==null)
 				stack = new ImageStack(dwWidth, biHeight);
 		while (true) {									//loop over all chunks
@@ -1033,6 +1062,9 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			long size = readInt() & SIZE_MASK;
 			long pos = raFile.getFilePointer();
 			long nextPos = pos + size;
+			if (nextPos > endPosition && nextPos < fileSize-8 && fileSize > FOUR_GB) {
+				endPosition =  fileSize;				//looks like old ImageJ AVI 1.0 >4GB: wrong endPosition
+			}
 			if ((type==type0xdb || type==type0xdc) && size>0) {
 				IJ.showProgress((double)frameNumber /lastFrameToRead);
 				if (verbose)
@@ -1347,7 +1379,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		return posSizeString(raFile.getFilePointer(), size);
 	}
 
-	private String posSizeString(long pos, long size) throws IOException {
+	private String posSizeString(long pos, long size) {
 		return "0x"+Long.toHexString(pos)+"-0x"+Long.toHexString(pos+size-1)+" ("+size+" Bytes)";
 	}
 
@@ -1516,4 +1548,9 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			readableSize += HUFFMAN_LENGTH;
 		}
 	}
+	
+	public void displayDialog(boolean displayDialog) {
+		this.displayDialog = displayDialog;
+	}
+	
 }
