@@ -25,7 +25,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	private static final int[] list = {AREA,MEAN,STD_DEV,MODE,MIN_MAX,
 		CENTROID,CENTER_OF_MASS,PERIMETER,RECT,ELLIPSE,SHAPE_DESCRIPTORS, FERET,
 		INTEGRATED_DENSITY,MEDIAN,SKEWNESS,KURTOSIS,AREA_FRACTION,STACK_POSITION,
-		LIMIT,LABELS,INVERT_Y,SCIENTIFIC_NOTATION,ADD_TO_OVERLAY};
+		LIMIT,LABELS,INVERT_Y,SCIENTIFIC_NOTATION,ADD_TO_OVERLAY,NaN_EMPTY_CELLS};
 
 	private static final String MEASUREMENTS = "measurements";
 	private static final String MARK_WIDTH = "mark.width";
@@ -51,6 +51,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 		rt = systemRT;
 		rt.showRowNumbers(true);
 		rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
+		rt.setNaNEmptyCells((systemMeasurements&NaN_EMPTY_CELLS)!=0);
 		measurements = systemMeasurements;
 	}
 	
@@ -69,6 +70,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if (rt==null)
 			rt = new ResultsTable();
 		rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
+		rt.setNaNEmptyCells((systemMeasurements&NaN_EMPTY_CELLS)!=0);
 		this.rt = rt;
 	}
 	
@@ -169,13 +171,14 @@ public class Analyzer implements PlugInFilter, Measurements {
 		labels[17]="Stack position"; states[17]=(systemMeasurements&STACK_POSITION)!=0;
 		gd.setInsets(0, 0, 0);
 		gd.addCheckboxGroup(10, 2, labels, states);
-		labels = new String[5];
-		states = new boolean[5];
+		labels = new String[6];
+		states = new boolean[6];
 		labels[0]="Limit to threshold"; states[0]=(systemMeasurements&LIMIT)!=0;
 		labels[1]="Display label"; states[1]=(systemMeasurements&LABELS)!=0;
 		labels[2]="Invert Y coordinates"; states[2]=(systemMeasurements&INVERT_Y)!=0;
 		labels[3]="Scientific notation"; states[3]=(systemMeasurements&SCIENTIFIC_NOTATION)!=0;;
 		labels[4]="Add to overlay"; states[4]=(systemMeasurements&ADD_TO_OVERLAY)!=0;;
+		labels[5]="NaN empty cells"; states[5]=(systemMeasurements&NaN_EMPTY_CELLS)!=0;;
 		gd.setInsets(0, 0, 0);
 		gd.addCheckboxGroup(3, 2, labels, states);
 		gd.setInsets(15, 0, 0);
@@ -219,7 +222,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 			else
 				systemMeasurements &= ~list[i];
 		}
-		if ((oldMeasurements&(~LIMIT)&(~SCIENTIFIC_NOTATION))!=(systemMeasurements&(~LIMIT)&(~SCIENTIFIC_NOTATION))&&IJ.isResultsWindow()) {
+		if ((oldMeasurements&(~SCIENTIFIC_NOTATION))!=(systemMeasurements&(~SCIENTIFIC_NOTATION))&&IJ.isResultsWindow()) {
 				rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
 				clearSummary();
 				rt.update(systemMeasurements, imp, null);
@@ -231,7 +234,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 	/** Measures the image or selection and adds the results to the default results table. */
 	public void measure() {
 		String lastHdr = rt.getColumnHeading(ResultsTable.LAST_HEADING);
-		if (lastHdr==null || lastHdr.charAt(0)!='S') {
+		if (lastHdr==null || lastHdr.charAt(0)!='M') {
 			if (!reset()) return;
 		}
 		firstParticle = lastParticle = 0;
@@ -259,6 +262,20 @@ public class Analyzer implements PlugInFilter, Measurements {
 		saveResults(stats, roi);
 	}
 	
+	/*
+	void showHeadings() {
+		String[] headings = rt.getHeadings();
+		int columns = headings.length;
+		if (columns==0)
+			return;
+		IJ.log("Headings: "+headings.length+" "+rt.getColumnHeading(ResultsTable.LAST_HEADING));
+		for (int i=0; i<columns; i++) {
+			if (headings[i]!=null)
+				IJ.log("   "+i+" "+headings[i]+" "+rt.getColumnIndex(headings[i]));
+		}
+	}
+	*/
+	
 	boolean reset() {
 		boolean ok = true;
 		if (rt.size()>0)
@@ -338,7 +355,8 @@ public class Analyzer implements PlugInFilter, Measurements {
 	
 	void measurePoint(Roi roi) {
 		if (rt.size()>0) {
-			if (!IJ.isResultsWindow()) reset();
+			if (!IJ.isResultsWindow())
+				reset();
 			int index = rt.getColumnIndex("X");
 			if (index<0 || !rt.columnExists(index)) {
 				clearSummary();
@@ -348,11 +366,23 @@ public class Analyzer implements PlugInFilter, Measurements {
 		FloatPolygon p = roi.getFloatPolygon();
 		ImagePlus imp2 = isRedirectImage()?getRedirectImageOrStack(imp):null;
 		if (imp2==null) imp2 = imp;
+		ImageStack stack = null;
+		if (imp2.getStackSize()>1)
+			stack = imp2.getStack();
 		for (int i=0; i<p.npoints; i++) {
-			ImageProcessor ip = imp2.getProcessor();
+			int position = 0;
+			if (roi instanceof PointRoi)
+				position = ((PointRoi)roi).getPointPosition(i);
+			ImageProcessor ip = null;
+			if (stack!=null && position>0 && position<=stack.size())
+				ip = stack.getProcessor(position);
+			else
+				ip = imp2.getProcessor();
 			ip.setRoi((int)p.xpoints[i], (int)p.ypoints[i], 1, 1);
 			ImageStatistics stats = ImageStatistics.getStatistics(ip, measurements, imp2.getCalibration());
-			saveResults(stats, new PointRoi(p.xpoints[i], p.ypoints[i]));
+			PointRoi point = new PointRoi(p.xpoints[i], p.ypoints[i]);
+			point.setPosition(position);
+			saveResults(stats, point);
 			if (i!=p.npoints-1) displayResults();
 		}
 	}
@@ -395,26 +425,59 @@ public class Analyzer implements PlugInFilter, Measurements {
 		}
 		boolean straightLine = roi.getType()==Roi.LINE;
 		int lineWidth = (int)Math.round(roi.getStrokeWidth());
-		ImageProcessor ip2;
+		ImageProcessor ip2 = imp2.getProcessor();
+		double minThreshold = ip2.getMinThreshold();
+		double maxThreshold = ip2.getMaxThreshold();
+		int limit = (Analyzer.getMeasurements()&LIMIT)!=0?LIMIT:0;
+		boolean calibrated = imp2.getCalibration().calibrated();
 		Rectangle saveR = null;
+		Calibration globalCal = calibrated?imp2.getGlobalCalibration():null;
+		Calibration localCal = null;
+		if (globalCal!=null) {
+			imp2.setGlobalCalibration(null);
+			localCal = imp2.getCalibration().copy();
+			imp2.setCalibration(globalCal);
+		}
 		if (straightLine && lineWidth>1) {
-			ip2 = imp2.getProcessor();
 			saveR = ip2.getRoi();
 			ip2.setRoi(roi.getPolygon());
+		} else if (lineWidth>1 && calibrated && limit!=0) {
+			Calibration cal = imp2.getCalibration().copy();
+			imp2.getCalibration().disableDensityCalibration();
+			ip2 = (new Straightener()).straightenLine(imp2, lineWidth);
+			imp2.setCalibration(cal);
+			ip2 = convertToOriginalDepth(imp2, ip2);
+			ip2.setCalibrationTable(cal.getCTable());
 		} else if (lineWidth>1) {
-			if ((measurements&AREA)!=0 || (measurements&MEAN)!=0)
+			if ((measurements&AREA)!=0 || (measurements&MEAN)!=0 || calibrated) {
 				ip2 = (new Straightener()).straightenLine(imp2, lineWidth);
-			else {
+				if (limit!=0)
+					ip2 = convertToOriginalDepth(imp2, ip2);
+			} else {
 				saveResults(new ImageStatistics(), roi);
 				return;
 			}
+		} else if (calibrated && limit!=0) {
+			Calibration cal = imp2.getCalibration().copy();
+			imp2.getCalibration().disableDensityCalibration();
+			ProfilePlot profile = new ProfilePlot(imp2);
+			imp2.setCalibration(cal);
+			double[] values = profile.getProfile();
+			if (values==null) return;
+			ip2 = new FloatProcessor(values.length, 1, values);
+			ip2 = convertToOriginalDepth(imp2, ip2);
+			ip2.setCalibrationTable(cal.getCTable());
 		} else {
 			ProfilePlot profile = new ProfilePlot(imp2);
 			double[] values = profile.getProfile();
 			if (values==null) return;
 			ip2 = new FloatProcessor(values.length, 1, values);
+			if (limit!=0)
+				ip2 = convertToOriginalDepth(imp2, ip2);
 		}
-		ImageStatistics stats = ImageStatistics.getStatistics(ip2, AREA+MEAN+STD_DEV+MODE+MIN_MAX, imp2.getCalibration());
+		if (limit!=0 && minThreshold!=ImageProcessor.NO_THRESHOLD)
+			ip2.setThreshold(minThreshold,maxThreshold,ImageProcessor.NO_LUT_UPDATE);
+		ImageStatistics stats = ImageStatistics.getStatistics(ip2, AREA+MEAN+STD_DEV+MODE+MIN_MAX+limit, imp2.getCalibration());
 		if (saveR!=null) ip2.setRoi(saveR);
 		if ((roi instanceof Line) && (measurements&CENTROID)!=0) {
 			FloatPolygon p = ((Line)roi).getFloatPoints();
@@ -427,8 +490,21 @@ public class Analyzer implements PlugInFilter, Measurements {
 			}
 		}
 		saveResults(stats, roi);
+		if (globalCal!=null && localCal!=null) {
+			imp2.setGlobalCalibration(globalCal);
+			imp2.setCalibration(localCal);
+		}
 	}
 	
+	private ImageProcessor convertToOriginalDepth(ImagePlus imp, ImageProcessor ip) {
+		if (imp.getBitDepth()==8)
+			ip = ip.convertToByte(false);
+		else if (imp.getBitDepth()==16)
+			ip = ip.convertToShort(false);
+		return ip;
+	}
+
+	
 	/** Saves the measurements specified in the "Set Measurements" dialog,
 		or by calling setMeasurements(), in the default results table.
 	*/
@@ -591,6 +667,10 @@ public class Analyzer implements PlugInFilter, Measurements {
 			} else if (roi.getType()==Roi.POINT)
 				savePoints(roi);
 		}
+		if ((measurements&LIMIT)!=0 && imp!=null && imp.getBitDepth()!=24) {
+			rt.addValue(ResultsTable.MIN_THRESHOLD, stats.lowerThreshold);
+			rt.addValue(ResultsTable.MAX_THRESHOLD, stats.upperThreshold);
+		}
 	}
 	
 	private void clearSummary() {
@@ -613,33 +693,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 		}
 		return (Math.abs(carea/2.0));
 	}
-	
-	/*
-	final double getConvexPerimeter(Roi roi, Polygon ch) {
-		if (roi==null || ch==null || !(roi instanceof PolygonRoi))
-			return 0.0;
-		int[] xp = ((PolygonRoi)roi).getXCoordinates();
-		int[] yp = ((PolygonRoi)roi).getYCoordinates();
-		int n = ((PolygonRoi)roi).getNCoordinates();
-		double perim = getPerimeter(xp, yp, n);
-		double convexPerim = getPerimeter(ch.xpoints, ch.ypoints, ch.npoints);
-		return convexPerim;
-	}
-	
-	final double getPerimeter(int[] xp, int yp[], int n) {
-		double dx, dy, perim=0.0;
-		for (int i=0; i<n-1; i++) {
-			dx = xp[i+1]-xp[i];
-			dy = yp[i+1]-yp[i];
-			perim += Math.sqrt(dx*dx+dy*dy);
-		}
-		dx = xp[n-1] - xp[0];
-		dy = yp[n-1] - yp[0];
-		perim += Math.sqrt(dx*dx+dy*dy);
-		return perim;
-	}
-	*/
-	
+		
 	void savePoints(Roi roi) {
 		if (imp==null) {
 			rt.addValue("X", 0.0);
@@ -667,15 +721,28 @@ public class Analyzer implements PlugInFilter, Measurements {
 		}
 		rt.addValue("X", cal.getX(x));
 		rt.addValue("Y", cal.getY(y, imp.getHeight()));
+		int position = roi.getPosition();
 		if (imp.isHyperStack() || imp.isComposite()) {
+			int channel = imp.getChannel();
+			int slice = imp.getSlice();
+			int frame = imp.getFrame();
+			if (position>0) {
+				int[] pos = imp.convertIndexToPosition(position);
+				channel = pos[0];
+				slice = pos[1];
+				frame = pos[2];
+			}
 			if (imp.getNChannels()>1)
-				rt.addValue("Ch", imp.getChannel());
+				rt.addValue("Ch", channel);
 			if (imp.getNSlices()>1)
-				rt.addValue("Slice", imp.getSlice());
+				rt.addValue("Slice", slice);
 			if (imp.getNFrames()>1)
-				rt.addValue("Frame", imp.getFrame());
-		} else if (imp.getStackSize()>1)
-			rt.addValue("Slice", cal.getZ(imp.getCurrentSlice()));
+				rt.addValue("Frame", frame);
+		} else if (imp.getStackSize()>1) {
+			if (position==0)
+				position = imp.getCurrentSlice();
+			rt.addValue("Slice", position);
+		}
 		if (imp.getProperty("FHT")!=null) {
 			double center = imp.getWidth()/2.0;
 			y = imp.getHeight()-y-1;
@@ -687,8 +754,6 @@ public class Analyzer implements PlugInFilter, Measurements {
 			rt.addValue("R", (imp.getWidth()/r)*cal.pixelWidth);
 			rt.addValue("Theta", theta);
 		}
-		//if ((measurements&MEAN)==0)
-		//	rt.addValue("Mean", value);
 	}
 
 	String getFileName() {
@@ -925,6 +990,7 @@ public class Analyzer implements PlugInFilter, Measurements {
 		if (rt==null)
 			rt = new ResultsTable();
 		rt.setPrecision((systemMeasurements&SCIENTIFIC_NOTATION)!=0?-precision:precision);
+		rt.setNaNEmptyCells((systemMeasurements&NaN_EMPTY_CELLS)!=0);
 		systemRT = rt;
 		summarized = false;
 		umeans = null;
