@@ -90,7 +90,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	private static double staticMaxSize = DEFAULT_MAX_SIZE;
 	private static boolean pixelUnits;
 	private static int staticOptions = Prefs.getInt(OPTIONS,CLEAR_WORKSHEET);
-	private static String[] showStrings = {"Nothing", "Outlines", "Bare Outlines", "Ellipses", "Masks", "Count Masks", "Overlay Outlines", "Overlay Masks"};
+	private static String[] showStrings = {"Nothing", "Outlines", "Bare Outlines", "Ellipses", "Masks", "Count Masks", "Overlay", "Overlay Masks"};
 	private static double staticMinCircularity=0.0, staticMaxCircularity=1.0;
 		
 	protected static final int NOTHING=0, OUTLINES=1, BARE_OUTLINES=2, ELLIPSES=3, MASKS=4, ROI_MASKS=5,
@@ -158,6 +158,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	private int lineWidth = nextLineWidth;
 	private boolean noThreshold;
 	private boolean calledByPlugin;
+	private boolean hyperstack;
 
 			
 	/** Constructs a ParticleAnalyzer.
@@ -216,8 +217,10 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		this.arg = arg;
 		this.imp = imp;
 		IJ.register(ParticleAnalyzer.class);
-		if (imp==null)
-			{IJ.noImage();return DONE;}
+		if (imp==null) {
+			IJ.noImage();
+			return DONE;
+		}
 		if (imp.getBitDepth()==24 && !isThresholdedRGB(imp)) {
 			IJ.error("Particle Analyzer",
 			"RGB images must be thresholded using\n"
@@ -249,6 +252,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		if (imp.getType()==ImagePlus.COLOR_RGB) {
 			ip = (ImageProcessor)imp.getProperty("Mask");
 			ip.setThreshold(255, 255, ImageProcessor.NO_LUT_UPDATE);
+			ip.setRoi(imp.getRoi());
 		}		
 		if (!analyze(imp, ip))
 			canceled = true;
@@ -399,13 +403,17 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 
 	boolean updateMacroOptions() {
 		String options = Macro.getOptions();
+		options = options.replace("show=[Overlay Outlines]", "show=Overlay");
+		Macro.setOptions(options);
 		int index = options.indexOf("maximum=");
-		if (index==-1) return false;
+		if (index==-1)
+			return false;
 		index +=8;
 		int len = options.length();
 		while (index<len-1 && options.charAt(index)!=' ')
 			index++;
-		if (index==len-1) return false;
+		if (index==len-1)
+			return false;
 		int min = (int)Tools.parseDouble(Macro.getValue(options, "minimum", "1"));
 		int max = (int)Tools.parseDouble(Macro.getValue(options, "maximum", "999999"));
 		options = "size="+min+"-"+max+options.substring(index, len);
@@ -435,6 +443,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			roiManager = staticRoiManager;
 			staticRoiManager = null;
 		}
+		hyperstack = imp.isHyperStack();
 		if (staticResultsTable!=null) {
 			rt = staticResultsTable;
 			staticResultsTable = null;
@@ -502,15 +511,23 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		}
 		calibration = redirectImp!=null?redirectImp.getCalibration():imp.getCalibration();
 		
+		if (measurements==0)
+			measurements = Analyzer.getMeasurements();
+		measurements &= ~LIMIT;	 // ignore "Limit to Threshold"
 		if (rt==null) {
-			rt = Analyzer.getResultsTable();
-			analyzer = new Analyzer(imp);
-		} else {
-			if (measurements==0)
-				measurements = Analyzer.getMeasurements();
-			analyzer = new Analyzer(imp, measurements, rt);
+			Frame table = WindowManager.getFrame("Results");
+			if (!showResults && table!=null) {
+				rt = new ResultsTable();
+				if (resetCounter && table instanceof TextWindow) {
+					IJ.run("Clear Results");
+					((TextWindow)table).close();
+					rt = Analyzer.getResultsTable();
+				}
+			} else
+				rt = Analyzer.getResultsTable();
 		}
-		if (resetCounter && slice==1) {
+		analyzer = new Analyzer(imp, measurements, rt);
+		if (resetCounter && slice==1 && rt.getCounter()>0) {
 			if (!Analyzer.resetCounter())
 				return false;
 		}
@@ -540,11 +557,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		ImageWindow win = imp.getWindow();
 		if (win!=null)
 			win.running = true;
-		if (measurements==0)
-			measurements = Analyzer.getMeasurements();
 		if (showChoice==ELLIPSES)
 			measurements |= ELLIPSE;
-		measurements &= ~LIMIT;	 // ignore "Limit to Threshold"
 		roiNeedsImage = (measurements&PERIMETER)!=0 || (measurements&SHAPE_DESCRIPTORS)!=0 || (measurements&FERET)!=0;
 		particleCount = 0;
 		wand = new Wand(ip);
@@ -589,8 +603,12 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		ip.reset();
 		if (displaySummary && IJ.getInstance()!=null)
 			updateSliceSummary();
-		if (addToManager && roiManager!=null && imp.getWindow()!=null)
-			roiManager.setEditMode(imp, true);
+		if (addToManager && roiManager!=null) {
+			if (imp.getWindow()!=null)
+				roiManager.setEditMode(imp, true);
+			else
+				roiManager.runCommand("show all with labels");
+		}
 		maxParticleCount = (particleCount > maxParticleCount) ? particleCount : maxParticleCount;
 		totalCount += particleCount;
 		if (!canceled)
@@ -740,8 +758,6 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		ip.setRoi(r.x+r.width, 0, width-(r.x+r.width), height);
 		ip.fill();
 		ip.resetRoi();
-		//IJ.log("erase: "+fillColor+"	"+level1+"	"+level2+"	"+excludeEdgeParticles);
-		//(new ImagePlus("ip2", ip.duplicate())).show();
 		return true;
 	}
 
@@ -814,7 +830,6 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	int counter = 0;
 	
 	void analyzeParticle(int x, int y, ImagePlus imp, ImageProcessor ip) {
-		//Wand wand = new Wand(ip);
 		ImageProcessor ip2 = redirectIP!=null?redirectIP:ip;
 		wand.autoOutline(x, y, level1, level2, wandMode);
 		if (wand.npoints==0)
@@ -855,7 +870,6 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			double perimeter = roi.getLength();
 			double circularity = perimeter==0.0?0.0:4.0*Math.PI*(stats.pixelCount/(perimeter*perimeter));
 			if (circularity>1.0) circularity = 1.0;
-			//IJ.log(circularity+"	"+perimeter+"  "+stats.area);
 			if (circularity<minCircularity || circularity>maxCircularity) include = false;
 		}
 		if (stats.pixelCount>=minSize && stats.pixelCount<=maxSize && include) {
@@ -911,8 +925,14 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				if (resetCounter)
 					roiManager.runCommand("reset");
 			}
-			if (imp.getStackSize()>1)
-				roi.setPosition(imp.getCurrentSlice());
+			if (imp.getStackSize()>1) {
+				int n = imp.getCurrentSlice();
+				if (hyperstack) {
+					int[] pos = imp.convertIndexToPosition(n);
+					roi.setPosition(pos[0],pos[1],pos[2]);
+				} else
+					roi.setPosition(n);
+			}
 			if (lineWidth!=1)
 				roi.setStrokeWidth(lineWidth);
 			roiManager.add(imp, roi, rt.getCounter());
@@ -936,7 +956,6 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 	}
 
 	void drawFilledParticle(ImageProcessor ip, Roi roi, ImageProcessor mask) {
-		//IJ.write(roi.getBounds()+" "+mask.length);
 		ip.setRoi(roi.getBounds());
 		ip.fill(mask);
 	}
@@ -954,8 +973,13 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				roi2.setStrokeWidth(lineWidth);
 			if (showChoice==OVERLAY_MASKS)
 				roi2.setFillColor(Color.cyan);
-			if (processStack)
-				roi2.setPosition(slice);
+			if (processStack) {
+				if (hyperstack) {
+					int[] pos = imp.convertIndexToPosition(slice);
+					roi2.setPosition(pos[0],pos[1],pos[2]);
+				} else
+					roi2.setPosition(slice);
+			}
 			overlay.add(roi2);
 		} else {
 			Rectangle r = roi.getBounds();
