@@ -17,7 +17,8 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 	ActionListener, AdjustmentListener, ItemListener {
 
 	public static final String LOC_KEY = "b&c.loc";
-	public static final String[] sixteenBitRanges = {"Automatic", "8-bit (0-255)", "10-bit (0-1023)", "12-bit (0-4095)", "15-bit (0-32767)", "16-bit (0-65535)"};
+	public static final String[] sixteenBitRanges = {"Automatic", "8-bit (0-255)", "10-bit (0-1023)",
+		"12-bit (0-4095)", "14-bit (0-16383)", "15-bit (0-32767)", "16-bit (0-65535)"};
 	static final int AUTO_THRESHOLD = 5000;
 	static final String[] channelLabels = {"Red", "Green", "Blue", "Cyan", "Magenta", "Yellow", "All"};
 	static final String[] altChannelLabels = {"Channel 1", "Channel 2", "Channel 3", "Channel 4", "Channel 5", "Channel 6", "All"};
@@ -36,7 +37,6 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 	int previousImageID;
 	int previousType;
 	int previousSlice = 1;
-	Object previousSnapshot;
 	ImageJ ij;
 	double min, max;
 	double previousMin, previousMax;
@@ -51,10 +51,12 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 	GridBagConstraints c;
 	int y = 0;
 	boolean windowLevel, balance;
-	Font monoFont = new Font("Monospaced", Font.PLAIN, 12);
+	Font monoFont = new Font("Monospaced", Font.PLAIN, 11);
 	Font sanFont = ImageJ.SansSerif12;
 	int channels = 7; // RGB
 	Choice choice;
+	private String blankMinLabel = "-------";
+	private String blankMaxLabel = "--------";
 
 	public ContrastAdjuster() {
 		super("B&C");
@@ -107,13 +109,17 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 			c.insets = new Insets(0, 10, 0, 10);
 			gridbag.setConstraints(panel, c);
 			panel.setLayout(new BorderLayout());
-			minLabel = new Label("      ", Label.LEFT);
+			minLabel = new Label(blankMinLabel, Label.LEFT);
 			minLabel.setFont(monoFont);
+			if (IJ.debugMode) minLabel.setBackground(Color.yellow);
 			panel.add("West", minLabel);
-			maxLabel = new Label("      " , Label.RIGHT);
+			maxLabel = new Label(blankMaxLabel, Label.RIGHT);
 			maxLabel.setFont(monoFont);
+			if (IJ.debugMode) maxLabel.setBackground(Color.yellow);
 			panel.add("East", maxLabel);
 			add(panel);
+			blankMinLabel = "       ";
+			blankMaxLabel = "        ";
 		}
 
 		// min slider
@@ -307,8 +313,7 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 		int type = imp.getType();
 		int slice = imp.getCurrentSlice();
 		RGBImage = type==ImagePlus.COLOR_RGB;
-		boolean snapshotChanged = RGBImage && previousSnapshot!=null && ((ColorProcessor)ip).getSnapshotPixels()!=previousSnapshot;
-		if (imp.getID()!=previousImageID || snapshotChanged || type!=previousType || slice!=previousSlice)
+		if (imp.getID()!=previousImageID || type!=previousType || slice!=previousSlice)
 			setupNewImage(imp, ip);
 		previousImageID = imp.getID();
 	 	previousType = type;
@@ -317,19 +322,20 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 	}
 
 	void setupNewImage(ImagePlus imp, ImageProcessor ip)  {
-		//IJ.write("setupNewImage");
 		Undo.reset();
 		previousMin = min;
 		previousMax = max;
-	 	if (RGBImage) {
+		boolean newRGBImage = RGBImage && !((ColorProcessor)ip).caSnapshot();
+	 	if (newRGBImage) {
 	 		ip.snapshot();
-	 		previousSnapshot = ((ColorProcessor)ip).getSnapshotPixels();
-	 	} else
-			previousSnapshot = null;
+	 		((ColorProcessor)ip).caSnapshot(true);
+	 	}
 		double min2 = imp.getDisplayRangeMin();
 		double max2 = imp.getDisplayRangeMax();
-		if (imp.getType()==ImagePlus.COLOR_RGB)
-			{min2=0.0; max2=255.0;}
+		if (newRGBImage) {
+			min2=0.0;
+			max2=255.0;
+		}
 		int bitDepth = imp.getBitDepth();
 		if (bitDepth==16 || bitDepth==32) {
 			imp.resetDisplayRange();
@@ -399,8 +405,6 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 			imp.setDisplayRange(min, max, channels);
 		else
 			imp.setDisplayRange(min, max);
-		if (!rgb)
-			imp.getProcessor().setSnapshotPixels(null); // disable undo
 	}
 
 	void updatePlot() {
@@ -421,16 +425,29 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 			if (type!=ImagePlus.GRAY16)
 				realValue = true;
 		}
-		int digits = realValue?2:0;
 		if (windowLevel) {
-			//IJ.log(min+" "+max);
+			int digits = realValue?2:0;
 			double window = max-min;
 			double level = min+(window)/2.0;
 			windowLabel.setText(IJ.d2s(window, digits));
 			levelLabel.setText(IJ.d2s(level, digits));
 		} else {
-			minLabel.setText(IJ.d2s(min, digits));
-			maxLabel.setText(IJ.d2s(max, digits));
+			int digits = realValue?4:0;
+			if (realValue) {
+				double s = min<0||max<0?0.1:1.0;
+				double amin = Math.abs(min);
+				double amax = Math.abs(max);
+				if (amin>99.0*s||amax>99.0*s) digits = 3;
+				if (amin>999.0*s||amax>999.0*s) digits = 2;
+				if (amin>9999.0*s||amax>9999.0*s) digits = 1;
+				if (amin>99999.0*s||amax>99999.0*s) digits = 0;
+				if (amin>9999999.0*s||amax>9999999.0*s) digits = -2;
+			}
+			String minString = IJ.d2s(min, min==0.0?0:digits) + blankMinLabel;
+			minLabel.setText(minString.substring(0,blankMinLabel.length()));
+			String maxString = blankMaxLabel + IJ.d2s(max, digits);
+			maxString = maxString.substring(maxString.length()-blankMaxLabel.length(), maxString.length());
+			maxLabel.setText(maxString);
 		}
 	}
 
@@ -480,8 +497,14 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 	/** Restore image outside non-rectangular roi. */
   	void doMasking(ImagePlus imp, ImageProcessor ip) {
 		ImageProcessor mask = imp.getMask();
-		if (mask!=null)
+		if (mask!=null) {
+			Rectangle r = ip.getRoi();
+			if (mask.getWidth()!=r.width||mask.getHeight()!=r.height) {
+				ip.setRoi(imp.getRoi());
+				mask = ip.getMask();
+			}
 			ip.reset(mask);
+		}
 	}
 
 	void adjustMin(ImagePlus imp, ImageProcessor ip, double minvalue) {
@@ -603,36 +626,36 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 		if (RGBImage) {
 			if (imp.getStackSize()>1)
 				applyRGBStack(imp);
-			else {
-				ip.snapshot();
-				reset(imp, ip);
-				imp.changes = true;
-				if (Recorder.record) {
-					if (Recorder.scriptMode())
-						Recorder.recordCall("IJ.run(imp, \"Apply LUT\", \"\");");
-					else
-						Recorder.record("run", "Apply LUT");
-				}
-			}
-			imp.unlock();
+			else
+				applyRGB(imp,ip);
 			return;
 		}
-		if (imp.getType()!=ImagePlus.GRAY8) {
+		int bitDepth = imp.getBitDepth();
+		if (bitDepth==32) {
 			IJ.beep();
-			IJ.showStatus("Apply requires an 8-bit grayscale image or an RGB stack");
+			IJ.showStatus("\"Apply\" does not work with 32-bit images");
 			imp.unlock();
 			return;
 		}
-		int[] table = new int[256];
+		int range = 256;
+		if (bitDepth==16) {
+			range = 65536;
+			int defaultRange = imp.getDefault16bitRange();
+			if (defaultRange>0)
+				range = (int)Math.pow(2,defaultRange)-1;
+		}
+		int tableSize = bitDepth==16?65536:256;
+		int[] table = new int[tableSize];
 		int min = (int)imp.getDisplayRangeMin();
 		int max = (int)imp.getDisplayRangeMax();
-		for (int i=0; i<256; i++) {
+		if (IJ.debugMode) IJ.log("Apply: mapping "+min+"-"+max+" to 0-"+(range-1));
+		for (int i=0; i<tableSize; i++) {
 			if (i<=min)
 				table[i] = 0;
 			else if (i>=max)
-				table[i] = 255;
+				table[i] = range-1;
 			else
-				table[i] = (int)(((double)(i-min)/(max-min))*255);
+				table[i] = (int)(((double)(i-min)/(max-min))*range);
 		}
 		ip.setRoi(imp.getRoi());
 		if (imp.getStackSize()>1 && !imp.isComposite()) {
@@ -659,13 +682,13 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 				imp.setSlice(current);
 				option = "stack";
 			} else {
-				if (ip.getMask()!=null) ip.snapshot();
+				ip.snapshot();
 				ip.applyTable(table);
 				ip.reset(ip.getMask());
 				option = "slice";
 			}
 		} else {
-			if (ip.getMask()!=null) ip.snapshot();
+			ip.snapshot();
 			ip.applyTable(table);
 			ip.reset(ip.getMask());
 		}
@@ -685,7 +708,30 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 		}
 	}
 
-	void applyRGBStack(ImagePlus imp) {
+	void applyRGB(ImagePlus imp, ImageProcessor ip) {
+		double min = imp.getDisplayRangeMin();
+		double max = imp.getDisplayRangeMax();
+ 		ip.setRoi(imp.getRoi());
+ 		ip.reset();
+		if (channels!=7)
+			((ColorProcessor)ip).setMinAndMax(min, max, channels);
+		else
+			ip.setMinAndMax(min, max);
+		ip.reset(ip.getMask());
+		imp.changes = true;
+		previousImageID = 0;
+	 	((ColorProcessor)ip).caSnapshot(false);
+		setup();
+		imp.deleteRoi();
+		if (Recorder.record) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("IJ.run(imp, \"Apply LUT\", \"\");");
+			else
+				Recorder.record("run", "Apply LUT");
+		}
+	}
+
+	private void applyRGBStack(ImagePlus imp) {
 		double min = imp.getDisplayRangeMin();
 		double max = imp.getDisplayRangeMax();
 		if (IJ.debugMode) IJ.log("applyRGBStack: "+min+"-"+max);
@@ -844,6 +890,7 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 				for (int c=1; c<=channels; c++) {
 					imp.setPositionWithoutUpdate(c, imp.getSlice(), imp.getFrame());
 					imp.setDisplayRange(min, max);
+					//IJ.log("setDisplayRange: "+c+" "+min+" "+max);
 				}
 				((CompositeImage)imp).reset();
 				imp.setPosition(channel, imp.getSlice(), imp.getFrame());
@@ -921,8 +968,9 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 		if (range==8) index = 1;
 		else if (range==10) index = 2;
 		else if (range==12) index = 3;
-		else if (range==15) index = 4;
-		else if (range==16) index = 5;
+		else if (range==14) index = 4;
+		else if (range==15) index = 5;
+		else if (range==16) index = 6;
 		return index;
 	}
 
@@ -931,11 +979,16 @@ public class ContrastAdjuster extends PlugInDialog implements Runnable,
 		if (index==1) range = 8;
 		else if (index==2) range = 10;
 		else if (index==3) range = 12;
-		else if (index==4) range = 15;
-		else if (index==5) range = 16;
+		else if (index==4) range = 14;
+		else if (index==5) range = 15;
+		else if (index==6) range = 16;
 		ImagePlus.setDefault16bitRange(range);
 		return range;
 	}
+	
+	public static String[] getSixteenBitRanges() {
+		return sixteenBitRanges;
+	}
 
 	void setWindowLevel(ImagePlus imp, ImageProcessor ip) {
 		min = imp.getDisplayRangeMin();
