@@ -139,7 +139,7 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 		boolean sw = method.equals("selectWindow");
 		if (textArea!=null && !(scriptMode&&sw||commandName!=null&&sw)) {
 			if (scriptMode && method.equals("roiManager"))
-				textArea.append("rm.runCommand(\""+arg+"\");\n");
+				textArea.append("rm.runCommand(imp,\""+arg+"\");\n");
 			else if (scriptMode && method.equals("run"))
 				textArea.append("IJ."+method+"(\""+arg+"\");\n");
 			else {
@@ -178,6 +178,11 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 		textArea.append(method+"("+a1+", "+a2+");\n");
 	}
 
+	public static void record(String method, int a1, int a2, String a3) {
+		if (textArea==null) return;
+		textArea.append(method+"("+a1+", "+a2+", \""+a3+"\");\n");
+	}
+
 	public static void record(String method, double a1, double a2) {
 		if (textArea==null) return;
 		int places = Math.abs(a1)<0.0001||Math.abs(a2)<0.0001?9:4;
@@ -203,13 +208,13 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 		if (textArea==null) return;
 		if (scriptMode&&method.startsWith("make")) {
 			if (method.equals("makeRectangle"))
-				recordString("imp.setRoi("+a1+", "+a2+", "+a3+", "+a4+");\n");
+				recordString("imp.setRoi("+a1+","+a2+","+a3+","+a4+");\n");
 			else if (method.equals("makeOval"))
-				recordString("imp.setRoi(new OvalRoi("+a1+", "+a2+", "+a3+", "+a4+"));\n");
+				recordString("imp.setRoi(new OvalRoi("+a1+","+a2+","+a3+","+a4+"));\n");
 			else if (method.equals("makeLine"))
-				recordString("imp.setRoi(new Line("+a1+", "+a2+", "+a3+", "+a4+"));\n");
+				recordString("imp.setRoi(new Line("+a1+","+a2+","+a3+","+a4+"));\n");
 			else if (method.equals("makeArrow"))
-				recordString("imp.setRoi(new Arrow("+a1+", "+a2+", "+a3+", "+a4+"));\n");
+				recordString("imp.setRoi(new Arrow("+a1+","+a2+","+a3+","+a4+"));\n");
 		} else {
 			if (method.equals("makeArrow")) {
 				ImagePlus imp = WindowManager.getCurrentImage();
@@ -279,8 +284,16 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 		recordCall(javaMode()?className+" "+call:call);
 	}
 
+	public static void recordRoi(Roi roi) {
+		if (roi==null)
+			return;
+		Polygon polygon = roi.getPolygon();
+		recordRoi(polygon, roi.getType());
+	}
+
 	public static void recordRoi(Polygon p, int type) {
-		if (textArea==null) return;
+		if (textArea==null)
+			return;
 		if (scriptMode)
 			{recordScriptRoi(p,type); return;}
 		if (type==Roi.ANGLE||type==Roi.POINT) {
@@ -294,7 +307,7 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 			String typeStr= type==Roi.ANGLE?"angle":"point";
 			textArea.append("makeSelection(\""+typeStr+"\","+xarr+","+yarr+");\n");
 		} else {
-			String method = type==Roi.POLYGON?"makePolygon":"makeLine";
+			String method = type>=Roi.LINE && type<=Roi.FREELINE?"makeLine":"makePolygon";
 			StringBuffer args = new StringBuffer();
 			for (int i=0; i<p.npoints; i++) {
 				args.append(p.xpoints[i]+",");
@@ -420,7 +433,7 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 	public static void saveCommand() {
 		String name = commandName;
 		if (name!=null) {
-			if (commandOptions==null && (name.equals("Fill")||name.equals("Clear")))
+			if (commandOptions==null && (name.equals("Fill")||name.equals("Clear")||name.equals("Draw")))
 				commandOptions = "slice";
 			if (!fgColorSet && (name.equals("Fill")||name.equals("Draw")))
 				setForegroundColor(Toolbar.getForegroundColor());
@@ -432,9 +445,24 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 			if (commandOptions!=null) {
 				if (name.equals("Open...") || name.equals("URL...")) {
 					String s = scriptMode?"imp = IJ.openImage":"open";
-					if (scriptMode && isTextOrTable(commandOptions))
-						s = "IJ.open";
-					textArea.append(s+"(\""+strip(commandOptions)+"\");\n");
+					String path = strip(commandOptions);
+					boolean openingLut = false;
+					if (scriptMode) {
+						if (isTextOrTable(commandOptions))
+							s = "IJ.open";
+						else if (path!=null && path.endsWith(".lut")) {
+							s = "lut = Opener.openLut";
+							openingLut = true;
+						}
+					}
+					textArea.append(s+"(\""+path+"\");\n");
+					ImagePlus imp = WindowManager.getCurrentImage();
+					if (openingLut && imp!=null && !imp.getTitle().endsWith(".lut"))
+						textArea.append("imp.setLut(lut);\n");
+				} else if (name.equals("TIFF Virtual Stack...") && scriptMode) {
+					String s = "imp = IJ.openVirtual";
+					String path = strip(commandOptions);
+					textArea.append(s+"(\""+path+"\");\n");
 				} else if (isSaveAs()) {
 							if (name.endsWith("..."))
 									name= name.substring(0, name.length()-3);
@@ -457,7 +485,11 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 					;
 				else if (name.equals("Run...")) // Plugins>Macros>Run
 					;
+				else if (scriptMode && name.equals("Text Image... ")) // File>Import>Text Image
+					;
 				else {
+					if (name.equals("Calibrate...")&&commandOptions.startsWith("function=None"))
+						commandOptions = commandOptions.substring(0,13);
 					String prefix = "run(";
 					if (scriptMode) {
 						boolean addImp = imageUpdated || (WindowManager.getCurrentImage()!=null
@@ -467,17 +499,26 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 						prefix = addImp?"IJ.run(imp, ":"IJ.run(";
 					}
 					textArea.append(prefix+"\""+name+"\", \""+commandOptions+"\");\n");
-					if (nonAscii(commandOptions))
-						textArea.append("  <<warning: the options string contains one or more non-ascii characters>>\n");
+					if (nonAscii(commandOptions)) {
+						if (commandOptions!=null && !commandOptions.contains("="+IJ.micronSymbol+"m"))
+							textArea.append("  <<warning: the options string contains one or more non-ascii characters>>\n");
+					}
 				}
 			} else {
 				ImagePlus imp = WindowManager.getCurrentImage();
 				Roi roi = imp!=null?imp.getRoi():null;
-				if (name.equals("Threshold...") || name.equals("Fonts...") || name.equals("Brightness/Contrast..."))
+				if (name.equals("Threshold...") || name.equals("Fonts...") || name.equals("Brightness/Contrast...") || name.equals("Channels Tool..."))
 					textArea.append((scriptMode?"//IJ.":"//")+"run(\""+name+"\");\n");
 				else if (name.equals("Start Animation [\\]"))
 					textArea.append("doCommand(\"Start Animation [\\\\]\");\n");
-				else if (name.equals("Add to Manager"))
+				else if (name.equals("Split Channels") && scriptMode) {
+					String text = "channels = ChannelSplitter.split(imp);\n";
+					if (javaMode())
+						text = "ImagePlus[] " + text;
+					textArea.append(text);
+				} else if (name.equals("Add to Manager"))
+					;
+				else if (name.equals("Find Commands..."))
 					;
 				else if (roi!=null && (roi instanceof TextRoi) && (name.equals("Draw")||name.equals("Add Selection...")))
 					textArea.append(((TextRoi)roi).getMacroCode(name, imp));
@@ -611,7 +652,8 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 			}
 			if (text.contains("overlay.add"))
 				text = (java?"Overlay ":"") + "overlay = new Overlay();\n" + text;
-			if ((text.contains("imp.")||text.contains("(imp")||text.contains("overlay.add")) && !text.contains("IJ.openImage") && !text.contains("IJ.createImage"))
+			if ((text.contains("imp.")||text.contains("(imp")||text.contains("overlay.add")) && !text.contains("IJ.openImage")
+			&& !text.contains("IJ.openVirtual") && !text.contains("IJ.createImage"))
 				text = (java?"ImagePlus ":"") + "imp = IJ.getImage();\n" + text;
 			if (text.contains("overlay.add"))
 				text = text + "imp.setOverlay(overlay);\n";
@@ -640,9 +682,10 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 		StringTokenizer st = new StringTokenizer(text, "\n");
 		int n = st.countTokens();
 		boolean impDeclared = false;
+		boolean lutDeclared = false;
 		String line;
 		StringBuffer sb = new StringBuffer();
-		for(int i=0; i<n; i++) {
+		for (int i=0; i<n; i++) {
 			line = st.nextToken();
 			if (line!=null && line.length()>3) {
 				sb.append("\t\t");
@@ -650,6 +693,10 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 					sb.append("ImagePlus ");
 					impDeclared = true;
 				}
+				if (line.startsWith("lut =") && !lutDeclared) {
+					sb.append("LUT ");
+					lutDeclared = true;
+				}
 				sb.append(line);
 				sb.append('\n');
 			}
@@ -762,5 +809,12 @@ public class Recorder extends PlugInFrame implements PlugIn, ActionListener, Ima
 			recordString("setOption(\"BlackBackground\", "+bb+");\n");
 		bbSet = true;
 	}
-
+	
+	/** Override windowActivated in PlugInFrame. */
+	public void windowActivated(WindowEvent e) {
+		if (IJ.isMacintosh() && !IJ.isJava17())
+			this.setMenuBar(Menus.getMenuBar());
+		WindowManager.setWindow(this);
+	}
+	
 }
