@@ -66,7 +66,14 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 		super("Threshold");
 		ImagePlus cimp = WindowManager.getCurrentImage();
 		if (cimp!=null && cimp.getBitDepth()==24) {
-			IJ.run(cimp, "Color Threshold...", "");
+			IJ.error("Threshold Adjuster",
+				"Image>Adjust>Threshold only works with grayscale images.\n"
+				+"What you can do:\n"
+				+"   Image>Type>8-bit (convert to grayscale)\n"
+				+"   Image>Type>RGB Stack (convert to RGB stack)\n"
+				+"   Image>Type>HSB Stack (convert to HSB stack)\n"
+				+"   Image>Color>Split Channels (convert to 3 grayscale images)\n"
+				+"   Image>Adjust>Color Threshold (do color thresholding)\n");
 			return;
 		}
 		if (instance!=null) {
@@ -264,7 +271,7 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 	}
 
 	public void imageUpdated(ImagePlus imp) {
-		if (imp.getID() == previousImageID && Thread.currentThread() != thread)
+		if (imp.getID()==previousImageID && Thread.currentThread()!=thread)
 			imageWasUpdated = true;
 	}
 
@@ -294,12 +301,19 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 			mode = modeChoice.getSelectedIndex();
 			setLutColor(mode);
 			doStateChange = true;
+			if (Recorder.record) {
+				if (Recorder.scriptMode())
+					Recorder.recordCall("ThresholdAdjuster.setMode(\""+modes[mode]+"\");");
+				else
+					Recorder.recordString("call(\"ij.plugin.frame.ThresholdAdjuster.setMode\", \""+modes[mode]+"\");\n");
+			}
 		} else
 			doAutoAdjust = true;
 		notify();
 	}
 
 	ImageProcessor setup(ImagePlus imp, boolean enableAutoThreshold) {
+		if (IJ.debugMode) IJ.log("ThresholdAdjuster.setup: "+enableAutoThreshold);
 		if (imp==null)
 			return null;
 		ImageProcessor ip;
@@ -326,21 +340,22 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 				imageWasUpdated || roiHashCode!=previousRoiHashCode) {
 			minThreshold = ip.getMinThreshold();
 			maxThreshold = ip.getMaxThreshold();
-			boolean isThresholded = minThreshold != ImageProcessor.NO_THRESHOLD
+			boolean isThreshold = minThreshold != ImageProcessor.NO_THRESHOLD
 					&& ip.getCurrentColorModel() != ip.getColorModel(); //does not work???
-			//IJ.log("Changed: min/max:"+minMaxChange +" id:"+ (id!=previousImageID)+" type:"+(type!=previousImageType)+" updated:"+imageWasUpdated+". isThresh="+isThresholded);
-			//IJ.log(minThreshold+"..."+maxThreshold);
-            //Undo.reset(); removed 2014-02-06 M. Schmid - why was it there?
-            if (not8Bits && minMaxChange && !isThresholded)
-                ip.resetMinAndMax();        //imp.updateAndDraw() is below
+			if (not8Bits && minMaxChange) {
+				double max1 = ip.getMax();
+				ip.resetMinAndMax();
+				if (maxThreshold==max1)
+					maxThreshold = ip.getMax();
+			}
 			ImageStatistics stats = plot.setHistogram(imp, entireStack(imp));
 			if (stats == null)
 				return null;
-			if (isThresholded) {
+			if (isThreshold) {
 				minThreshold = scaleDown(ip, minThreshold);
 				maxThreshold = scaleDown(ip, maxThreshold);
 			} else {
-				if (enableAutoThreshold && !isThresholded)
+				if (enableAutoThreshold && !isThreshold)
 					autoSetLevels(ip, stats);
 				else
 					minThreshold = ImageProcessor.NO_THRESHOLD;  //may be an invisible threshold after 'apply'
@@ -472,14 +487,16 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 				ip.setRoi(imp.getRoi());
 				histogram = ip.getHistogram();
 				minThresholdInt = (int)Math.round(ip.getMinThreshold());
+				if (minThresholdInt<0) minThresholdInt=0;
 				maxThresholdInt = (int)Math.round(ip.getMaxThreshold());
+				if (maxThresholdInt>65535) maxThresholdInt=65535;
 				minValue=(int)ip.getMin(); maxValue=(int)ip.getMax();
 			}
 			for (int i=minValue; i<minThresholdInt; i++)
 				below += histogram[i];
 			for (int i=minThresholdInt; i<=maxThresholdInt; i++)
 				inside += histogram[i];
-			for (int i=maxThresholdInt+1; i<maxValue; i++)
+			for (int i=maxThresholdInt+1; i<=maxValue; i++)
 				above += histogram[i];
 			int total = below + inside + above;
 			//IJ.log("<"+minThresholdInt+":"+below+" in:"+inside+"; >"+maxThresholdInt+":"+above+" sum="+total);
@@ -625,7 +642,7 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 		if (Recorder.record) {
 			if (imp.getBitDepth()==32) {
 				if (Recorder.scriptMode())
-					Recorder.recordCall("IJ.setThreshold("+ip.getMinThreshold()+", "+ip.getMaxThreshold()+");");
+					Recorder.recordCall("IJ.setThreshold(imp, "+IJ.d2s(ip.getMinThreshold(),4)+", "+IJ.d2s(ip.getMaxThreshold(),4)+");");
 				else
 					Recorder.record("setThreshold", ip.getMinThreshold(), ip.getMaxThreshold());
 			} else {
@@ -634,11 +651,19 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 				if (cal.isSigned16Bit()) {
 					min = (int)cal.getCValue(level1);
 					max = (int)cal.getCValue(level2);
+					if (Recorder.scriptMode())
+						Recorder.recordCall("IJ.setThreshold(imp, "+min+", "+max+");");
+					else
+						Recorder.record("setThreshold", min, max);
 				}
 				if (Recorder.scriptMode())
-					Recorder.recordCall("IJ.setThreshold(imp, "+min+", "+max+");");
-				else
-					Recorder.record("setThreshold", min, max);
+					Recorder.recordCall("IJ.setRawThreshold(imp, "+min+", "+max+", null);");
+				else {
+					if (cal.calibrated())
+						Recorder.record("setThreshold", min, max, "raw");
+					else
+						Recorder.record("setThreshold", min, max);
+				}
 			}
 		}
 	}
@@ -790,21 +815,60 @@ public class ThresholdAdjuster extends PlugInDialog implements PlugIn, Measureme
 			ThresholdAdjuster ta = ((ThresholdAdjuster)instance);
 			ImagePlus imp = WindowManager.getCurrentImage();
 			if (imp!=null && ta.previousImageID==imp.getID()) {
+				if ((imp.getCurrentSlice()!=ta.previousSlice) && ta.entireStack(imp))
+					return;
 				ta.previousImageID = 0;
 				ta.setup(imp, false);
 			}
 		}
     }
 
-	/** Returns the current method ("Default", "Huang", etc). */
+	/** Returns the current thresholding method ("Default", "Huang", etc). */
 	public static String getMethod() {
 		return method;
 	}
 	
+	/** Sets the thresholding method ("Default", "Huang", etc). */
+	public static void setMethod(String thresholdingMethod) {
+		boolean valid = false;
+		for (int i=0; i<methodNames.length; i++) {
+			if (methodNames[i].equals(thresholdingMethod)) {
+				valid = true;
+				break;
+			}
+		}
+		if (valid) {
+			method = thresholdingMethod;
+			if (instance!=null)
+				instance.methodChoice.select(method);
+		}
+	}
+	
 	/** Returns the current mode ("Red","B&W" or"Over/Under"). */
 	public static String getMode() {
 		return modes[mode];
 	}
+	
+	/** Sets the current mode ("Red","B&W" or"Over/Under"). */
+	public static void setMode(String tmode) {
+		if (instance!=null) synchronized (instance) {
+			ThresholdAdjuster ta = ((ThresholdAdjuster)instance);
+			if (modes[0].equals(tmode))
+				mode = 0;
+			else if (modes[1].equals(tmode))
+				mode = 1;
+			else if (modes[2].equals(tmode))
+				mode = 2;
+			else
+				return;
+			ta.setLutColor(mode);
+			ta.doStateChange = true;
+			ta.modeChoice.select(mode);
+			ta.notify();
+		}
+	}
+
+
 
 } // ThresholdAdjuster class
 
@@ -838,6 +902,7 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
     }
 
 	ImageStatistics setHistogram(ImagePlus imp, boolean entireStack) {
+		if (IJ.debugMode) IJ.log("ThresholdAdjuster:setHistogram: "+entireStack+" "+entireStack2);
 		double mean = entireStack?imp.getProcessor().getStatistics().mean:0.0;
 		if (entireStack && stats!=null && imp.getID()==imageID2 
 		&& entireStack==entireStack2 && mean==mean2)
@@ -882,6 +947,7 @@ class ThresholdPlot extends Canvas implements Measurements, MouseListener {
 		ip.setRoi(roi);
 		if (stats==null)
 			stats = ImageStatistics.getStatistics(ip, AREA+MIN_MAX+MODE, null);
+		if (IJ.debugMode) IJ.log("  stats: "+stats);
 		int maxCount2 = 0;
 		histogram = stats.histogram;
 		originalModeCount = histogram[stats.mode];
