@@ -450,10 +450,7 @@ public class FloatProcessor extends ImageProcessor {
 							v2 = (float)Math.exp(c*Math.log(v1));
 						break;
 					case LOG:
-						if (v1<=0f)
-							v2 = 0f;
-						else
-							v2 = (float)Math.log(v1);
+						v2 = (float)Math.log(v1);
 						break;
 					case EXP:
 						v2 = (float)Math.exp(v1);
@@ -490,7 +487,12 @@ public class FloatProcessor extends ImageProcessor {
 		}
 	}
 
-	public void invert() {process(INVERT, 0.0);}
+	/** Each pixel in the image is inverted using p=max-(p-min), where 'min'
+		and 'max' are the display range limits set using setMinAndMax(). */
+	public void invert() {
+		process(INVERT, 0.0);
+	}
+	
 	public void add(int value) {process(ADD, value);}
 	public void add(double value) {process(ADD, value);}
 	public void set(double value) {process(SET, value);}
@@ -685,7 +687,6 @@ public class FloatProcessor extends ImageProcessor {
 					} else
 						pixels[index++] = 0;
 				}
-				showProgress((double)(y-roiY)/roiHeight);
 			}
 		}
 	}
@@ -868,14 +869,16 @@ public class FloatProcessor extends ImageProcessor {
 		double xScale = (double)dstWidth/roiWidth;
 		double yScale = (double)dstHeight/roiHeight;
 		if (interpolationMethod!=NONE) {
-			dstCenterX += xScale/2.0;
-			dstCenterY += yScale/2.0;
+			if (dstWidth!=width) dstCenterX+=xScale/4.0;
+			if (dstHeight!=height) dstCenterY+=yScale/4.0;
 		}
+		int inc = getProgressIncrement(dstWidth,dstHeight);
 		ImageProcessor ip2 = createProcessor(dstWidth, dstHeight);
 		float[] pixels2 = (float[])ip2.getPixels();
 		double xs, ys;
 		if (interpolationMethod==BICUBIC) {
 			for (int y=0; y<=dstHeight-1; y++) {
+				if (inc>0&&y%inc==0) showProgress((double)y/dstHeight);
 				ys = (y-dstCenterY)/yScale + srcCenterY;
 				int index = y*dstWidth;
 				for (int x=0; x<=dstWidth-1; x++) {
@@ -888,6 +891,7 @@ public class FloatProcessor extends ImageProcessor {
 			double ylimit = height-1.0, ylimit2 = height-1.001;
 			int index1, index2;
 			for (int y=0; y<=dstHeight-1; y++) {
+				if (inc>0&&y%inc==0) showProgress((double)y/dstHeight);
 				ys = (y-dstCenterY)/yScale + srcCenterY;
 				if (interpolationMethod==BILINEAR) {
 					if (ys<0.0) ys = 0.0;
@@ -906,15 +910,20 @@ public class FloatProcessor extends ImageProcessor {
 				}
 			}
 		}
+		if (inc>0) showProgress(1.0);
 		return ip2;
 	}
 	
-	FloatProcessor downsize(int dstWidth, int dstHeight) {
+	FloatProcessor downsize(int dstWidth, int dstHeight, String msg) {
 		FloatProcessor ip2 = this;
+		if (msg!=null)
+			ij.IJ.showStatus("downsizing in x"+msg);
 		if (dstWidth<roiWidth) {	  //downsizing in x
 			ip2 = ip2.downsize1D(dstWidth, roiHeight, true);
 			ip2.setRoi(0, 0, dstWidth, roiHeight);	//prepare roi for resizing in y
 		}
+		if (msg!=null)
+			ij.IJ.showStatus("downsizing in y"+msg);
 		if (dstHeight<roiHeight)	  //downsizing in y
 			ip2 = ip2.downsize1D(ip2.getRoi().width, dstHeight, false);
 		if (ip2.getWidth()!=dstWidth || ip2.getHeight()!=dstHeight)
