@@ -246,7 +246,7 @@ public abstract class ImageProcessor implements Cloneable {
     	for (int i=0; i<mapSize; i++) {
 			r2 = rLUT[i]&0xff; g2 = gLUT[i]&0xff; b2 = bLUT[i]&0xff;
     		distance = (r2-r1)*(r2-r1)+(g2-g1)*(g2-g1)+(b2-b1)*(b2-b1);
-			//ij.IJ.write(i+" "+minIndex+" "+distance+" "+(rLUT[i]&255));
+			//ij.IJ.log("  "+i+" "+minIndex+" "+distance+"   "+(rLUT[i]&255)+" "+(gLUT[i]&255)+" "+(bLUT[i]&255));
     		if (distance<minDistance) {
     			minDistance = distance;
     			minIndex = i;
@@ -1062,29 +1062,54 @@ public abstract class ImageProcessor implements Cloneable {
 
 	/** Draws a line from the current drawing location to (x2,y2). */
 	public void lineTo(int x2, int y2) {
+		int xMin = clipXMin-lineWidth/2-1;  //need not draw dots outside of this rect
+		int xMax = clipXMax+lineWidth/2+1;
+		int yMin = clipYMin-lineWidth/2-1;
+		int yMax = clipYMax+lineWidth/2+1;
 		int dx = x2-cx;
 		int dy = y2-cy;
 		int absdx = dx>=0?dx:-dx;
 		int absdy = dy>=0?dy:-dy;
 		int n = absdy>absdx?absdy:absdx;
-		double xinc = (double)dx/n;
-		double yinc = (double)dy/n;
+		double xinc = dx!=0 ? (double)dx/n : 0; //single point (dx=dy=n=0): avoid division by zero
+		double yinc = dy!=0 ? (double)dy/n : 0;
 		double x = cx;
 		double y = cy;
-		n++;
-		cx = x2; cy = y2;
-		if (n>1000000) return;
-		do {
+		cx = x2; cy = y2;       //keep end point as starting for the next lineTo
+		int i1 = 0;
+		if (dx>0)
+			i1 = Math.max(i1, (int)((xMin-x)/xinc));
+		else if (dx<0)
+			i1 = Math.max(i1, (int)((xMax-x)/xinc));
+		else if (x<xMin || x>xMax)
+			return; // vertical line outside y range
+		if (dy>0)
+			i1 = Math.max(i1, (int)((yMin-y)/yinc));
+		else if (dy<0)
+			i1 = Math.max(i1, (int)((yMax-y)/yinc));
+		else if (y<yMin || y>yMax)
+			return; // horizontal line outside y range
+		int i2 = n;
+		if (dx>0)
+			i2 = Math.min(i2, (int)((xMax-x)/xinc));
+		else if (dx<0)
+			i2 = Math.min(i2, (int)((xMin-x)/xinc));
+		if (dy>0)
+			i2 = Math.min(i2, (int)((yMax-y)/yinc));
+		else if (dy<0)
+			i2 = Math.min(i2, (int)((yMin-y)/yinc));
+		x += i1*xinc;
+		y += i1*yinc;
+		for (int i=i1; i<=i2; i++) {
 			if (lineWidth==1)
 				drawPixel((int)Math.round(x), (int)Math.round(y));
 			else if (lineWidth==2)
 				drawDot2((int)Math.round(x), (int)Math.round(y));
 			else
-				drawDot((int)x, (int)y);
+				drawDot((int)Math.round(x), (int)Math.round(y));
 			x += xinc;
 			y += yinc;
-		} while (--n>0);
-		//if (lineWidth>2) resetRoi();
+		}
 	}
 		
 	/** Draws a line from (x1,y1) to (x2,y2). */
@@ -1093,6 +1118,31 @@ public abstract class ImageProcessor implements Cloneable {
 		lineTo(x2, y2);
 	}
 
+	/* Draws a line using the Bresenham's algorithm that is 4-connected instead of 8-connected.<br>
+		Based on code from http://stackoverflow.com/questions/5186939/algorithm-for-drawing-a-4-connected-line<br>
+		Author: Gabriel Landini (G.Landini at bham.ac.uk)
+	*/
+	 public void drawLine4(int x1, int y1, int x2, int y2) {
+		int dx = Math.abs(x2 - x1);
+		int dy = Math.abs(y2 - y1);
+		int sgnX = x1 < x2 ? 1 : -1;
+		int sgnY = y1 < y2 ? 1 : -1;
+		int e = 0;
+		for (int i=0; i < dx+dy; i++) {
+			putPixel(x1, y1, fgColor);
+			int e1 = e + dy;
+			int e2 = e - dx;
+			if (Math.abs(e1) < Math.abs(e2)) {
+				x1 += sgnX;
+				e = e1;
+			} else {
+				y1 += sgnY;
+				e = e2;
+			}
+		}
+		putPixel(x2, y2, fgColor);
+	}
+
 	/** Draws a rectangle. */
 	public void drawRect(int x, int y, int width, int height) {
 		if (width<1 || height<1)
@@ -1219,7 +1269,7 @@ public abstract class ImageProcessor implements Cloneable {
 		int descent = metrics.getDescent();
 		g.setFont(font);
 
-		if (antialiasedText && cxx>=00 && cy-h>=0) {
+		if (antialiasedText && cxx>=0 && cy-h>=0) {
 			Java2.setAntialiasedText(g, true);
 			setRoi(cxx, cy-h, w, h);
 			ImageProcessor ip = crop();
@@ -1704,7 +1754,7 @@ public abstract class ImageProcessor implements Cloneable {
 	/** Uses the current interpolation method (bilinear or bicubic)
 		to find the pixel value at real coordinates (x,y). */
 	public abstract double getInterpolatedPixel(double x, double y);
-
+	
 	/** Uses the current interpolation method to find the pixel value at real coordinates (x,y).
 		For RGB images, the argb values are packed in an int. For float images,
 		the value must be converted using Float.intBitsToFloat().  Returns zero
@@ -1780,19 +1830,6 @@ public abstract class ImageProcessor implements Cloneable {
 		return z;
 	}	
 
-	/*
-		// a = 0.5
-	double cubic2(double x) {
-		if (x < 0) x = -x;
-		double z = 0;
-		if (x < 1)
-			z = 1.5*x*x*x + -2.5*x*x + 1.0;
-		else if (x < 2)
-			z = -0.5*x*x*x + 2.5*x*x - 4.0*x + 2.0;
-		return z;
-	}
-	*/	
-
 	private final double getInterpolatedEdgeValue(double x, double y) {
 		int xbase = (int)x;
 		int ybase = (int)y;
@@ -2009,13 +2046,20 @@ public abstract class ImageProcessor implements Cloneable {
 		else {  //downsizing with averaging in at least one direction: convert to float
 			ImageProcessor ip2 = createProcessor(dstWidth, dstHeight);
 			FloatProcessor fp = null;
-			for (int channelNumber=0; channelNumber<getNChannels(); channelNumber++) {
+			int channels = getNChannels();
+			boolean showStatus = getProgressIncrement(width,height)>0;
+			boolean showProgress = showStatus && channels>1;
+			if (showProgress) showProgress(0.15);
+			for (int channelNumber=0; channelNumber<channels; channelNumber++) {
 				fp = toFloat(channelNumber, fp);
 				fp.setInterpolationMethod(interpolationMethod);
 				fp.setRoi(getRoi());
-				FloatProcessor fp2 = fp.downsize(dstWidth, dstHeight);
+				String msg = showStatus?" ("+(channelNumber+1)+"/"+channels+")":null;
+				FloatProcessor fp2 = fp.downsize(dstWidth, dstHeight, msg);
 				ip2.setPixels(channelNumber, fp2);
+			    if (showProgress) showProgress(0.40+0.25*channelNumber);
 			}
+			if (showProgress) showProgress(1.0);
 			return ip2;
 		}
 	}
@@ -2244,7 +2288,7 @@ public abstract class ImageProcessor implements Cloneable {
 		threshold(getAutoThreshold());
 	}
 
-	/**	Returns a pixel value (threshold) that can be used to divide the image into objects 
+	/** Returns a pixel value (threshold) that can be used to divide the image into objects 
 		and background. It does this by taking a test threshold and computing the average 
 		of the pixels at or below the threshold and pixels above. It then computes the average
 		of those two, increments the threshold, and repeats the process. Incrementing stops 
@@ -2486,8 +2530,12 @@ public abstract class ImageProcessor implements Cloneable {
 		useBicubic = b;
 	}
 	
-	/* Calculates and returns statistics (area, mean, std-dev, mode, min, max,
-		centroid, center of mass, 256 bin histogram) for this image or ROI. */
+	/** Calculates and returns statistics (area, mean, std-dev, mode, min, max,
+	 * centroid, center of mass, 256 bin histogram) for this image or ROI. Use the
+	 * setRoi(Roi) method to limit statistics to a non-rectangular area.
+	 * @see ImageProcessor#setRoi	
+	 * @see ImageStatistics	
+	*/
 	public ImageStatistics getStatistics() {
 		// 127 = AREA+MEAN+STD_DEV+MODE+MIN_MAX+CENTROID+CENTER_OF_MASS
 		return ImageStatistics.getStatistics(this, 127, null);
@@ -2495,11 +2543,10 @@ public abstract class ImageProcessor implements Cloneable {
 	
 	/** Blurs the image by convolving with a Gaussian function. */
 	public void blurGaussian(double sigma) {
-		double accuracy = getBitDepth()==8||getBitDepth()==24?0.002:0.0002;
 		resetRoi();
 		GaussianBlur gb = new GaussianBlur();
 		gb.showProgress(false);
-        gb.blurGaussian(this, sigma, sigma, accuracy);
+        gb.blurGaussian(this, sigma);
 	}
 
 	/** Uses the Process/Math/Macro command to apply macro code to this image. */
@@ -2541,5 +2588,20 @@ public abstract class ImageProcessor implements Cloneable {
 	public Overlay getOverlay() {
 		return overlay;
 	}
-
+	
+	protected int getProgressIncrement(int w, int h) {
+		if (progressBar==null)
+			return 0;
+		int inc = 0;
+		int threshold = 15000000;
+		if (interpolationMethod==BICUBIC)
+			threshold = 5000000;
+		boolean isBig = w*h>threshold;
+		if (isBig) {
+			inc = h/30;
+			if (inc<1) inc=1;
+		}
+		return inc;
+	}
+	
 }
